// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.gui.StatusLightPanel;

public class InformationDelegator extends com.bbn.openmap.InformationDelegator{
	private static int dismissDelay = 10000;//10 seconds by default
    private String fudgeString = " ";
    
	@Override
	public void initInfoWidgets() {
        Debug.message("info", "InformationDelegator.initInfoWidgets");

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

        setFont(new Font("Helvetica", Font.PLAIN, 9));
        setLayout(gridbag);

        progressBar = new JProgressBar();
        gridbag.setConstraints(progressBar, c);
        add(progressBar);
        progressBar.setVisible(false);

        JPanel infoLinePanel = new JPanel();
        c.weightx = 1;
        c.anchor = GridBagConstraints.WEST;
        c.fill = GridBagConstraints.HORIZONTAL;

        gridbag.setConstraints(infoLinePanel, c);

        GridBagLayout gridbag2 = new GridBagLayout();
        GridBagConstraints c2 = new GridBagConstraints();
        infoLinePanel.setLayout(gridbag2);

        infoLineHolder = new JLabel(fudgeString);
        c2.weightx = .9;
        c2.fill = GridBagConstraints.HORIZONTAL;
        c2.anchor = GridBagConstraints.WEST;
        c2.insets = new Insets(3, 10, 3, 10);
        gridbag2.setConstraints(infoLineHolder, c2);
        infoLinePanel.add(infoLineHolder);

        infoLineHolder2 = new JLabel(fudgeString, SwingConstants.RIGHT);
        c2.weightx = .1;
        c2.anchor = GridBagConstraints.EAST;
        gridbag2.setConstraints(infoLineHolder2, c2);
        infoLinePanel.add(infoLineHolder2);

        addInfoLine(COORDINATE_INFO_LINE, infoLineHolder);
        addInfoLine(MAP_OBJECT_INFO_LINE, infoLineHolder2);

        add(infoLinePanel);

        c.weightx = 0;
        c.anchor = GridBagConstraints.EAST;
        statusBar = new StatusLightPanel();
        statusBar.setBorder(BorderFactory.createEmptyBorder(0,0,0,20));
        gridbag.setConstraints(statusBar, c);
        add(statusBar);
        statusBar.setVisible(showLights);
        
    }
	
	@Override
	public void initToolTip(){

		if (ttmanager == null) {
            // make sure the MapBean is registered first
            ttmanager = ToolTipManager.sharedInstance();
            ttmanager.setDismissDelay(dismissDelay);
            ttmanager.registerComponent(map);
            ttmanager.setEnabled(true);
            return;
        }

        // If it already exists, clear out the current tip
        if (map != null) {
            map.setToolTipText(null);
        }	
	}
	/**
	 * add the add/remove Map scrollbar button
	 */
	public void addScrollButton(JToggleButton scroll){
        add(scroll);
        updateUI();
	}
	
	public void addExternMapButton(JToggleButton button){
        add(button);
        updateUI();		
	}
	
	public void setTooltipManagerDismissDelay(int delay){
		if(ttmanager!=null)
			ttmanager.setDismissDelay(delay);
	}
	
	public static String tooltipDimissDelay = "tooltipDismissDelay";
	
	public void setProperties(String prefix, Properties props) {
		setPropertyPrefix(prefix);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        statusBar.setProperties(prefix, props);
        setShowLights(PropUtils.booleanFromProperties(props, prefix
                + ShowLightsProperty, showLights));
        setShowInfoLine(PropUtils.booleanFromProperties(props, prefix
                + ShowInfoLineProperty, getShowInfoLine()));
        setShowCoordsInfoLine(PropUtils.booleanFromProperties(props, prefix
                + ShowCoordsInfoLineProperty, getShowCoordsInfoLine()));
        String pl = props.getProperty(prefix + PreferredLocationProperty);
        if (pl != null) {
            setPreferredLocation(pl);
        }
		String delay = props.getProperty(prefix + tooltipDimissDelay, String.valueOf(dismissDelay));
		dismissDelay = Integer.parseInt(delay);
	}
	
	public Properties getProperties(Properties props) {
		Properties properties = super.getProperties(props);
		String prefix = PropUtils.getScopedPropertyPrefix(this);
		properties.put(prefix+tooltipDimissDelay,String.valueOf(dismissDelay));
		return properties;	
	}
}
