// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.animation;

import com.bbn.openmap.Layer;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.animation.gui.AnimationDSChainPanel;
//import fr.unicaen.openmap.animation.gui.AnimationDSChainToolGUI;
import fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage;
import fr.unicaen.openmap.dataStorage.AnimationDataStorageListener;
import fr.unicaen.openmap.dataStorage.DataStorageIteratorAdapter;
import fr.unicaen.openmap.dataStorage.SortDataStorage;
import fr.unicaen.openmap.dataStorage.gui.AdaptorButtonArray;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.ValueComparator;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.Timer;

/**
 *
 * @author jchauvea
 */
public class AnimationDSChain implements AnimationDataStorageListener, AnimationDSChainListener {

    private ArrayList<MonoValuedFilterDataStorage> animationDatastorages;
    private AnimationDSChainPanel gui;
    private JComponent toolGUI;
    private List<String> values;
    private MetaAnimator metaAnimator;
    private Timer timer;
    private AbstractAction nextValueAction;
    //private int delay = 0;
    private String currentValue;
    private int currentIndex = 0;
    private int nbStep;
    private String name;
    private AnimationDSChain slave;
    private AnimationDSChain master;
    private boolean active; //when slave, true if the loop is not finished
    private HashMap<String, String> animationsToAdd = new HashMap<String, String>();
    //private String value;
    public static String listAnimationsPropertyString = "listAnimations";
    public static String layersList = "layersList";
    public static String delayPropertyString = "delay";
    //public static String valuePropertyString = "value";

    public AnimationDSChain(MetaAnimator ma, String name) {
        this.metaAnimator = ma;
        this.name = name;
        animationDatastorages = new ArrayList<MonoValuedFilterDataStorage>();
        values = new ArrayList<String>();
        //timer
        nextValueAction = new AbstractAction() {

            public void actionPerformed(ActionEvent e) {
                if (slave != null && slave.isActive())
                    return;
                if(getValues().isEmpty())
                    return;
                if(currentIndex > getValues().size())
                    currentIndex = 0;
                String val = getValues().get(currentIndex);
                if (gui != null)
                    gui.updateSelectedValue(val);
                nextStep(val);
                currentIndex++;
                if (currentIndex == nbStep) {
                    if (master != null) {
                        active = false;
                        stop();
                    }
                    currentIndex = 0;
                }
            }
        };
        timer = new Timer(0, getNextValueAction());
    }

    public void addAnimationDataStorage(MonoValuedFilterDataStorage ads) {
        animationDatastorages.add(ads);
        metaAnimator.getAvailableDSList().remove(ads);
        metaAnimator.getUsedDSList().add(ads);
        metaAnimator.fireAnimationDSSelected(this, ads);
        if (gui != null)
            gui.refresh(true);
    }

    public void removeAnimationDataStorage(MonoValuedFilterDataStorage ads) {
        animationDatastorages.remove(ads);
        metaAnimator.getAvailableDSList().add(ads);
        metaAnimator.getUsedDSList().remove(ads);
        metaAnimator.fireAnimationDSUnselected(this, ads);
        if (gui != null)
            gui.refresh(true);

    }

    /**
     * @return the animationDatastorages
     */
    public ArrayList<MonoValuedFilterDataStorage> getAnimationDatastorages() {
        return animationDatastorages;
    }

    public JComponent getGUI() {
        if (gui == null) {
            gui = new AnimationDSChainPanel(this);
        }
        return gui;
    }

    public JComponent getToolGUI() {
        if (toolGUI == null && gui != null)
            toolGUI = gui.getToolGUI();
        return toolGUI;
    }

    public List<String> computeUnionValues() {
        getValues().clear();
        for (MonoValuedFilterDataStorage ads : animationDatastorages) {
            if (ads.getFilterColumnName() != null && !ads.getFilterColumnName().equals(""))
                for (String val : ads.keyValues) {
                    if (!values.contains(val) && !val.equals(ads.noFilter))
                        getValues().add(val);
                }
        }
        nbStep = getValues().size();
        //System.out.println("VALUES SIZE " + values.size());
        return getValues();
    }

    public void sortValues (SortDataStorage.Format format, boolean asc) throws ParseException{
        ValueComparator.sortValues(getValues(), format);
        if (!asc)
            Collections.reverse(getValues());
        gui.refresh(false);
        //toolGUI.refresh();
    }

    public void animationRemoved(MonoValuedFilterDataStorage ads) {
        animationDatastorages.remove(ads);
        if (gui != null) {
            gui.refresh(true);
            gui.removeAnimationFromGUI(ads);
        }
        if (animationDatastorages.isEmpty() && metaAnimator.getNbChain() > 1) {
            metaAnimator.removeChain(this);
        }
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void animationAdded(MonoValuedFilterDataStorage ads) {
        //animationDatastorages.add(ads);
        //metaAnimator.getAvailableDSList().remove(ads);
        if (gui != null)
            gui.addAnimationGUI(ads);
    }

    public void animationChanged(MonoValuedFilterDataStorage ads) {
        //throw new UnsupportedOperationException("Not supported yet.");
        if (gui != null)
            gui.refresh(true);
//        if(toolGUI != null)
//            toolGUI.refresh();

    }

    /*public void animationModeChanged(AnimationDataStorage ads){
    if(animationDatastorages.contains(ads))
    gui.setSelectedMode(ads.getSelectedMode());
    }*/
    /**
     * @return the metaAnimator
     */
    public MetaAnimator getMetaAnimator() {
        return metaAnimator;
    }

    /**
     * @return the mode
     */
//    public AnimationDataStorage.Mode getMode() {
//        return mode;
//    }
    /**
     * @param mode the mode to set
     */
//    public void setMode(AnimationDataStorage.Mode mode) {
//        this.mode = mode;
//        for (AnimationDataStorage ads : animationDatastorages)
//            ads.setSelectedMode(mode);
//    }
    /**
     * @return the delay
     */
    public int getDelay() {
        return timer.getDelay() / 1000;
    }

    /**
     * @param delay the delay to set
     */
    public void setDelay(int delay) {
        //this.delay = delay;
        timer.setDelay(delay * 1000);

    }

    public void start() {
        //currentIndex = 0;
        if (master != null)
            active = true;
        timer.start();
    }

    public void nextStep(String value) {
        //System.out.println("NEXT STEP------------>" + value);
        currentValue = value;
        currentIndex = values.indexOf(value);
        for (MonoValuedFilterDataStorage ads : animationDatastorages)
            ads.nextStep(currentValue);
        if (getSlave() != null && timer.isRunning())
            getSlave().start();
    }

    public boolean nextStep(){
        getNextValueAction().actionPerformed(null);
        if(currentIndex == 0) // loop(export) is finished
            return false;
        return true;
    }

    public void stop() {
        timer.stop();
        if (slave != null)
            slave.stop();
    }

    public String getCurrentValue() {
        return currentValue;
    }

    /**
     * @return the values
     */
    public List<String> getValues() {
        return values;
    }

    /**
     * @return the nextValueAction
     */
    public AbstractAction getNextValueAction() {
        return nextValueAction;
    }

    public void delete() {
        for (int i = 0; i < animationDatastorages.size(); i++)
            removeAnimationDataStorage(animationDatastorages.get(i));
        metaAnimator.removeChain(this);
    }

    public String getName() {
        return this.name;
    }

    public MonoValuedFilterDataStorage addAnimationToLayer(Layer l) {
        try {
            ThematicDataDisplayLayer layer = null;
            try {
                layer = (ThematicDataDisplayLayer) l;
                MonoValuedFilterDataStorage ds = (MonoValuedFilterDataStorage) AdaptorButtonArray.getDataStorageConstructor(MonoValuedFilterDataStorage.class.getName()).newInstance(
                        new Object[]{l, layer.getDataStorageIterator().getDataStorage()});
                layer.setDataStorageIterator(new DataStorageIteratorAdapter(ds));
                layer.dataStorageEvent(ds);
                return ds;
            } catch (ClassCastException e) {
                JOptionPane.showMessageDialog(gui, "Impossible to add an animation to " + l.getName(), "Bad layer", JOptionPane.ERROR_MESSAGE);
            }
        } catch (InstantiationException ex) {
            Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvocationTargetException ex) {
            Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;

    }

    public int getSize() {
        return animationDatastorages.size();
    }

    public void animationDSSelected(MonoValuedFilterDataStorage ads) {
        if (gui != null)
            gui.removeAnimationFromGUI(ads);
    }

    public void animationDSUnselected(MonoValuedFilterDataStorage ads) {
        if (gui != null)
            gui.addAnimationGUI(ads);
    }

    /**
     * @return the slave
     */
    public AnimationDSChain getSlave() {
        return slave;
    }

    /**
     * @param slave the slave to set
     */
    public void setSlave(AnimationDSChain slave) {
        this.slave = slave;
    }

    /**
     * @return the master
     */
    public AnimationDSChain getMaster() {
        return master;
    }

    /**
     * @param master the master to set
     */
    public void setMaster(AnimationDSChain master) {
        this.master = master;
    }

    public boolean isActive() {
        return active && timer.isRunning();
    }
    public static String chainPrefix = "chain";

    public void setProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix + chainPrefix + "." + metaAnimator.getChainRank(this));
        timer.setDelay(Integer.parseInt(props.getProperty(prefix + delayPropertyString, "0")));
        String layersString = props.getProperty(prefix + layersList, "");
        animationsToAdd.clear();
        if (layersString == null)
            return;
        String[] layers = layersString.split(" ");
        for (String l : layers) {
            if (l.equals(""))
                continue;
            String ranks = props.getProperty(prefix + l, "");
            animationsToAdd.put(l, props.getProperty(prefix + l, ranks));
            //for(String r : ranks){
            //animationsToAdd.put(l, animationsToAdd.get(l)+ " " + r);
                /*AnimationDataStorage anim = metaAnimator.findAnimationDataStorage(l, Integer.parseInt(r));
            if(anim != null)
            addAnimationDataStorage(anim);*/
            //}
        }

    }

    public void addAnimationsToAdd() {
        if (animationsToAdd.isEmpty())
            return;
        for (String l : animationsToAdd.keySet()) {
            String[] ranks = animationsToAdd.get(l).split(" ");
            for (String r : ranks) {
                try {
                    MonoValuedFilterDataStorage anim = metaAnimator.findAnimationDataStorage(l, Integer.parseInt(r));
                    if (anim != null) {
                        addAnimationDataStorage(anim);
                    }
                } catch (NumberFormatException ex) {
                    //continue;
                }
            }
        }
    }

    public Properties getProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix + chainPrefix + "." + metaAnimator.getChainRank(this));
        HashMap<String, String> animationPerLayers = new HashMap<String, String>();
        props.put(prefix + delayPropertyString, String.valueOf(timer.getDelay()));
        for (MonoValuedFilterDataStorage ads : animationDatastorages) {
            ThematicDataDisplayLayer layer = ads.getLayer();
            String index = String.valueOf(
                    layer.getDataStorageIterator().getDataStorage().getAnimationDataStorageList().indexOf(ads));
            if (!animationPerLayers.containsKey(layer.getName())) {
                animationPerLayers.put(layer.getName(), index);
            } else {
                animationPerLayers.put(layer.getName(), animationPerLayers.get(layer.getName() + " " + index));
            }
        }
        String animatedLayers = "";
        for (String key : animationPerLayers.keySet()) {
            String ranks = animationPerLayers.get(key);
            animatedLayers += " " + key;
            props.put(prefix + key, ranks);

        }
        props.put(prefix + layersList, animatedLayers);

        return props;
    }
}
