// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import java.text.NumberFormat;
import java.util.Hashtable;
import java.util.Map;
import java.util.Enumeration;

/**
 * DataObject store a row of data and OMDataLocation as 2 hash tables
 * <p><em>Note</em>: We should at least replace Hashtables with HashMaps.
 * The best would be to use simple ArrayList, columnnames beeing stored in the model.
 */
public class DataObject extends Object implements Comparable<DataObject>{

	public static String sortParameter;
	
    private Hashtable<String, Object> datas = null;
    private Hashtable<String,OMDataLocation> omdls = null;
    
    protected Hashtable<String, Object> getDatas() { return datas; }
    protected Hashtable<String,OMDataLocation> getOmdls() { return omdls; }
    
    /**
     * Constructor.
     */
    public DataObject() {
	this.datas = new Hashtable<String, Object>();
	this.omdls = new Hashtable<String,OMDataLocation>();
    }

    public DataObject(Hashtable<String, Object> datas, Hashtable<String,OMDataLocation> omdls) {
	this.datas = datas;
	this.omdls = omdls;
    }
    
    /**
     * index is the name of datas (column) stored.
     * @return Object : the value stored in this column.
     */
    public Object getData(String index) {
	return datas.get(index);
    }
    
    /**
     * Add data.
     * @return boolean : true if it succeeds, false otherwise.
     */
    public boolean addData(String index, Object data) {
	try {
	    datas.put(index, data);
	    return true;
	} catch (Exception e) {
	    return false;
	}	
    }
    
    /**
     *
     */
    public OMDataLocation getOMDataLocation (String index) {
	return omdls.get(index);
    }
    
    /**
     * Add an OMDataLocation to the DataObject.
     * @return boolean : true if it succeeds, false otherwise.
     */
    public boolean addOMDataLocation (String index, OMDataLocation omdl) {
	try {
	    omdls.put(index, omdl);
	    return true;
	} catch (Exception e) {
	    return false;
	}
    }

    /**
     * Remove OMDataLocation
     */
    public void removeOMDataLocation(String index){
        OMDataLocation omdl = omdls.remove(index);
//        if(omdl == null)
//            System.out.println("IL EST NULL");
//        else
//        {
//            System.out.println("IL EST PAS NULL " + omdl.hashCode()+ " du DATAOBJECT " + hashCode());
//        }
    }

    /**
     * This method merges one DataObject into another.
     * We're using putAll method from Hashtable class.
     */
    public void putAll(DataObject obj) {
	datas.putAll(obj.getDatas());
	omdls.putAll(obj.getOmdls());
    }

    public String toString() {
	return "DataObject <" + datas + omdls +">";
    }
    
    public String toHTML() {
	StringBuffer ret = new StringBuffer("<html><body><table>");
	for (Enumeration<String> keys = getDatas().keys(); keys.hasMoreElements();) {
	    String key = keys.nextElement();
	    String s;
	    Object o; 
	    if((o = getDatas().get(key)) instanceof Number){
	    	NumberFormat df = NumberFormat.getNumberInstance();
	    	s = df.format((Number)getDatas().get(key));
	    }
	    else
	    	s = o.toString();
	    ret.append("<tr><td><b>" + key + "</b></td><td> " + s + " </td></tr>");
	    // ret.append("<br><b>" + key + ":</b> " + getDatas().get(key).toString());
	}
	ret.append("</table></body></html>");
	return ret.toString();
    }

    public int hashCode() {
	int h =  datas.hashCode() +  omdls.hashCode();
	return h;
    }

    public boolean equals(Object o) {
	DataObject odo = (DataObject) o;
	if (datas.size() == 0)
	    return odo.datas.size() == 0;
	if (odo.datas.size() == 0)
	    return false;
	for(Map.Entry<String, Object> e : datas.entrySet()) {
	    Object oo = odo.datas.get(e.getKey());
	    if (e.getValue() == null)
		if (oo == null)
		    continue;
		else
		    return false;
	    if (! e.getValue().equals(oo))
		return false;
	}
	return true;
    }
    
	public int compareTo(DataObject dObj) {
		if(sortParameter == null)
			return 0;
		else{
			Object data0 = getData(sortParameter);
			Object data1 = dObj.getData(sortParameter);
			
			if(data0 instanceof Double){
				return ((Double)data0).compareTo((Double)data1);
			}
			else{
				return (data0.toString()).compareTo(data1.toString());
			}
		}
	}
}
