// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;

import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

/**
 * Standard implementation of interface DataStorageInterface.
 * This class should be sufficient for most uses of DataStorage.
 * You should not have to modify this class.
 */
public class DataStorageIteratorAdapter implements DataStorageIterator {

    /**
     * Current index for the dataStorage.
     */
    protected int current = 0;
    /**
     * The DataStorage, whatever it is.
     */
    protected DataStorage dataStorage;

    /**
     * Constructor.
     */
    public DataStorageIteratorAdapter(DataStorage dataStorage) {
        this.dataStorage = dataStorage;
        reset();
    }

    /**
     * Can we call next(); ?
     */
    public boolean hasNext() {
        return current < dataStorage.getCount();
    }

    /**
     * Get the next DataObject. No check is done here !
     */
    public DataObject next() {
        return dataStorage.getElementAt(current++);
    }

    // private boolean isFirstReset = true;
    /**
     * Go to the first element of the list.
     */
    public void reset() {
        current = 0;
        // fausse bonne idée ???
        // if (isFirstReset)
        //     isFirstReset = false;
        // else
        //     dataStorage.setWarning(false);
    }

    /**
     * Tell if we can iterate or not the DataStorage. Calls <code>dataStorage.isReady();</code>
     */
    public boolean isReady() {
        return dataStorage != null && dataStorage.isReady();
    }

    /**
     * Get the DataStorage GUI.
     */
    public JComponent getGUI() {
        return dataStorage.getGUI();
    }

    /**
     * Get the DataTable GUI.
     */
    public JComponent getDataGUI() {

        // JPanel pan = new JPanel();
        // pan.setLayout(new BorderLayout());
        // pan.add(dataStorage.getDataGUI(),BorderLayout.CENTER);
        // JPanel panTop = new JPanel();
        // JButton exportGML = new JButton(Util.loadIcon("gmlExport.png"));
        // exportGML.setToolTipText(i18n.get(DataStorageIterator.class, "gmlDS","GML export"));
        // panTop.add(exportGML);
        // pan.add(panTop, BorderLayout.NORTH);
        // exportGML.addActionListener(new ActionListener() {
        // 	    public void actionPerformed(ActionEvent event) {
        // 	    	GMLSaveFileChooser gsave =
        // 	    		new GMLSaveFileChooser();
        // 	    	if (gsave.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
        // 		    GMLWriter writer = new GMLWriter(gsave.getSelectedFile().getAbsolutePath());
        // 		    writer.write(dataStorage.saveAsGML());
        // 	    	}
        // 	    }});
        // return pan;

        return dataStorage.getDataGUI();
    }

    public Properties getProperties(String prefix, Properties properties) {
        return dataStorage.getProperties(prefix, properties);
    }

    /**
     * We must change this ! Hack ... 
     */
    public DataStorage getDataStorage() {
        return dataStorage;
    }

    public int getCurrentIndex() {
        return current;
    }

    /**
     * returns the DataTableModel to access names (and types) of columns
     */
    //   public DataTableModel getDataTableModel() { return dataStorage.getDefaultDataTableModel(); }
    /**
     * get info about the model ...
     */
    public String[] getDataColumnNames() {
        /*if(dataStorage == null || dataStorage.getDefaultDataTableModel() == null)
        return new String[0];*/
        /*while(dataStorage == null || !dataStorage.isReady()){
        try {
        Thread.sleep(1000);
        System.out.println("DATASTORAGE NOT READY ; DS = " + dataStorage);
        //t.wait(1000);
        } catch (InterruptedException ex) {
        Logger.getLogger(DataStorageIteratorAdapter.class.getName()).log(Level.SEVERE, null, ex);
        }
        }*/
        //System.out.println();
        //Debug.message("DataStorage", "DS class = "+dataStorage.getClass()+" ; tableModel= "+ dataStorage.getDefaultDataTableModel()+"");
        return dataStorage.getDefaultDataTableModel().getDataColumnNames();
    }

    /**
     * get info about the model ...
     */
    public String[] getOMColumnNames() {
        //String [] res = dataStorage.getDefaultDataTableModel().getOMColumnNames();
        //String [] res2 = dataStorage.getDefaultDataTableModel().getDataColumnNames();
        //if(Debug.debugging("DataStorageIteratorAdapter")){
        //	System.out.println("[DATASTORAGEITERATORADAPTER] "+ dataStorage.getClass());

        //}
        return dataStorage.getDefaultDataTableModel().getOMColumnNames();
    }

    /**
     * returns the real data column name given its index
     */
    public String getRealColumnName(int index) {
        return dataStorage.getDefaultDataTableModel().getRealColumnName(index);
    }

    /**
     * returns the real OM column name given its index
     */
    public String getRealOMColumnName(int index) {
        return dataStorage.getDefaultDataTableModel().getRealOMColumnName(index);
    }

    public int getIndexForColumnName(String name) {
        return dataStorage.getDefaultDataTableModel().getIndexForColumnName(name);
    }

    public int getIndexForOMColumnName(String name) {
        return dataStorage.getDefaultDataTableModel().getIndexForOMColumnName(name);
    }

    public BoundingBox getBoundingBox() {
        try {
            BoundingBox boundingBox = null;
            DataObject dObj;
            reset();
            while (hasNext()) {
                dObj = next();
                Hashtable<String, OMDataLocation> omdls = dObj.getOmdls();
                for (String key : omdls.keySet()) {
                    OMDataLocation omdl = omdls.get(key);
                    BoundingBox bb = omdl.getBoundingBox();
		    
		    double upperLeftLon, upperLeftLat, lowerRightLon, lowerRightLat;
		    if (bb != null) {
			upperLeftLon = bb.getUpperLeftLon();
			upperLeftLat = bb.getUpperLeftLat();
			lowerRightLon = bb.getLowerRightLon();
			lowerRightLat = bb.getLowerRightLat();
		    } else {
			Centroid c = omdl.getCentroid();
			if (c == null) {
			    continue;
			} 
			upperLeftLon = c.getLon();
			upperLeftLat = c.getLat();
			lowerRightLon = c.getLon();
			lowerRightLat = c.getLat();
		    }
		    if (boundingBox == null) {
			boundingBox = new BoundingBox(upperLeftLon, upperLeftLat, 
						      lowerRightLon,	lowerRightLat);
                    } else {
			boundingBox.extendTo(upperLeftLon, upperLeftLat);
			boundingBox.extendTo(lowerRightLon, lowerRightLat);
		    }
                }
            }
            reset();
	    Debug.message("DataStorage", "DataStorageIteratorAdapter.getBoundingBox() -> "+ boundingBox);
            return boundingBox;
        } catch (NullPointerException e) {
	    e.printStackTrace();
            return null;
        }
    }

    /**
     * return cols with no doublons
     * @return
     */
//    public String[] getKeyableDataColumnNames() {
//        String[] cols = getDataColumnNames();
//        ArrayList<String> resCols = new ArrayList<String>();
//        ArrayList<String> list;
//        for (String col : cols) {
//            reset();
//            list = new ArrayList<String>();
//            boolean uniq = true;
//            while (hasNext()) {
//                DataObject data = next();
//                if(data.getData(col) == null || list.contains(data.getData(col).toString())){
//                    uniq = false;
//                    break;
//                }
//                else
//                    list.add(data.getData(col).toString());
//            }
//            if(uniq){
//                resCols.add(col);
//            }
//        }
//        return resCols.toArray(new String[resCols.size()]);
//    }
}
