// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.omGraphics.EditableOMCircle;
import com.bbn.openmap.omGraphics.EditableOMGraphic;
import com.bbn.openmap.omGraphics.EditableOMGraphicList;
import com.bbn.openmap.omGraphics.EditableOMLine;
import com.bbn.openmap.omGraphics.EditableOMPoint;
import com.bbn.openmap.omGraphics.EditableOMPoly;
import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.tools.drawing.DrawingToolRequestor;
import com.bbn.openmap.tools.drawing.OMDrawingTool;
import com.bbn.openmap.tools.drawing.OMLineLoader;
import com.bbn.openmap.tools.drawing.OMPointLoader;
import com.bbn.openmap.tools.drawing.OMPolyLoader;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.EditableDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.EditableDataTableStoragePanel;
import fr.unicaen.openmap.tools.drawing.TMDrawingToolLauncher;
import fr.unicaen.openmap.util.Util;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 *
 * @author jchauvea
 */
public class EditableDataStorage extends InformationDataStorage implements RemovableDataStorage, DataStorageListener {

    private DataStorage _dataStorage;
    private OMDrawingTool drawingTool;
    private TMDrawingToolLauncher drawingLauncher;
    private DrawingToolRequestor requestor;
    private Box gui;
    protected ArrayList<DataObject> dataObjects;
    private String dataColumn;
    private String omColumn;

    public EditableDataStorage(DataStorageListener dsl) {
        super(dsl);
        dataTableModel = new EditableDataTableModel(dataTable);
        dtsp = new EditableDataTableStoragePanel(this, dataTableModel);
        drawingTool = new OMDrawingTool();
        OMLineLoader lineLoader = new OMLineLoader();
        OMPolyLoader polyLoader = new OMPolyLoader();
        OMPointLoader pointLoader = new OMPointLoader();
        drawingTool.addLoader(lineLoader);
        drawingTool.addLoader(polyLoader);
        drawingTool.addLoader(pointLoader);
        drawingLauncher = new TMDrawingToolLauncher();
        drawingLauncher.setDrawingTool(drawingTool);
    }

    public EditableDataStorage(DataStorageListener dsl, DataStorage ds) {
        this(dsl);
        this._dataStorage = ds;
        initialize();
    }

    public void highlightInDataTable(int index) {
        ((EditableDataTableStoragePanel) dtsp).highlightInDataTable(index);

    }

    @Override
    public boolean isReady() {
        return true;
    }

    public DataStorage getDataStorage() {
        return _dataStorage;
    }

    public void setDataStorage(DataStorage dataStorage) {
        _dataStorage = dataStorage;
    }

    public JComponent getGUI() {
        if (gui == null) {
            gui = Box.createVerticalBox();
            gui.add(drawingLauncher);
            Box panCombos = Box.createVerticalBox();
            JPanel wrapComboDataCols = new JPanel();
            JPanel wrapComboOMCols = new JPanel();
            final JComboBox comboDataCols = new JComboBox(_dataStorage.getDefaultDataTableModel().getDataColumnNames());
            final JComboBox comboOMCols = new JComboBox(_dataStorage.getDefaultDataTableModel().getOMColumnNames());
            comboDataCols.addItemListener(new ItemListener() {

                public void itemStateChanged(ItemEvent arg0) {
                    dataColumn = comboDataCols.getSelectedItem().toString();
                }
            });
            comboOMCols.addItemListener(new ItemListener() {

                public void itemStateChanged(ItemEvent arg0) {
                    omColumn = comboOMCols.getSelectedItem().toString();
                }
            });

            if(omColumn == null)
                omColumn = comboOMCols.getSelectedItem().toString();
            else
                comboOMCols.setSelectedItem(omColumn);
            if(dataColumn == null)
                dataColumn = comboDataCols.getSelectedItem().toString();
            else
                comboDataCols.setSelectedItem(dataColumn);
            
            panCombos.add(wrapComboDataCols);
            panCombos.add(wrapComboOMCols);
            wrapComboDataCols.add(new JLabel(i18n.get(this, "indexCol", "Index column")));
            wrapComboDataCols.add(comboDataCols);
            wrapComboOMCols.add(new JLabel(i18n.get(this,"graphicCol","Graphic column")));
            wrapComboOMCols.add(comboOMCols);
            gui.add(panCombos);
            gui.add(getDataStorage().getGUI());
            gui.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"title","Edition/Selection/Drawing tool")));
        }
        return gui;
    }

    public void setRequestor(DrawingToolRequestor requestor) {
        this.requestor = requestor;
        drawingLauncher.setCurrentRequestor(requestor);
        //drawingLauncher.resetGUI();
        //drawingLauncher.updateUI();
    }

    public OMDrawingTool getDrawingTool() {
        return drawingTool;
    }

    public void addOMGraphic(OMGraphic omg) {
        DataObject dObj = new DataObject();
        OMDataLocation omdl = new OMDataLocation(omg);
        dObj.addData(dataColumn, getCount() + 1);
        dObj.addOMDataLocation(omColumn, omdl);
        dataObjects.add(dObj);
        setTable();
    }

    public void initialize() {
        dataObjects = new ArrayList<DataObject>(_dataStorage.getCount());
        for (int i = 0; i < _dataStorage.getCount(); i++) {
            dataObjects.add(_dataStorage.getElementAt(i));
        }
        setTable();
    }

    protected boolean setTable() {

        if (dataObjects == null) {
            return false;
        }

        dataTable.setTable(dataObjects,
                new ArrayList<String>(Arrays.asList(_dataStorage.getDefaultDataTableModel().getDataColumnNames())),
                new ArrayList<String>(Arrays.asList(_dataStorage.getDefaultDataTableModel().getOMColumnNames())),
                _dataStorage.getDataAttributesNamesAliases());
        ((DataTableStoragePanel) dtsp).fireStructureChanged();
        dsl.dataStorageEvent(this);
        return true;
    }

    public DataStorage removeItSelf() {
        _dataStorage.dsl = dsl;
        dsl.dataStorageEvent(_dataStorage);
        return _dataStorage;
    }

    public boolean removeOMGraphicAt(int index, OMGraphic omg) {
        DataObject dObj = getElementAt(index);
        for (String s : getDefaultDataTableModel().getOMColumnNames()) {
            OMGraphic tmpOmg = dObj.getOMDataLocation(s).getOMGraphic();
            if (tmpOmg == null) continue;
            if (tmpOmg instanceof OMGraphicList) {
                OMGraphicList l = (OMGraphicList) tmpOmg;
                if (l.contains(omg)) {
                    l.remove(omg);
                    return true;
                }
            }
            if (tmpOmg == omg) {
                dObj.addOMDataLocation(s, new OMDataLocation());
                return true;
            }
        }
        return false;
    }

    public void removeDataObjectAt(int index) {
        dataObjects.remove(index);
        setTable();
    }

    public void editOMGraphicAt(int index, OMGraphic omg) {
        if (omg instanceof OMGraphicList) {
            for (int i = 0; i < ((OMGraphicList) omg).size(); i++)
                editOMGraphicAt(index, ((OMGraphicList) omg).getOMGraphicAt(i));
            return;
        }
        EditableOMGraphic eomg = drawingTool.getEditableGraphic(omg);
        drawingTool.setCurrentEditable(null);
        OMGraphic newOMG = drawingTool.edit(eomg, requestor);

        DataObject dObj = getElementAt(index);
        for (int i = 0; i < getDefaultDataTableModel().getOMColumnNames().length; i++) {
            String s = getDefaultDataTableModel().getOMColumnNames()[i];
            if (dObj.getOMDataLocation(s) == null)
                continue;
            OMGraphic tmpOmg = dObj.getOMDataLocation(s).getOMGraphic();
            if (tmpOmg == null) continue;
            if (tmpOmg instanceof OMGraphicList) {
                OMGraphicList l = (OMGraphicList) tmpOmg;
                if (l.contains(omg)) {
                    l.remove(omg);
                    l.add(newOMG);
                }
            }
            if (tmpOmg == omg) {
                dObj.getOMDataLocation(s).setOMGraphic(omg);

            }
        }
    }

    public static EditableOMGraphic makeEditableOMGraphicfor(OMGraphic omg) {
        if (omg instanceof OMPoly)
            return new EditableOMPoly((OMPoly) omg);
        if (omg instanceof OMPoint)
            return new EditableOMPoint((OMPoint) omg);
        if (omg instanceof OMCircle)
            return new EditableOMCircle((OMCircle) omg);
        if (omg instanceof OMLine)
            return new EditableOMLine((OMLine) omg);
        if (omg instanceof OMGraphicList) {
            EditableOMGraphicList eomgl = new EditableOMGraphicList();
            for (OMGraphic o : (OMGraphicList) omg) {
                eomgl.add(makeEditableOMGraphicfor(o));
            }
            return eomgl;
        }
        return null;
    }
    public static final String editDSPropertyString = "editDatastorage";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String dataColumnPropertyString = "dataColumn";
    public static final String omColumnPropertyString = "omColumn";


    public void setProperties(String prefix, Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + editDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        dataColumn = properties.getProperty(prefix + dataColumnPropertyString, "");
        omColumn = properties.getProperty(prefix + omColumnPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        _dataStorage = Util.loadDataStorage(dsClassName, prefix, properties, this);
        initialize();
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + editDSPropertyString + ".";
        properties.put(prefix + dataColumnPropertyString, dataColumn);
        properties.put(prefix + omColumnPropertyString, omColumn);
        if (_dataStorage != null) {
            properties.put(prefix + dataStorageClassPropertyString, _dataStorage.getClass().getCanonicalName());
            // Debug.message("DataStorage", "PivotDataStorage.getProperties() - DataStorage member is a: "+dataStorage.getClass().getCanonicalName());
            properties = _dataStorage.getProperties(prefix, properties);
        }
        return properties;

    }

    public void dataStorageEvent(DataStorage source) {
        Debug.message("DataStorage", "[EditableDatastorage:] dataStorageEvent <== " + source);

        if (isReady()) {
            Debug.message("DataStorage", "[EditableDatastorage] data ready ***");

            initialize();
        }
    }
}
