// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * HashedDataStorage.java
 *
 * Created on 15 february 2007
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;

import java.util.ArrayList;
import java.util.Properties;
import java.util.HashMap;

import java.awt.event.*;
import javax.swing.*;

/**
 * A HashedDataStorage is a DataStorage made of :
 * <ol><li>a DataStorage of any type;</li>
 *     <li>a column name used for the (join) index;</li>
 *     <li>a hashtable that gives the ordinal index in the DataStorage of a row for a given value of the join column.</li>
 * </ol>
 * Solely used for the second DataStorage of a DualDataStorage. 
 * Therefore set and getProperties methods don't behave exactly as for other standalone DataStorage.
 */
public class HashedDataStorage extends DataStorage implements DataStorageListener {

    /* --------------------------  Begin   Properties  ---------------------*/
     public static final String joinColumnNamePropertyString = "joinColumnName";        
     public static final String dataStoragePropertyString = "ds";        
     public static final String dataStorageClassPropertyString = "dataStorageClass";
    /* --------------------------  End     Properties  ---------------------*/

    private DataStorage dataStorage = null;

    private String joinColumnName = "";
    protected void setJoinColumnName(String s) { 
	if (s != null && s.equals(joinColumnName)) {
	    Debug.message("DataStorage", "[HashedDataStorage] skip reinit index for joinColumnName:" + s + "##");
	    return;
	}
	joinColumnName = s; 
	Debug.message("DataStorage", "[HashedDataStorage] init index for joinColumnName:" + s + "##");
	//	try {
	//	    throw new Exception("trace");
	//	} catch (Exception e) {
	//	    e.printStackTrace();
	//	}
	createHashtable(dataStorage);
    }

    /** get the ordinal index in the DataStorage of a row of a given value of the join column */
    private HashMap<String, Integer> hashtable = null;

    public HashedDataStorage(DataStorageListener dsl) {
	super(dsl);
    }    

    public HashedDataStorage(DataStorageListener dsl, DataStorage dataStorage, String joinColumnName) {
	super(dsl);
	this.dataStorage = dataStorage;
	setJoinColumnName(joinColumnName);	
    }

//   public HashedDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
//	this(dsl);
//	this.dataStorage = dataStorage;
//    }  

   
    public void setProperties(String prefix, java.util.Properties properties) { 
	prefix = PropUtils.getScopedPropertyPrefix(prefix) + dataStoragePropertyString + ".";
	String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString);
	dataStorage = Util.loadDataStorage(dsClassName, prefix, properties, this);
	dataStorage.setProperties(prefix, properties);
	setJoinColumnName(properties.getProperty(prefix + joinColumnNamePropertyString, ""));
    }
    
    /*
    public void loadDataStorage(String dataStorageClassName, String prefix, Properties properties) {
	setProperties(prefix, properties);
	dataStorage = Util.loadDataStorage(dataStorageClassName, prefix, properties, this);
	Debug.message("DataStorage", "[HashedDataStorage]"+prefix+" uses "+dataStorageClassName+" and "+ joinColumnName + " index column name"); 
    }
 
    public void loadDataStorage(String dataStorageClassName, String name) {
	setName(name);
	dataStorage = Util.createDataStorage(dataStorageClassName, this);
	Debug.message("DataStorage", "[HashedDataStorage] uses "+dataStorageClassName+" and "+ joinColumnName + " index column name"); 
    }

    */

   public java.util.Properties getProperties(java.util.Properties properties) {
	return getProperties("", properties);
    }



    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix) + dataStoragePropertyString + ".";

	properties.put(prefix + dataStorageClassPropertyString, dataStorage.getClass().getCanonicalName());
	properties = dataStorage.getProperties(prefix, properties);
	properties.put(prefix+joinColumnNamePropertyString, getJoinColumnName());

	return properties;
    }	



    /* <forwarded methods to dataStorage member> */
    public DataTableModel getDefaultDataTableModel() { return dataStorage != null ? dataStorage.getDefaultDataTableModel() : null; }

    public boolean isReady() {
        //System.out.println(this.classNameForDebug +" Ready? : "+ dataStorage != null && dataStorage.isReady() && hashtable != null);
        /*Debug.message("DataStorage", "ds = "+dataStorage +"; ds isReady? = "+dataStorage.isReady() +
                " ; Hastable = "+ hashtable);*/
        /*if(dataStorage != null && dataStorage.isReady()){
            createHashtable(dataStorage);
            return true;
        }
        return false;*/
        return dataStorage != null && dataStorage.isReady() && hashtable != null ;
    }

    public int getCount() { return dataStorage.getCount(); }

    public DataObject getElementAt(int i) { return dataStorage.getElementAt(i); }
    /* </forwarded methods to dataStorage member> */

    private JComboBox joinColumnComboBox = new JComboBox();;

    // prevent JComboBox to set selected the added item
    private boolean _inhibitSetJoinColumnName = false;
    protected boolean isSetJoinColumnNameInhibited() { return _inhibitSetJoinColumnName; }

    private void resetJoinColumnComboBox(String[] names) {
	//joinColumnName = "";
	hashtable = null;
	setJoinColumnComboBox(names);
    }

    private void setJoinColumnComboBox(String[] names) {
	_inhibitSetJoinColumnName = true;
	joinColumnComboBox.removeAllItems();
	for (String s : names)
	    joinColumnComboBox.addItem(s);
	_inhibitSetJoinColumnName = false;
	joinColumnComboBox.setSelectedIndex(-1);
    }

    // <Warning message>
    private JLabel warningMessage = new JLabel();
    // </Warning message>

    public JComponent getGUI() {

	Box hashPanel = Box.createVerticalBox();
	// add here specific control for HashedDataStorage

	// <Warning message>
	if (joinColumnName == null || "".equals(joinColumnName))
	   warningMessage.setText("<html>You <font color='red'>must</font> set the Join attribute</html>");
	hashPanel.add(warningMessage);
	// </Warning message>

	JPanel joinColumnNamePanel = new JPanel();
	joinColumnNamePanel.add(new JLabel("Join attribute "));
	if (getDefaultDataTableModel() != null) {
	    setJoinColumnComboBox(getDefaultDataTableModel().getDataColumnNames());
	    Debug.message("DataStorage", "[HashedDataStorage:] set joinColumnComboBox to "+joinColumnName);
	    joinColumnComboBox.setSelectedIndex(getDefaultDataTableModel().getIndexForColumnName(joinColumnName));
	}
	joinColumnComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (getDefaultDataTableModel() != null) {
			int index = ((JComboBox)e.getSource()).getSelectedIndex();
			if (index >= 0 && ! isSetJoinColumnNameInhibited()) {
			    setJoinColumnName(getDefaultDataTableModel().getRealColumnName(index));
			    // <Warning message>
			    warningMessage.setText("");
			    // </Warning message>
			}
		    }}});
	joinColumnNamePanel.add(joinColumnComboBox);
	hashPanel.add(joinColumnNamePanel);

	Box panel = Box.createVerticalBox();
	panel.add(hashPanel);
	// add the datastorage controls
	panel.add(dataStorage.getGUI());

	// wrap the panel in a flow layout panel in order to shrink the components
	JPanel wrapPanel = new JPanel();
	wrapPanel.add(panel);
	return wrapPanel;
    }

    public void dataStorageEvent (DataStorage source) {
	Debug.message("DataStorage", "[HashedDataStorage:] dataStorageEvent <== " + source);

	// We update the GUI
	if (getDefaultDataTableModel() != null)
	    resetJoinColumnComboBox(getDefaultDataTableModel().getDataColumnNames());
	// If can create the Hashtable, it will tell the listener we're ready
	createHashtable(source);
    }

    public boolean createHashtable(DataStorage dataStorage) {
	if (joinColumnName == null || "".equals(joinColumnName)) {
	    Debug.message("DataStorage", "[HashedDataStorage:] joinColumnName unset, skip hashtable creation");
	    return false;
	}
	if (! dataStorage.isReady()) {
	    Debug.message("DataStorage", "[HashedDataStorage:] dataStorage " + dataStorage + "not ready, skip hashtable creation");
	    return false;
	}

	hashtable = new HashMap<String, Integer>(dataStorage.getCount());
	for (int i = 0; i < dataStorage.getCount(); i++) {
	    //	    System.out.println("HashedDataStorage.createHashtable: '"+ dataStorage.getElementAt(i).getData(joinColumnName) + "' => " + i);
	    Object data = dataStorage.getElementAt(i).getData(joinColumnName);
        hashtable.put(data == null ? "" : data.toString(), i);
	}
	Debug.message("DataStorage", "[HashedDataStorage:] hashtable created");
	dsl.dataStorageEvent(this);
	return true;
    }
    
    public String getJoinColumnName() {
	return joinColumnName;
    }

    public DataObject getElementForJoinValue(String v) {
	if (hashtable == null)
	    return null;
	Integer index = hashtable.get(v);
	if (index == null) {
	    if (getWarning()) 
		System.err.println("No record in "+ dataStorage.getName() + " with " + joinColumnName + " = '"+v+"'");
	    return null;
	}
	return dataStorage.getElementAt(index);
    }

    public DataStorage getDataStorage(){
	return dataStorage;
    }
    
    public void setWarning(boolean b) { 
	super.setWarning(b); 
	if (dataStorage != null)
	    dataStorage.setWarning(b);
    }
}

