// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * FilterDataStorage.java
 *
 * Created on 27 february 2008
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.animation.MetaAnimator;
import fr.unicaen.openmap.dataStorage.gui.MonoValuedFilterDataStoragePanel;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.util.ImageExporter;
import fr.unicaen.openmap.util.Util;
import java.util.ArrayList;
import java.util.Map;
import java.util.LinkedHashMap;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A FilterDataStorage is a DataStorage made of :
 * <ol>
 * <li>a DataStorage of any type;</li>
 * <li>a column name used for filtering;</li>
 * <li>a value.</li>
 * </ol>
 * Its data is the restriction of the DataStorage rows for which the column
 * value is equal to the given value.
 */
public class MonoValuedFilterDataStorage extends FilterDataStorage {

    public static final String filterColumnNamePropertyString = "filterColumnName";
    public static final String filterColumnValuePropertyString = "filterColumnValue";
    public static final String modePropertyString = "mode";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String animationDSPropertyString = "animationDS";
    // private ArrayList<AnimationDataStorageListener> listeners = new ArrayList<AnimationDataStorageListener>();
    /** get the filtered DataTable given a value of the filter column */
    protected Map<String, ArrayList<DataObject>> hashtable;
    public String noFilter =
            i18n.get(MonoValuedFilterDataStorage.class, "noFilter", "-- no Filter --");
    protected Box animationGUI;
    private Exporter exporter;
    //private int nbStepAnim = 0;
    private boolean exportMode;
    private JLabel columnLabel;
    private JLabel valueLabel;
    private JComboBox metaGUI;
    private MetaAnimator metaAnimator;

    /**
     * @return the nextValueAction
     */
//    public Action getNextValueAction() {
//        return nextValueAction;
//    }
    public enum Mode {

        AUTO("automatic"),
        MANUAL("manual"),
        STEP("step");
        private final String label;

        Mode(String label) {
            this.label = Environment.getI18n().get(MonoValuedFilterDataStorage.class, label, label);
        }

        @Override
        public String toString() {
            return label;
        }
    }
    private String filterColumnValue = "";
    private int filterColumnValueIndex = -1;
    //protected Mode selectedMode = Mode.AUTO;

    public MonoValuedFilterDataStorage(DataStorageListener dsl) {
        this(dsl, null, null);
    }

    public MonoValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        this(dsl, dataStorage, null);
    }

    public MonoValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage,
            String filterColumnName) {
        super(dsl, dataStorage, filterColumnName);
        AnimationDSEventManager.fireAddAnimation(this);
        initialize();
    }

    /**
     * @param metaAnimator the metaAnimator to set
     */
    public void setMetaAnimator(MetaAnimator metaAnimator) {
        this.metaAnimator = metaAnimator;

    }

//    public void setNbStepAnimation(int nb){
//        nbStepAnim = nb;
//    }
//    public int getNbStepAnimation(){
//        return nbStepAnim;
//    }
    public void setExportMode(boolean b) {
        exportMode = b;
        initFilterExpressionPanel();
        filterExpressionPanel.updateUI();
    }

    public String getFilterColumnName() {
        return filterColumnName;
    }

    protected void setFilterColumnName(String s) {
//        try{}
//        catch(Exception e){System.out.println(s);e.printStackTrace();}
        filterColumnName = s;
        Debug.message("DataStorage",
                "[FilterDataStorage] (re)init index for filterColumnName:" + s);
        createHashtable(getDataStorage());
        
        // if(labGUI != null)
        //    labGUI.setText("Colonne de filtre = " + s );
    }

    protected ArrayList<DataObject> getFilteredData() {
        if (hashtable == null)
            return new ArrayList<DataObject>();
        return hashtable.get(filterColumnValue);
    }

    protected boolean createHashtable(DataStorage dataStorage) {
        if (!dataStorage.isReady()) {
            Debug.message("DataStorage", "[FilterDataStorage:] dataStorage " + dataStorage + "not ready, skip hashtable creation");
            return false;
        }

        // setColumnNames();

        if (filterColumnName == null || "".equals(filterColumnName)) {
            Debug.message("DataStorage",
                    "[FilterDataStorage:] filterColumnName unset, skip hashtable creation");
            return false;
        }

        hashtable = new LinkedHashMap<String, ArrayList<DataObject>>();
        // step 1: collect the different filterColumnName values, associating a
        // null ArrayList
        for (int i = 0; i < dataStorage.getCount(); i++) {
            // System.out.println("FilterDataStorage.createHashtable: '"+
            // dataStorage.getElementAt(i).getData(filterColumnName) + "' => " +
            // i);
            Object d = dataStorage.getElementAt(i).getData(filterColumnName);
            hashtable.put(d == null ? "" : d.toString(), null);
        }
        // step 2: build the keyValues table and add the joker value
        keyValues = new ArrayList<String>();
        for (String value : hashtable.keySet()) {
            keyValues.add(value);
        }

        //Test
        ////Collections.sort(keyValues); // we should use a sorted insertion !

        keyValues.add(0, noFilter);
        // step 3: create the ArrayLists
        for (String value : keyValues) {
            hashtable.put(value, new ArrayList<DataObject>());
        }
        // step 4: populate the ArrayLists

        ArrayList<DataObject> fullData = hashtable.get(noFilter);
        hashtable.put(noFilter, fullData);
        for (int i = 0; i < dataStorage.getCount(); i++) {
            DataObject data = dataStorage.getElementAt(i);
            Object d = data.getData(filterColumnName);
            String value = (d == null ? "" : d.toString());
            hashtable.get(value).add(data);
            fullData.add(data);
        }

        Debug.message("DataStorage", "[FilterDataStorage:] hashtable created");
        return true;
    }

    public String getFilterColumnValue() {
        return filterColumnValue;
    }

    protected int getFilterColumnValueIndex() {
        return filterColumnValueIndex;
    }

    public void setFilterColumnValue(String value) {
        if (value == null || keyValues == null) {
            return;
            //value = "";
        }
        filterColumnValue = value;
        filterColumnValueIndex = keyValues.indexOf(filterColumnValue);
        setTable();
    }

    protected void setFilterColumnValueIndex(int index) {
        filterColumnValueIndex = index;
        try {
            //System.out.println("FILTER AVANT =" + filterColumnValue);
            filterColumnValue = keyValues.get(index);
            //System.out.println("FILTER APRES =" + filterColumnValue);
        } catch (IndexOutOfBoundsException e) {
            filterColumnValue = "";
            filterColumnValueIndex = -1;
        }

        setTable();
    }

    @Override
    public void dataStorageEvent(DataStorage source) {
        String tmp = filterColumnName;
        super.dataStorageEvent(source);      
        updateMetaGUI(tmp);

    }

    protected int incrFilterColumnValueIndex() {
        return incrFilterColumnValueIndex(true);
    }

    protected int incrFilterColumnValueIndex(boolean setFilter) {
        AnimationDSEventManager.fireChangedAnimation(this);
        int index = filterColumnValueIndex + 1;
        if (index == 0) // 0 is for no-filter
        {
            index = 1;
        }
        if (index >= keyValues.size()) {
            index = 1;
        }
        if (setFilter) {
            setFilterColumnValueIndex(index);
        }
        return index;
    }

    /** returns the original DataStorage */
    public DataStorage removeItSelf() {
        /*if (automaticTimer != null) {
        automaticTimer.stop();
        }*/
        AnimationDSEventManager.fireRemoveAnimation(this);
        return super.removeItSelf();
    }

    // TODO: set and get the filterColumnValue in the properties?
    @Override
    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + animationDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        //selectedMode = getModeFromString(properties.getProperty(prefix + modePropertyString, ""));
        filterColumnName = properties.getProperty(prefix + filterColumnNamePropertyString, "");
        filterColumnValue = properties.getProperty(prefix + filterColumnValuePropertyString, "");
        //initFilterExpressionPanel();
        initialize();


    }

    private void initialize() {     
        if (!getDataStorage().isReady()) {
            Debug.message("DataStorage", "[AnimationDS] can't initialize because dataStorage is not ready !!!!");
            return;
        }
        setColumnNames();
        dataTableModel = new DataTableModel(dataTable);
        resetDataTableStoragePanel(dataTableModel);
        setFilterColumnName(filterColumnName);
        setFilterColumnValue(filterColumnValue);
        setTable();
        //Debug.message("DataStorage", "[MonoValuedFilterDataStorage] send event to " + dsl);
        //System.out.println("-------------------------------------ANIMATION DS INIT " + dataTable.getRowCount());
        //initFilterExpressionPanel();
    }

    @Override
    public java.util.Properties getProperties(java.util.Properties properties) {
        return getProperties("", properties);
    }

    @Override
    public java.util.Properties getProperties(String prefix,
            java.util.Properties properties) {
        properties = super.getProperties(prefix, properties);
        //prefix = PropUtils.getScopedPropertyPrefix(prefix); // it's not done anywhere so...
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + animationDSPropertyString + ".";
        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        String fcol = getFilterColumnName() == null ? "" : getFilterColumnName();
        properties.put(prefix + filterColumnNamePropertyString,
                fcol);
        properties.put(prefix + filterColumnValuePropertyString,
                getFilterColumnValue());
        //properties.put(prefix + modePropertyString, selectedMode.toString());
        return properties;
    }

    
    public boolean isReady() {
        return getDataStorage() != null && getDataStorage().isReady() /*&& hashtable != null*/;
    }
    //private JComboBox filterColumnComboBox = new JComboBox();
    //private Action nextValueAction;
    ///protected javax.swing.Timer automaticTimer;
    //protected JComboBox filterColumnValueComboBox, toolFilterColumnValueComboBox, filterColumnNameComboBox;
    //private JLabel labValue;
    private JButton toolNextButton;

    public void initFilterExpressionPanel() {

        if (filterExpressionPanel == null) {
            filterExpressionPanel = new JPanel(new BorderLayout());
        }
        filterExpressionPanel.removeAll();
        JPanel gridPanel = new JPanel(new GridLayout(2, 2));
        filterExpressionPanel.setBorder(
                BorderFactory.createTitledBorder(i18n.get(this, "filter", "Filter")));
        filterExpressionPanel.add(gridPanel, BorderLayout.CENTER);
        if (getDataStorage() == null) {
            warningMessage.setText(i18n.get(MonoValuedFilterDataStorage.class,
                    "noDSWarning",
                    "<html><font color='red'>No data storage</font></html>"));
            return;
        }


        //filterExpressionPanel.add(boxMode);
        gridPanel.add(new JLabel(i18n.get(MonoValuedFilterDataStorage.class,
                "filterAttribute", "Filter attribute ")));
        columnLabel = new JLabel(filterColumnName);
        gridPanel.add(columnLabel);

        /* if (keyValues == null) {
        gridPanel.add(new JPanel());
        gridPanel.add(new JPanel());
        return;
        }*/

        gridPanel.add(new JLabel(i18n.get(this, "filtervalue", "Filter value")));
        if (filterColumnValue == null || "".equals(filterColumnValue)) {
            valueLabel = new JLabel("Non paramétré");
            valueLabel.setFont(new Font("Arial", Font.ITALIC, 12));
        }
        valueLabel = new JLabel(filterColumnValue);
        gridPanel.add(valueLabel);

        if (metaAnimator != null) {
            JButton confFilterButton = new JButton(
                    i18n.get(MonoValuedFilterDataStorage.class,"configure","Configure"));
            JPanel wrapConf = new JPanel();
            confFilterButton.setBackground(Color.gray);
            confFilterButton.setOpaque(false);
            wrapConf.add(confFilterButton);
            filterExpressionPanel.add(
                    wrapConf,
                    BorderLayout.SOUTH);
            confFilterButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    metaAnimator.popup();
                }
            });

        }

        updateMetaGUI(filterColumnName);
    }

//    public AnimationDataStoragePanel getAnimationGUI() {
//
//
//    }
    public JComponent getGUI() {
        if (animationGUI == null) {
            animationGUI = Box.createVerticalBox();// AnimationDataStoragePanel(this);
            initFilterExpressionPanel();
            animationGUI.add(getFilterExpressionPanel());
            animationGUI.add(getDataStorage().getGUI());
        }
        return animationGUI;
    }

//        protected void setColumnNames() {
//        String[] names;
//        names = getDataStorage().getDefaultDataTableModel().getDataColumnNames();
//        dataColumnNames = new ArrayList<String>(names.length);
//        boolean filterColumnNameIsValid = false;
//        for (String n : names) {
//            if (n.equals(filterColumnName)) {
//                filterColumnNameIsValid = true;
//            }
//            dataColumnNames.add(n);
//        }
//        if (!filterColumnNameIsValid) {
//            // <Warning message>
//            warningMessage.setText(i18n.get(MonoValuedFilterDataStorage.class,
//                    "warning2", "Select a column name"));
//            // </Warning message>
//            filterColumnName = "";
//        }
//        names = getDataStorage().getDefaultDataTableModel().getOMColumnNames();
//        omColumnNames = new ArrayList<String>(names.length);
//        for (String n : names) {
//            omColumnNames.add(n);
//        }
//        System.out.println("SET COLUMN NAMES" + dataColumnNames.size());
//        initFilterExpressionPanel();
//    }

    public JComponent getMetaGUI() {
        //System.out.println("GET METAGUI " + dataColumnNames.size());
        if (metaGUI == null) {

            metaGUI = new JComboBox(/*dataColumnNames.toArray()*/getDataStorage().getDefaultDataTableModel().getDataColumnNames());
            metaGUI.setSelectedItem(filterColumnName);
            metaGUI.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    if (getDataStorage() != null) {
                        int index = ((JComboBox) e.getSource()).getSelectedIndex();
                        if (index >= 0) {
                            setFilterColumnName(getDataStorage().getDefaultDataTableModel().getRealColumnName(
                                    index));
                            updateColumn(filterColumnName);
                            //filterColumnNameComboBox.setSelectedIndex(index);
                            //filterColumnNameComboBox.updateUI();
                        }
                        AnimationDSEventManager.fireChangedAnimation(MonoValuedFilterDataStorage.this);

                    }
                }
            });
        }
        return metaGUI;
    }

    private void updateMetaGUI(String filter) {
        if(metaGUI == null)
            return;
        metaGUI.removeAllItems();
        for (String s : dataColumnNames) {
            metaGUI.addItem(s);
        }
        metaGUI.setSelectedItem(filter);
//        if(filterColumnName!=null && !"".equals(filterColumnName))
//            metaGUI.setSelectedItem(filterColumnName);
    }

    public JComponent getFilterExpressionPanel() {
        return filterExpressionPanel;
    }

    //public JComponent getSortPanel(){ return sortPanel;}
//    public void setSelectedMode(Mode m) {
//        if (selectedMode == Mode.AUTO) {
//            automaticTimer.stop();
//        }
//        selectedMode = m;
//        getAnimationGUI().setSelectedMode(m);
//
//    }
//
//    public Mode getSelectedMode() {
//        return selectedMode;
//    }
    public Mode getModeFromString(String st) {
        for (Mode m : Mode.values()) {
            if (m.toString().equals(st)) {
                return m;
            }
        }
        return null;
    }

    public void export(String folder, ImageExporter.ImageType type) {
        //setNbStepAnimation(0);
        if (exporter == null) {
            //exporter = new Exporter(folder, getNextValueAction(), type);
        }
        exporter.start();
    }

    public void stopExport() {
        exporter.stopThread();
    }

//    @Override
//    public String toString() {
//        return getName();
//    }

    public String getFullName() {
        return getLayer().getName();/* + " " + filterColumnName*/
    }

    public boolean isSet() {
        return getFilterColumnName() != null && !"".equals(getFilterColumnName());
    }

    public void nextStep(String value) {
        if (keyValues.contains(value))
            filterColumnValue = value;
        setTable();
    }

    public void updateValue(String value) {
        valueLabel.setText(value);
    }

    public void updateColumn(String column) {
        columnLabel.setText(column);
    }
}

class Exporter extends Thread {

    private String fileName = "AnimationExport-";
    private int index = 0;
    private Action action;
    private String folder;
    private boolean end;
    private ImageExporter.ImageType type;

    public Exporter(String folder, Action action, ImageExporter.ImageType type) {
        this.action = action;
        this.folder = folder;
        this.type = type;
    }

    public void run() {
        while (!end) {
            ImageExporter.exportInFile(folder + fileName + index + "." + type.toString().toLowerCase(),
                    type);
            index++;
            action.actionPerformed(null);
        }
    }

    public synchronized void stopThread() {
        end = true;
    }
}
