// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.expr.RequestFilterDataStorage;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.util.TreeSet;

/**
 *
 * @author jchauvea
 */
public class RequestFilterDataStoragePanel extends JPanel {

    protected I18n i18n = Environment.getI18n();
    protected JList fieldsList;
    protected JList valuesList;
    protected ExpressionPanel exprPanel;
//    protected JButton execButton;
//    protected JButton clearButton;
//    protected JTextArea expressionArea;
//    protected JLabel labState;

    public RequestFilterDataStoragePanel(final RequestFilterDataStorage ds) {
        setLayout(new BorderLayout());

        //field list
        fieldsList = new JList(ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames());
        fieldsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JPanel wrapFieldList = new JPanel(new BorderLayout());
        JLabel labFieldList = new JLabel(i18n.get(this, "dataColumns", "Data Columns"));
        wrapFieldList.add(labFieldList, BorderLayout.NORTH);
        JScrollPane sc = new JScrollPane(fieldsList);
        sc.setPreferredSize(new Dimension(200, 150));
        sc.setMaximumSize(new Dimension(200, 150));
        wrapFieldList.add(sc, BorderLayout.CENTER);
        fieldsList.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent lse) {
                if (lse.getValueIsAdjusting()) {
                    String colName = fieldsList.getSelectedValue().toString();
                    exprPanel.appendText(colName);
		    TreeSet<Object> valuesSet = new TreeSet<Object>();
		    for (Object v : ds.getDataStorage().getDefaultDataTableModel().getValues(colName))
			if (v != null) 
			    valuesSet.add(v);
                    valuesList.setListData(valuesSet.toArray());
                    try {
                        fieldsList.setSelectedIndices(new int[]{});
                    } catch (Exception e) {
                    }
                }
            }
        });


        //values List
        valuesList = new JList();
        valuesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JPanel wrapValuesList = new JPanel(new BorderLayout());
        JLabel labValuesList = new JLabel(i18n.get(this, "values", "Values"));
        wrapValuesList.add(labValuesList, BorderLayout.NORTH);
        JScrollPane sc2 = new JScrollPane(valuesList);
        sc2.setPreferredSize(new Dimension(200, 150));
        sc2.setMaximumSize(new Dimension(200, 150));
        wrapValuesList.add(sc2, BorderLayout.CENTER);
        valuesList.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent lse) {
                if (lse.getValueIsAdjusting()) {
                    exprPanel.appendText(valuesList.getSelectedValue().toString());
                    try {
                        fieldsList.setSelectedIndices(new int[]{});
                    } catch (Exception e) {
                    }
                }
            }
        });

        //buttons panel
        JPanel buttonsPanel = new JPanel(new GridLayout(3, 3));
        for (final RequestFilterDataStorage.requestElement e : RequestFilterDataStorage.requestElement.values()) {
            ExprButton b = new ExprButton(e.toString());
            buttonsPanel.add(b);
            b.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                   exprPanel.appendText(e.toString());
                }
            });

        }


        JPanel mainPanel = new JPanel();
        mainPanel.add(wrapFieldList);
        mainPanel.add(buttonsPanel);
        mainPanel.add(wrapValuesList);

        exprPanel = new ExpressionPanel(ds);

        add(mainPanel, BorderLayout.CENTER);
        add(exprPanel, BorderLayout.SOUTH);
	setBorder(BorderFactory.createTitledBorder(i18n.get(this, "title", "Expression Filter")));
	
        //Expression area panel
//        expressionArea = new JTextArea(1, 40);
//        execButton = new JButton("Execute");
//        clearButton = new JButton("Clear");
//
//        JPanel pBottom = new JPanel();
//        labState = new JLabel(Util.loadIcon("green.png"));
//        pBottom.add(labState);
//        pBottom.add(expressionArea);
//        pBottom.add(execButton);
//        pBottom.add(clearButton);
//
//        execButton.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent ae) {
//                //ds.setExpr("AddColumn(if(" + expressionArea.getText() + ";0;1))");
//                updateExpr(ds, expressionArea.getText());
//                ds.initialize();
//            }
//        });
//
//        clearButton.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent arg0) {
//                expressionArea.setText("");
//            }
//        });
//
//        add(pBottom, BorderLayout.SOUTH);
//
//        expressionArea.addKeyListener(new KeyListener() {
//
//            public void keyTyped(KeyEvent arg0) {
//                updateExpr(ds, expressionArea.getText());
//            }
//
//            public void keyPressed(KeyEvent arg0) {
//                //throw new UnsupportedOperationException("Not supported yet.");
//                updateExpr(ds, expressionArea.getText());
//            }
//
//            public void keyReleased(KeyEvent arg0) {
//                updateExpr(ds, expressionArea.getText());
//                //throw new UnsupportedOperationException("Not supported yet.");
//                //System.out.println("KeyReleased");
//            }
//        });

    }

//    private void updateExpr(ExprDataStorage ds, String text) {
//        boolean s = ds.setExpr("AddColumn(if(" + text + ";0;1))");
//        System.out.println("Syntaxe de la requête " + s);
//        if(s)
//            labState.setIcon(Util.loadIcon("green.png"));
//        else
//            labState.setIcon(Util.loadIcon("red.png"));
//        labState.updateUI();
//    }
}
