// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gml.in;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.util.Util;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.dataStorage.Centroid;

public class GMLParser extends DefaultHandler {

    private ArrayList<DataObject> dataObjects;
    private ArrayList<String> columnNames;
    private ArrayList<String> omGraphicNames;
    private StringBuffer tempStringBuffer;
    private OMDataLocation tempOMDataLocation;
    private DataObject tempDataObject;
    private String tempColumnName;
    private OMGraphicList tempOmgList;
    private pointType currentPoint;
    private LatLonPoint upperLeft, lowerRight;
    private int countShape = 0;
    //private String NO_TAG = "NO_TAG";
    private String currentTag = "NO_TAG";
    /*private DataObject tempDataObject = null;
    private Object tempData = null;
    private OMDataLocation tempOMDataLocation = null;
    private String tempOMDataLocationName = null;
    private String columnName = null;

    private StringBuffer tempStringBuffer = new StringBuffer("");
    private boolean firstObject = true;
     */

    public enum pointType {

        UPPER_LEFT, LOWER_RIGHT, CENTROID, GEO_POINT
    }

    public GMLParser(String fileName) {
        tempStringBuffer = new StringBuffer("");
        dataObjects = new ArrayList<DataObject>();
        columnNames = new ArrayList<String>();
        omGraphicNames = new ArrayList<String>();
        parse(fileName);

    }

    public ArrayList<DataObject> getDataObjects() {
        return dataObjects;
    }

    public ArrayList<String> getColumnNames() {
        return columnNames;
    }

    public ArrayList<String> getOMGraphicsNames() {
        return omGraphicNames;
    }

    public void parse(String fileName) {

        InputStream in = Util.open(fileName, true);
        parse(in);
//
//        FileInputStream in;
//        String newFileName = fileName;
//        try {
//            in = new FileInputStream(fileName);
//        } catch (FileNotFoundException e) {
//            newFileName = Util.locateFile(fileName);
//            if (newFileName == null || newFileName == "") {
//                System.err.println("[GMLDataStorage] ### File not found " + fileName + " ###");
//                return;
//            }
//            try {
//                in = new FileInputStream(newFileName);
//            } catch (FileNotFoundException ee) {
//                System.err.println("[GMLDataStorage] ### File not found exception for " + fileName + " as " + newFileName);
//                return;
//            }
//        }
//        try {
//            parse(new GZIPInputStream(in));
//        } catch (Exception notGZIPexception) {
//            try {
//                parse(new FileInputStream(newFileName));
//            } catch (FileNotFoundException ee) {
//                System.err.println("[GMLDataStorage] ### File not found exception for " + fileName + " as " + newFileName);
//                return;
//            }
//        }
    }

    public void parse(InputStream file) {

        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(false);
        try {
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse(file, this);
        } catch (SAXParseException spe) {
            System.err.println("[GMLDataStorage] Parse Error, line " + spe.getLineNumber() + ": " + spe.getMessage());
        } catch (SAXException sxe) {
            System.err.println("[GMLDataStorage] " + sxe.getMessage());
        } catch (ParserConfigurationException pce) {
            System.err.println("[GMLDataStorage] " + pce.getMessage());
        } catch (IOException ioe) {
            System.err.println("[GMLDataStorage] " + ioe.getMessage());
        }
        //System.out.println("nombre de data = " + dataObjects.size());
    }

    private void debugElement(String qName, Attributes attrs) {
        System.err.println("Element: " + qName);
        for (int i = 0; i < attrs.getLength(); i++) {
            String qn = attrs.getQName(i);
            String type = attrs.getType(i);
            String value = attrs.getValue(i);
            System.err.println("qn: " + qn);
            System.err.println("type: " + type);
            System.err.println("value: " + value);
            System.err.println("");
        }
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
        currentTag = qName;

        if (qName.equals(GMLDtd.DS_FEATURE)) {
            tempDataObject = new DataObject();
            countShape = 0;
            return;
        }
        if (qName.equals(GMLDtd.GEOMETRY_PROPERTY) || qName.equals(GMLDtd.GEOMETRY_PROPERTY_OMPG)) {
            tempOMDataLocation = new OMDataLocation();
            countShape++;
            return;
        }

        if (qName.equals(GMLDtd.MUTLI_POLYGON)) {
            tempOmgList = new OMGraphicList();
            return;
        }

        if (qName.equals(GMLDtd.LINE_STRING)) {
            if (tempOmgList == null)
                tempOmgList = new OMGraphicList();
            return;
        }

        if (qName.equals(GMLDtd.POINT)) {
            if (attrs.getValue("name") != null) {
                if (attrs.getValue("name").equals(GMLDtd.CENTROID)) {
                    //tempOMDataLocation = new OMDataLocation();
                    currentPoint = pointType.CENTROID;
                    return;
                }
                if (attrs.getValue("name").equals(GMLDtd.UPPER_LEFT)) {
                    currentPoint = pointType.UPPER_LEFT;
                    return;
                }
                if (attrs.getValue("name").equals(GMLDtd.LOWER_RIGHT)) {
                    currentPoint = pointType.LOWER_RIGHT;
                    return;
                }
            } else
                currentPoint = pointType.GEO_POINT;
        }

    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        try {
            if (qName.equals(GMLDtd.DATA_NAME) || qName.equals(GMLDtd.DATA_NAME_OMPG)) {
                if (!columnNames.contains(tempStringBuffer.toString().trim())) {
                    columnNames.add(tempStringBuffer.toString().trim());
                }
                tempColumnName = tempStringBuffer.toString().trim();
                tempStringBuffer = new StringBuffer("");
                return;
            }
            if (qName.equals(GMLDtd.COORDINATES)) {
                if (tempOMDataLocation != null) {
                    OMPoly om = new OMPoly(
                            getPointsFromString(tempStringBuffer.toString()),
                            OMPoly.DECIMAL_DEGREES,
                            OMPoly.LINETYPE_STRAIGHT);

                    om.setIsPolygon(true);
                    om.setRenderType(OMGraphic.RENDERTYPE_LATLON);
                    om.putAttribute(Util.OMGraphicIndex, dataObjects.size());
                    //tempOmgList.addOMGraphic(om);
                    tempOmgList.add(om);
                }

                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.DATA_VALUE) || qName.equals(GMLDtd.DATA_VALUE_OMPG)) {
                if (tempDataObject != null && tempColumnName != null) {
                    //System.out.println("colonne: "+tempColumnName.trim()+" - valeur: "+tempStringBuffer.toString().trim());
                    tempDataObject.addData(tempColumnName, tempStringBuffer.toString().trim());
                }
                tempStringBuffer = new StringBuffer("");
                return;
            }
            if (qName.equals(GMLDtd.FEATURE_MEMBER)) {
                tempOMDataLocation = null;
                tempDataObject = null;
                //tempColumnName = null;
                tempStringBuffer = new StringBuffer("");
                return;

            }

            if (qName.equals(GMLDtd.DS_FEATURE)) {
                dataObjects.add(tempDataObject);
                tempDataObject = null;
                tempStringBuffer = new StringBuffer("");
                return;
            }


            if (qName.equals(GMLDtd.MUTLI_POLYGON)) {
                tempOMDataLocation.setOMGraphic(tempOmgList);
                try {
                    tempOMDataLocation.setBoundingBox(new BoundingBox(upperLeft, lowerRight));
                } catch (NullPointerException exc) {
		    // System.out.println(upperLeft +","+lowerRight);
                }
                //tempDataObject.addOMDataLocation("shape"+countShape, tempOMDataLocation);
                //omGraphicNames.add("shape"+countShape);
                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.GEOMETRY_PROPERTY) || qName.equals(GMLDtd.GEOMETRY_PROPERTY_OMPG)) {
                tempDataObject.addOMDataLocation("shape" + countShape, tempOMDataLocation);
                omGraphicNames.add("shape" + countShape);
                return;
            }

            if (qName.equals(GMLDtd.POS)) {
                double point[] = getPointFromString(tempStringBuffer.toString().trim());
                //System.out.println(tempStringBuffer.toString().trim());
                switch (currentPoint) {
                    case CENTROID:
                        tempOMDataLocation.setCentroid(new Centroid(point[1], point[0]));
                        break;
                    case UPPER_LEFT:
                        upperLeft = new LatLonPoint.Double(point[0], point[1]);
                        break;
                    case LOWER_RIGHT:
                        lowerRight = new LatLonPoint.Double(point[0], point[1]);
                        break;
                    case GEO_POINT:
                        tempOmgList.add(new OMPoint(point[0], point[1]));
                        return;
                }
                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.POS_LIST)) {
                double point[] = getCoordsFromString(tempStringBuffer.toString().trim());
                tempOmgList.add(new OMLine(
                        point[0],
                        point[1],
                        point[2],
                        point[3],
                        OMLine.LINETYPE_STRAIGHT));
                return;
            }


            tempStringBuffer = new StringBuffer("");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * This method will only concatenete buffers for
     */
    public void characters(char buf[], int offset, int len) throws SAXException {
        tempStringBuffer.append(buf, offset, len);
    }

    /**
     * Error handling is reported above.
     */
    public void error(SAXParseException e) throws SAXParseException {
        throw e;
    }

    /**
     * Warning is handled to stderr.
     */
    public void warning(SAXParseException err) throws SAXParseException {
        System.err.println("[XmlParser] Warning, line " + err.getLineNumber() + ", " + err.getSystemId());
        System.err.println("[XmlParser] " + err.getMessage());
    }

    private double[] getPointsFromString(String coords) {
        try {
            ArrayList<Double> pointsTmp = new ArrayList<Double>();
            String[] tab = coords.split(" ");
            //System.out.println("tableau de coords = "+tab);
            for (int i = 0; i < tab.length; i++) {
                String[] latlon = tab[i].split(",");
                //System.out.println(latlon.length);
                if (latlon.length == 2) {
                    pointsTmp.add(Double.valueOf(latlon[1]));
                    pointsTmp.add(Double.valueOf(latlon[0]));
                }

            }
            double[] points = new double[pointsTmp.size()];
            for (int j = 0; j < pointsTmp.size(); j++) {
                points[j] = pointsTmp.get(j).doubleValue();
            }
            return points;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    
    private double[] getCoordsFromString(String coords) {
            String[] stValues = coords.split(" ");
            double[] coordinates = new double[4];
            for(int i = 0; i < coordinates.length; i++)
                coordinates[i] = Double.valueOf(stValues[i]);
            return coordinates;
    }



    private double[] getPointFromString(String pointString) {
        double point[] = new double[2];
        String[] coords = pointString.split(" ");
        point[0] = Double.valueOf(coords[0]);
        point[1] = Double.valueOf(coords[1]);
        return point;
    }
}


