// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.dataStorage.AggregateDataStorage;
import fr.unicaen.openmap.dataStorage.PivotDataStorage;
import fr.unicaen.openmap.gui.component.JComponentCellEditor;
import fr.unicaen.openmap.gui.component.JComponentCellRenderer;
import fr.unicaen.openmap.gui.component.JComponentTable;
import fr.unicaen.openmap.gui.component.JComponentTableModel;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

/**
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 */
public class AggregateDataStoragePanel extends JPanel {


    protected I18n i18n = Environment.getI18n();
    protected AggregateDataStorage aggDS;
    public AggregateDataStoragePanel(AggregateDataStorage ds) {
        this.aggDS = ds;

	Box gridPanel = Box.createVerticalBox();
     /***************************************/
	Box tablePanel = Box.createVerticalBox();
	JComponentTable table;
	{
	    String[] attributesNames = ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames();
	    ds.initAttributesHashMapFromLists(attributesNames);

	    String criteriaToolTipText = i18n.get(this, "criteria", I18n.TOOLTIP, "Lead to one row per criteria value");
	    //String pivotToolTipText = i18n.get(this, "pivot", I18n.TOOLTIP, "Lead to one statistic attribute per pivot value");
	    String statisticsToolTipText = i18n.get(this, "statistics", I18n.TOOLTIP, "The statistics values aggregated with each criteria group");
	    String discardToolTipText = i18n.get(this, "discard", I18n.TOOLTIP, "The discarded attributes");
	    String aggregateFunctionToolTipText = i18n.get(this, "aggregateFunction", I18n.TOOLTIP, "The aggregate function applied to the statistic attribute");

	    String[] columnNames = {
		i18n.get(PivotDataStorage.class, "attributes", "Attributes"),
		i18n.get(PivotDataStorage.class, "discard", "Discard"),
		i18n.get(PivotDataStorage.class, "criteria", "Criteria"),
		i18n.get(PivotDataStorage.class, "statistics", "Statistics"),
		i18n.get(PivotDataStorage.class, "aggregateFunction", "Aggregate Function"),
	    };
	    Object[][] data = new Object[attributesNames.length][columnNames.length];
	    JComponentTableModel model = new JComponentTableModel(data, columnNames);
	    table = new JComponentTable(model);
	    table.setRowHeight(25);
	    table.setBackground(new Color(238,238,238));
	    table.setDefaultRenderer( JComponent.class, new JComponentCellRenderer() );
	    DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
	    renderer.setHorizontalAlignment(SwingConstants.CENTER);
	    TableColumn col1 = table.getColumnModel().getColumn(0);
	    col1.setCellRenderer(renderer);
	    table.setDefaultEditor( JComponent.class, new JComponentCellEditor() );

	    int index = 0;
	    for (String name : attributesNames) {
		table.setValueAt(name, index, 0);


		JComboBox cb = ds.new AggregateFunctionComboBox(name);
		cb.setToolTipText(aggregateFunctionToolTipText);

		table.setValueAt(cb, index, 4);
		cb.setEnabled(ds.hasAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));
	    
		ButtonGroup attributeRBGroup = new ButtonGroup();
		JRadioButton  rb1, rb2, rb3;

		// 3 times the same lines!

		rb1 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.DISCARD, cb, table));
		rb1.setToolTipText(discardToolTipText);
		rb1.setVerticalAlignment(JRadioButton.CENTER);
		rb1.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.DISCARD));
		attributeRBGroup.add(rb1);
	    
		table.setValueAt(rb1, index, 1);


		rb2 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.CRITERIA, cb, table));
		rb2.setToolTipText(criteriaToolTipText);
		rb2.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.CRITERIA));
		attributeRBGroup.add(rb2);
		table.setValueAt(rb2, index, 2);

		rb3 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.STATISTIC, cb, table));
		rb3.setToolTipText(statisticsToolTipText);
		rb3.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));
		attributeRBGroup.add(rb3);
		table.setValueAt(rb3, index, 3);
   
		index ++;
	    }
	}

	JTable table2;
	{
	    String[] geometryAttributesNames = ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames();
	    ds.initGeometryAttributesHashMapFromLists(geometryAttributesNames);
	    String discardToolTipText = i18n.get(this, "discard", I18n.TOOLTIP, "The discarded attributes");
	    // We use AttributeType.STATISTIC instesd of GEOMETRY
	    String geometryToolTipText = i18n.get(this, "geometry", I18n.TOOLTIP, "The geometries to be aggregated");
	    String aggregateFunctionToolTipText = i18n.get(this, "geometryAggregateFunction", I18n.TOOLTIP, "The aggregate function applied to the geometry");
	    String parameterToolTipText = i18n.get(this, "parameter", I18n.TOOLTIP, "The column used as a second parameter");

	    String[] columnNames = {
		i18n.get(PivotDataStorage.class, "attributes", "Attributes"),
		i18n.get(PivotDataStorage.class, "discard", "Discard"),
		i18n.get(PivotDataStorage.class, "geometry", "Geometry"),
		i18n.get(PivotDataStorage.class, "aggregateFunction", "Aggregate Function"),
		i18n.get(PivotDataStorage.class, "parameter", "Parameter"),
	    };
	    Object[][] data = new Object[geometryAttributesNames.length][columnNames.length];
	    JComponentTableModel model = new JComponentTableModel(data, columnNames);
	    table2 = new JComponentTable(model);
	    table2.setRowHeight(25);
	    table2.setBackground(new Color(238,238,238));
	    table2.setDefaultRenderer( JComponent.class, new JComponentCellRenderer() );
	    DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
	    renderer.setHorizontalAlignment(SwingConstants.CENTER);
	    TableColumn col1 = table2.getColumnModel().getColumn(0);
	    col1.setCellRenderer(renderer);
	    table2.setDefaultEditor( JComponent.class, new JComponentCellEditor() );

	    int index = 0;
	    for (String name : geometryAttributesNames) {
		table2.setValueAt(name, index, 0);


		JComboBox cbParameter = ds.new ParameterComboBox(name);
		cbParameter.setToolTipText(parameterToolTipText);

		table2.setValueAt(cbParameter, index, 4);
		cbParameter.setEnabled(ds.geometryAggregateFunctionNeedsSecondParameter(name));

		JComboBox cb = ds.new GeometryAggregateFunctionComboBox(name, cbParameter);
		cb.setToolTipText(aggregateFunctionToolTipText);

		table2.setValueAt(cb, index, 3);
		cb.setEnabled(ds.hasGeometryAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));

		ButtonGroup attributeRBGroup = new ButtonGroup();
		JRadioButton  rb1, rb2;

		// 2 times the same lines!

		rb1 = new JRadioButton(ds.new RBGeometryAction(name, PivotDataStorage.AttributeType.DISCARD, cb, table2));
		rb1.setToolTipText(discardToolTipText);
		rb1.setVerticalAlignment(JRadioButton.CENTER);
		rb1.setSelected(ds.hasGeometryAttributeType(name, PivotDataStorage.AttributeType.DISCARD));
		attributeRBGroup.add(rb1);
	    
		table2.setValueAt(rb1, index, 1);

		rb2 = new JRadioButton(ds.new RBGeometryAction(name, PivotDataStorage.AttributeType.STATISTIC, cb, table2));
		rb2.setToolTipText(geometryToolTipText);
		rb2.setSelected(ds.hasGeometryAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));
		attributeRBGroup.add(rb2);
		table2.setValueAt(rb2, index, 2);
   
		index ++;
	    }
	}
	
	JButton validateButton = new JButton(i18n.get(this, "validate", "Validate"));
	// should be active only if at least a criteria is chosen -- TBD.
	validateButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    aggDS.initAttributesListsFromHashMap();
		    aggDS.initGeometryAttributesListsFromHashMap();
		    aggDS.initialize();
		}
	    });
	/*attributeAndValidatePanel*/
	JPanel validWrap = new JPanel();
	validWrap.add(validateButton);
        tablePanel.add(table.getTableHeader());
	tablePanel.add(/*new JScrollPane(*/table/*)*/);
        tablePanel.add(Box.createRigidArea(new Dimension(200,20)));
        tablePanel.add(table2.getTableHeader());
	tablePanel.add(/*new JScrollPane(*/table2/*)*/);
	tablePanel.add(validWrap);
      
	Box panel = Box.createVerticalBox();
	panel.add(tablePanel);

	// add the datastorage controls
	panel.add(ds.getDataStorage().getGUI());

	table.setFillsViewportHeight(true);
	table.updateUI();
	table2.setFillsViewportHeight(true);
	table2.updateUI();
	//pivotPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "name", "Pivot")));
     /****************************************/
	gridPanel.add(panel);
	// wrap the panel in a flow layout panel in order to shrink the components
	JPanel wrapPanel = new JPanel();
	wrapPanel.add(gridPanel);
	wrapPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "name", "Aggregate")));
	add(wrapPanel);
   }

}
