// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.Contours;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class ContoursPanel extends JPanel {

    private Contours contours;
    private Box isoValuesPan;
    private JCheckBox checkLines;
    private JCheckBox checkPolys;
    private ArrayList<JSpinner> isoSpinners;
    protected I18n i18n = Environment.getI18n();

    public ContoursPanel(Contours contours) {
        this.contours = contours;
        isoSpinners = new ArrayList<JSpinner>();
        JComboBox comboColumns =
                new JComboBox(contours.getDataColumnNamesForTriangularGrid());
	if ("".equals(contours.getDataColumnName()))
	    comboColumns.setSelectedIndex(-1);
	else
	    comboColumns.setSelectedItem(contours.getDataColumnName());
        comboColumns.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
		Debug.message("Contours", ("setDataColumnName_012 "+((JComboBox)e.getSource()).getSelectedItem().toString()));
                ContoursPanel.this.contours.setDataColumnName_012(((JComboBox) e.getSource()).getSelectedItem().toString());
            }
        });
        JButton validateButton = new JButton(i18n.get(this, "validate", "Validate"));

        validateButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                updateValuesAndGUI();
                //ContoursPanel.this.contours.setWithLines(checkLines.isSelected());
                //ContoursPanel.this.contours.setWithPolygons(checkPolys.isSelected());
                ContoursPanel.this.contours.initialize();
            }
        });

        JButton addIsoValueButton = new JButton(i18n.get(this, "addIso", "+"));
        JButton rmIsoValueButton = new JButton(i18n.get(this, "rmIso", "-"));
        addIsoValueButton.setBackground(Color.gray);
        rmIsoValueButton.setBackground(Color.gray);
        addIsoValueButton.setOpaque(false);
        rmIsoValueButton.setOpaque(false);
        addIsoValueButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                addIsoSpinner();
            }
        });

        rmIsoValueButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                rmIsoSpinner();
            }
        });
        JPanel validatePanel = new JPanel();
        JPanel addIsoValuePanel = new JPanel();
        validatePanel.add(validateButton);
        addIsoValuePanel.add(addIsoValueButton);
        addIsoValuePanel.add(rmIsoValueButton);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        JPanel wrapCombo = new JPanel();
        wrapCombo.add(comboColumns);
        add(wrapCombo);


        JPanel linesPanel = new JPanel();
        JPanel polysPanel = new JPanel();
        checkLines = new JCheckBox(i18n.get(this, "withlines","with lines"));
        JTextField fieldLinesName = new JTextField("", 15);
        checkPolys = new JCheckBox(i18n.get(this, "withpolys","with polygons"));
        JTextField fieldPolysName = new JTextField("", 15);
        linesPanel.add(checkLines);
        JLabel linesLabName = new JLabel(i18n.get(this, "linename","Lines name:"));
        linesPanel.add(linesLabName);
        linesPanel.add(fieldLinesName);
        polysPanel.add(checkPolys);
        JLabel polysLabName = new JLabel(i18n.get(this, "polyname","Polygons name:"));
        polysPanel.add(polysLabName);
        polysPanel.add(fieldPolysName);
        add(linesPanel);
        add(polysPanel);

        checkLines.setSelected(contours.isWithLines());
        checkPolys.setSelected(contours.isWithPolygons());
        fieldLinesName.setText(contours.getContourColumnName());
        fieldPolysName.setText(contours.getIsoPolyColumnName());

        Dimension dim = new Dimension(120, 25);
        ComponentSizer.fixComponentSize(checkLines, dim);
        ComponentSizer.fixComponentSize(checkPolys, dim);
        ComponentSizer.fixComponentSize(linesLabName, dim);
        ComponentSizer.fixComponentSize(polysLabName, dim);


        isoValuesPan = Box.createVerticalBox();
        add(isoValuesPan);
        for(double d : contours.getContoursValues()){
            JSpinner sp = new JSpinner(new SpinnerNumberModel(
                d,
                Double.MAX_VALUE * -1,
                Double.MAX_VALUE,
                1.0));
            ComponentSizer.fixComponentSize(sp, new Dimension(120, 25));
            JPanel wrapSpinner = new JPanel();
            wrapSpinner.add(sp);
            //wrapSpinner.add();
            isoValuesPan.add(wrapSpinner);
            isoSpinners.add(sp);
        }
        JPanel buttonsPanel = new JPanel();
        //wrapCombo.add(addIsoValuePanel);
        //buttonsPanel.add(addIsoValuePanel);
        buttonsPanel.add(validatePanel);
        add(addIsoValuePanel);
        add(buttonsPanel);
        add(contours.getDataStorage().getGUI());
        setBorder(BorderFactory.createTitledBorder(i18n.get(this, "contours", "Contours")));

        checkLines.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent arg0) {
                ContoursPanel.this.contours.setWithLines(checkLines.isSelected());
            }
        });
        checkPolys.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent arg0) {
                ContoursPanel.this.contours.setWithPolygons(checkPolys.isSelected());
            }
        });
    }

    private void addIsoSpinner() {
        double step = 1.0;
        double val = 1.0;
        double min = Double.MAX_VALUE * -1.0;
        double max = Double.MAX_VALUE;
        JSpinner spinner = new JSpinner(new SpinnerNumberModel(
                val,
                min,
                max,
                step));

        isoSpinners.add(spinner);
        contours.getContoursValues().add(val);
        ComponentSizer.fixComponentSize(spinner, new Dimension(120, 25));
        JPanel wrapSpinner = new JPanel();
        wrapSpinner.add(spinner);
        isoValuesPan.add(wrapSpinner);
        updateUI();

    }

    private void rmIsoSpinner() {
        isoSpinners.remove(isoSpinners.get(isoSpinners.size() - 1)); // remove the last spinner
        updateValuesAndGUI();
    }
    /**
     * sorts spinners values
     */
    private void updateValuesAndGUI(){
        Collections.sort(isoSpinners, new Comparator<JSpinner>() {
		public int compare(JSpinner sp1, JSpinner sp2) {
                double d1 = Double.parseDouble(sp1.getValue().toString());
                double d2 = Double.parseDouble(sp2.getValue().toString());
                return Util.compare(d1, d2);
            }
        });
        isoValuesPan.removeAll();
        updateUI();
	contours.getContoursValues().clear();
        int i = 0;
        for(JSpinner sp : isoSpinners){
            JPanel wrapSpinner = new JPanel();
            wrapSpinner.add(sp);
            isoValuesPan.add(wrapSpinner);
            double d = Double.parseDouble(sp.getValue().toString());
            contours.getContoursValues().add(d);
            i++;
        }
        updateUI();
    }
}
