// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * MultiDataTableModel.java
 *
 * Created on 20 march 2006
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.util.Debug;
import javax.swing.table.AbstractTableModel;
import java.util.*;
import fr.unicaen.openmap.dataStorage.*;

/**
 * Link between graphical table display and the 'real' data
 * for multi DataStorage : DualDataStorage, ...
 * @author Jacques.Madelaine@info.unicaen.fr
 */
public class MultiDataTableModel extends DataTableModel {
    
    DataStorage ds;
    String[] dataColumnNames;
    String[] omColumnNames;

    public MultiDataTableModel(DataStorage ds, String[] dataColumnNames, String [] omColumnNames) { 
	super(null) ; // we override all methods that use the table attribute
	this.ds = ds; 
	this.dataColumnNames = dataColumnNames;
	this.omColumnNames = omColumnNames;
	Debug.message("DataStorage", "new MultiDataTableModel(ds, "+Arrays.asList(dataColumnNames)+", "+Arrays.asList(omColumnNames)+")");
    }

    // implementation of non editable model
    public int getRowCount() { return ds.getCount(); }
    public int getColumnCount() { return dataColumnNames.length; }

    
    public Object getValueAt(int row, int col) {
	DataObject dataObject = ds.getElementAt(row);
	// System.out.println("=== MultiDataTableModel.getValueAt("+row+", "+col+"): "+dataObject+"["+dataColumnNames[col]+"] => "+dataObject.getData(dataColumnNames[col]));
	return dataObject.getData(dataColumnNames[col]);
    }  
    public String getColumnName(int column) { return dataColumnNames[column]; }  
    public String getRealColumnName(int column) { return dataColumnNames[column]; }  
     
    public boolean isCellEditable(int row, int col) { 
      return false; 
    }

    public Class getColumnClass(int c) { 
	try { return getValueAt(0,c).getClass(); } 
	catch(Exception e) {} 
	return "".getClass(); 
    }


    public int getOMColumnCount() { return omColumnNames.length; }
    public String getOMColumnName(int c){ return omColumnNames[c]; }
    public String getRealOMColumnName(int c){ return omColumnNames[c]; }

    public DataObject getDataObjectAt(int row){
        return ds.getElementAt(row);
    }
}
