// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kernel;


import java.util.ArrayList;
import fr.unicaen.openmap.dataStorage.BoundingBox;

/**
 * See http://en.wikipedia.org/wiki/Kernel_(statistics)
 */
public abstract class Kernel {

    protected double radius;

    public Kernel() {
	this(1);
    }

    public Kernel(double radius) {
	this.radius = radius;
    }

    public void setRadius(double r) { radius = r; }
    public double getRadius() { return radius; }

    public abstract double k(double d);

    /**
     * The kernel may prepare the data used by getSurroundingCellIndices.
     * The BoundingBox is only used by Gaussian.
     */
    public abstract void prepare(float longitudeResolution, float latitudeResolution, BoundingBox bb);

    /** Method to find the cells that intersects the circle defined by a center given cell offsets.
     *
     * @param lon the center longitude
     * @param lat the center latitude
     */
    public abstract ArrayList<Integer> getSurroundingCellIndices(int lon, int lat);
}
