// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.db;

import java.util.ArrayList;

/**
 * This class is used within the package to group all database
 * connection information.
 */
public class DBParams {

    //----------------------------------------------------------------------
    // Private data
    //----------------------------------------------------------------------    
    private String server = "localhost";
    private int port = 5432;
    private String database = "esri";
    private String login = "postgres";
    private String password = "";
    private ArrayList<DBListener> dbListeners = new ArrayList<DBListener>();

    //----------------------------------------------------------------------
    // Data access methods
    //----------------------------------------------------------------------
    public void setServer(String server) {
        this.server = server;
    }

    public String getServer() {
        return server;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public void setPort(String port) {
        try {
            this.port = Integer.valueOf(port).intValue();
        } catch (NumberFormatException e) {
        }
    }

    public String getPort() {
        return "" + port;
    }

    public void setDatabase(String database) {
        this.database = database;
    }

    public String getDatabase() {
        return database;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getLogin() {
        return login;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getPassword() {
        return password;
    }

    /**
     * Constructor.
     */
    public DBParams(String server, int port, String database, String login, String password) {
        this.server = server;
        this.port = port;
        this.database = database;
        this.login = login;
        this.password = password;
    }

    /**
     * Display a nice object.
     */
    public String toString() {
        return login + "@" + server + ":" + port + "/" + database;
    }

    public void addDBListerner(DBListener dbl) {
        if (!dbListeners.contains(dbl)) {
            dbListeners.add(dbl);
        }
    }

    public void fireDBParamsChanged() {
        for (DBListener dbl : dbListeners) {
            dbl.dbParamsChanged();
        }
    }
}
