// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import javax.swing.JOptionPane;

import com.bbn.openmap.Layer;
import com.bbn.openmap.PropertyConsumer;
import com.bbn.openmap.plugin.PlugIn;
import com.bbn.openmap.plugin.PlugInLayer;
import com.bbn.openmap.util.ComponentFactory;
import fr.unicaen.openmap.LayerHandler;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.util.TooManyLegendException;
import fr.unicaen.openmap.util.propertyEditor.Inspector;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

public class LayerAddPanel extends com.bbn.openmap.gui.LayerAddPanel {

    String thematicClass = "fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer";

    /**
     * Creates the LayerPanel.
     */
    public LayerAddPanel() {
        super();
        inspector = new Inspector();
        inspector.addActionListener((ActionListener) this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public void createPanel() {
        removeAll();

        JButton configureButton = new JButton(i18n.get(LayerAddPanel.class, "configureButton", "Configure"));
        configureButton.addActionListener(this);
        configureButton.setActionCommand(configureActionCommand);

        String defaultLayerName = i18n.get(LayerAddPanel.class, "defaultLayerName", DefaultLayerName);
        prefixTextField = new JTextField(defaultLayerName, 12);

        Object[] layerTypes = getLayerClasses().keySet().toArray();

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.insets = new Insets(10, 10, 5, 10);

        if (layerTypes.length == 0) {
            String message = i18n.get(LayerAddPanel.class,
                    "noLayersAvailableMessage",
                    "No Layers available for creation.");
            JLabel label = new JLabel(message);
            gridbag.setConstraints(label, c);
            add(label);
        } else {
            list = new JComboBox(/*layerTypes*/);
            ArrayList<String> array = new ArrayList<String>(getLayerClasses().keySet());
            //Collections.sort(array);
            String thema = null;
            for (String s : array) {
                String layerClass = (String) layerClasses.get(s);
                if (layerClass.equals(thematicClass)) {
                    thema = s;
                }
            }
            if (thema != null) {
                array.remove(thema);
                array.add(0, thema);
            }

            for (String s : array) {
                list.addItem(s);
            }

            gridbag.setConstraints(list, c);
            add(list);
            c.insets = new Insets(5, 10, 10, 10);
            c.gridwidth = GridBagConstraints.RELATIVE;
            gridbag.setConstraints(prefixTextField, c);
            add(prefixTextField);

            c.weightx = 0;
            gridbag.setConstraints(configureButton, c);
            add(configureButton);
        }
        invalidate();


        /******************************/
        /*list.removeAllItems();
        ArrayList<String> array = new ArrayList<String>(getLayerClasses().keySet());
        //Collections.sort(array);
        String thema = null;
        for (String s : array) {
        String layerClass = (String) layerClasses.get(s);
        if (layerClass.equals(thematicClass)) {
        thema = s;
        }
        }
        if (thema != null) {
        array.remove(thema);
        array.add(0, thema);
        }

        for (String s : array) {
        list.addItem(s);
        }*/
    }

    @Override
    /**
     * Method associated with the ActionListener interface.
     */
    public void actionPerformed(ActionEvent e) {

        if (e.getActionCommand() == configureActionCommand) {
            // instanciate a default instance of the chosen layer
            // and bring up the Inspector to configure it
            String prettyName = (String) list.getSelectedItem();
            String prefix = prefixTextField.getText().trim();

            if (prettyName == null) {
                return;
            }

            String newClassName = (String) layerClasses.get(prettyName);

            layer = ComponentFactory.create(newClassName);
            if (layerHandler instanceof fr.unicaen.openmap.LayerHandler) {
                LayerHandler lh = (fr.unicaen.openmap.LayerHandler) layerHandler;
                if (layer instanceof LegendLayer && lh.isLegended()) {
                    JOptionPane.showMessageDialog(this,
                            i18n.get(TooManyLegendException.class, "message", "A legend already exists"),
                            "",
                            JOptionPane.ERROR_MESSAGE);
                    return;
                }
            }

            if (layer instanceof PropertyConsumer) {

                if (layer instanceof PlugIn) {
                    PlugInLayer pil = new PlugInLayer();
                    pil.setPlugIn((PlugIn) layer);
                    pil.setName(prefix);
                    layer = pil;
                }

                // Set the prefix to a modified version of the pretty
                // name.
                prefix = propertyHandler.getUniquePrefix(prefix);
                if (layer instanceof Layer) {
                    // Set the pretty name to what the user chose.
                    ((Layer) layer).setName(prefix);
                }
                ((PropertyConsumer) layer).setPropertyPrefix(prefix);

                inspector.inspectPropertyConsumer((PropertyConsumer) layer);
                windowSupport.killWindow();

            }
        } else if (e.getActionCommand() == Inspector.doneCommand) {
            // the confirmation button of the Inspector panel was
            // pressed
            // find the beancontext and add the layer at hand (var.
            // layer)

            if (layer != null && layerHandler != null) {
                if (layer instanceof Layer) {
                    // Let's add it on top, so the user can find it
                    // easier, instead of adding it to the bottom and
                    // having it lost behind some other layers.
                    layerHandler.addLayer((Layer) layer, 0);

                } else if (layer instanceof PlugIn) {
                    PlugInLayer pil = (PlugInLayer) ((PlugIn) layer).getComponent();
                    layerHandler.addLayer(pil, 0);
                }
                prefixTextField.setText(DefaultLayerName);
            } else if (layerHandler != null) {
                String message = i18n.get(LayerAddPanel.class,
                        "noLayerHandlerMessage",
                        "Layer Handler not found.\nCan't find anything to add the layer to.");
                JOptionPane.showMessageDialog(this, message);
            } else {
                String message = i18n.get(LayerAddPanel.class,
                        "noLayerCreatedMessage",
                        "No Layer instantiated.");
                JOptionPane.showMessageDialog(this, message);
            }
        } else if (e.getActionCommand() == Inspector.cancelCommand) {
            if (layer != null && propertyHandler != null) {
                propertyHandler.removeUsedPrefix(((PropertyConsumer) layer).getPropertyPrefix());
            }
        } else {
            showPanel();
        }
    }

    /**
     * check if the new LegendLayer can be added to the HandlerLayer
     * this method should be owned by our LayerHandler
     * @param layer
     * @return
     */
    private boolean canAddLegendLayer() {
        for (Layer l : layerHandler.getLayers()) {
            if (l instanceof LegendLayer) {
                return false;
            }
        }
        return true;
    }
}
