// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.io.IOException;
import java.util.Properties;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToggleButton;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MouseDelegator;
import com.bbn.openmap.PropertyHandler;
import com.bbn.openmap.event.MapMouseMode;
import com.bbn.openmap.gui.OMToolComponent;
import com.bbn.openmap.util.ComponentFactory;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.layer.utils.LayerLoader;
import fr.unicaen.openmap.util.TooManyLegendException;
import fr.unicaen.openmap.util.Util;
import javax.swing.JOptionPane;

/**
 * This class allows to have shortcuts buttons installed in the toolbar
 * for loadind a given layer from a properties file.
 * You may specify an icon. The icon file can be in one directory of the CLASSPATH.
 * <p>Warning: in order to have a nice look of the toolbar, please use an icon of size 20x20
 * <p>
 * In order to have these shortcut buttons in your appli, 
 * you must specify it in your properties file. By ex.:
 * <pre>
 * shortcut.class = fr.unicaen.openmap.gui.LayerShortCutTool
 * shortcut.properties = exemple.properties
 * shortcut.layer = graticule
 * shortcut.icon = graticule_shortcut.png
 * shortcut.tootlip = Add a graticule layer
 * </pre>
 * and add the name <code>shorcut</code> to the list of components 
 * in the property <code>openmap.components</code>  
 */
public class LayerShortCutTool extends OMToolComponent{

	protected JButton bShortCut;
	
	protected LayerHandler layerHandler;
	
	protected String propertiesPath;
	
	protected Layer layer = null;
	
	protected String layerName;
	
	protected String iconName;
	
	protected String tooltip;
	
    public  LayerShortCutTool() {
        super();
      
       // setKey("Graticule");
       // bGraticule = new JToggleButton(Util.loadIcon("grid.png"));
       // bGraticule.setToolTipText(i18n.get(this, "graticuleadd", "Add a graticule layer"));
       // add(bGraticule);
       // useAsTool = true;
       /* bGraticule.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				if(bGraticule.isSelected()){
					Object o = ComponentFactory.create("com.bbn.openmap.layer.GraticuleLayer");
					layer = (Layer)o;
					layer.setName("Graticule layer");
					layerHandler.addLayer(layer);
					bGraticule.setToolTipText(i18n.get(GraticuleTool.this, "graticulerm", "Remove the graticule layer"));
				}else{
					layerHandler.removeLayer(layer);
					bGraticule.setToolTipText(i18n.get(GraticuleTool.this, "graticuleadd", "Add a graticule layer"));
				}
			}});*/
        
    }
    
    public Container getFace() {
        if (getUseAsTool()) {
            return bShortCut;
        } else {
            return null;
        }
    }
    
    public boolean getUseAsTool(){return true;}
    
    /**
     * Looks for LayerHandler.
     */
    public void findAndInit(Object someObj) {

        if (someObj instanceof LayerHandler) {
           layerHandler = (LayerHandler) someObj;
     	
        }
    }
    
    public static final String iconPropertyString = "icon";
    public static final String layerNamePropertyString = "layer";
    public static final String propertiesPropertyString = "properties";
    public static final String tooltipPropertyString ="tooltip";
    
    public void setProperties(String prefix, Properties props) {
        super.setProperties(prefix, props);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        layerName = props.getProperty(prefix
				+ layerNamePropertyString, "");
        iconName = props.getProperty(prefix
				+ iconPropertyString, "");
        propertiesPath =  props.getProperty(prefix
				+ propertiesPropertyString, "");
        tooltip = props.getProperty(prefix
				+ tooltipPropertyString, "");
        setPanel();

        
    }

    public Properties getProperties(Properties props) {
        props = super.getProperties(props);
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        /*props.put(prefix + UseAsToolProperty, new Boolean(useAsTool).toString());*/
        props.put(prefix + layerNamePropertyString, layerName);
        props.put(prefix + iconPropertyString, iconName);
        props.put(prefix + propertiesPropertyString, propertiesPath);
        props.put(prefix + tooltipPropertyString, propertiesPath);
        
        return props;
    }
    
    /**
     *  Listen for changes to the active mouse mode and for any changes
     *  to the list of available mouse modes
     */
    public void propertyChange(PropertyChangeEvent evt) {
        setPanel();
    }
    
    
    protected void setPanel(){
    	bShortCut = new JButton(Util.loadIcon(iconName));
    	bShortCut.setToolTipText(tooltip);
    	bShortCut.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				try{
					LayerLoader loader = 
						new LayerLoader(Util.getURLforFileName(propertiesPath), layerHandler);
					loader.initLayer(layerName);
				}
				catch(TooManyLegendException e) {
                                    JOptionPane.showMessageDialog(LayerShortCutTool.this, e.getMessage(), "", JOptionPane.ERROR_MESSAGE);
                                }
			}});
    }
}
