// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import com.bbn.openmap.proj.ProjectionStack;

import fr.unicaen.openmap.util.Util;

public class ProjectionStackTool extends com.bbn.openmap.gui.ProjectionStackTool{
	
    protected static transient String backName = "backproj.png";
    protected static transient String forwardName = "forwardproj.png";
    protected static transient String dimBackName = "dimbackproj.png";
    protected static transient String dimForwardName = "dimforwardproj.png";

    protected boolean dimBackButton = true;
    protected boolean dimForwardButton = true;

    ImageIcon backIcon;
    ImageIcon dimBackIcon;
    ImageIcon forwardIcon;
    ImageIcon dimForwardIcon;

	public ProjectionStackTool(){
		super();		
		backButton.setBackground(Color.WHITE);
		forwardButton.setBackground(Color.WHITE);
		backButton.setOpaque(false);
		forwardButton.setOpaque(false);
	}
	
    public void resetButtons(boolean enableBackButton,
                             boolean enableForwardButton) {

        if (backIcon == null) {
           backIcon = Util.loadIcon(backName);
        }

        if (dimBackIcon == null) {
            dimBackIcon = Util.loadIcon(dimBackName);
        }

        if (forwardIcon == null) {
            forwardIcon = Util.loadIcon(forwardName);
        }

        if (dimForwardIcon == null) {
            dimForwardIcon = Util.loadIcon(dimForwardName);
        }

        ImageIcon active;
        String toolTip;
        String disabled = " ("
                + i18n.get(com.bbn.openmap.gui.ProjectionStackTool.class, "disabled", "disabled")
                + ")";
        int size;

        toolTip = i18n.get(com.bbn.openmap.gui.ProjectionStackTool.class,
                "backTip",
                "Go back to previous projection");
        if (enableBackButton) {
            active = backIcon;
        } else {
            active = dimBackIcon;
            toolTip += " " + disabled;
        }


        if (backButton == null) {
            backButton = new JButton(Util.loadIcon("backproj.png"));
            backButton.setMargin(new Insets(0, 0, 0, 0));
            backButton.setBorderPainted(false);
            backButton.setActionCommand(ProjectionStack.BackProjCmd);
            if (listeners != null) {
                size = listeners.size();
                for (int i = 0; i < size; i++) {
                    backButton.addActionListener((ActionListener) listeners.elementAt(i));

                }
            }
        } 
        else {
            backButton.setIcon(active);
        }
        backButton.setToolTipText(toolTip);

        toolTip = i18n.get(com.bbn.openmap.gui.ProjectionStackTool.class,
                "forwardTip",
                "Go forward to next projection");
        if (enableForwardButton) {
            active = forwardIcon;
        } else {
            active = dimForwardIcon;
            toolTip += " " + disabled;
        }

        if (forwardButton == null) {
            forwardButton = new JButton(Util.loadIcon("forwardproj.png"));
            forwardButton.setMargin(new Insets(0, 0, 0, 0));
            forwardButton.setBorderPainted(false);
            forwardButton.setActionCommand(ProjectionStack.ForwardProjCmd);

            if (listeners != null) {
                size = listeners.size();
                for (int i = 0; i < size; i++) {
                    forwardButton.addActionListener((ActionListener) listeners.elementAt(i));

                }
            }
        } else {
            forwardButton.setIcon(active);
        }
        forwardButton.setToolTipText(toolTip);

    }
}
