// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Layer;
import java.awt.Point;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.bbn.openmap.LightMapHandlerChild;
import com.bbn.openmap.PropertyConsumer;

import com.bbn.openmap.util.ComponentFactory;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.gui.dnd.DndTransferHandler;
import fr.unicaen.openmap.gui.dnd.DropArea;
import fr.unicaen.openmap.gui.dnd.MovableBox;
import fr.unicaen.openmap.gui.dnd.MovableBoxesContainer;
import fr.unicaen.openmap.gui.dnd.VerticalDropArea;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.beans.PropertyChangeListener;

/**
 * Manage themamap panels
 * @author jchauvea
 *
 */
public class ThemaMapPanelsManager extends com.bbn.openmap.gui.OMControlPanel implements PropertyChangeListener {

    protected JComponent layerGUIPanel, panTitle;
    protected JFrame externalFrame;
    protected DndTransferHandler transferHandler;
    protected LayersPanelTree layersPanel;
    protected boolean showLayersPanel;
    protected boolean showNavigationPanel;
    protected boolean showConfigurationPanel;
    protected LayersBox layersBox;
    protected NavigationBox navigationBox;
    protected ConfigurationBox configBox;
    protected MovableBoxesContainer westBox, eastBox, southBox;
    protected JPanel configPanel;
    protected boolean externMap, scrollableMap;
    protected String atlasPath;

    public enum themamap_mode {

        CLASSIC, ATLAS
    };
    public static themamap_mode MODE;
    public static boolean BOXES_ON_TOP;
    LinkedList<JComponent> children = new LinkedList<JComponent>();

    public ThemaMapPanelsManager() {
        removeAll();
        westBox = new MovableBoxesContainer(BoxLayout.Y_AXIS);
        eastBox = new MovableBoxesContainer(BoxLayout.Y_AXIS);

        southBox = new MovableBoxesContainer(BoxLayout.X_AXIS);

        transferHandler = ThemaMapPanel.transferHandler;

    }

    public void setLayerGUIPanel(JComponent comp) {
        layerGUIPanel = comp;
        if (layersBox != null) {
            layersBox.getLayersPanel().setLayerGUIPanel(comp);
        }

    }

    private Object[] buidObjectsFromProperties(Vector items, Properties props) {
        Object[] objs = new Object[items.size()];
        if (!items.isEmpty()) {
            for (int i = 0; i < items.size(); i++) {
                String itemPrefix = (String) items.elementAt(i);
                Object obj = buildObjFromProperties(props, itemPrefix);
                objs[i] = obj;
                if (obj instanceof LayersBox) {
                    layersBox = (LayersBox) obj;
                    layersPanel = layersBox.layersPanel;
                    /*layersPanel.addPropertyChangeListener(LayersPanel.LayerSelectedCmd, this);
                    layersPanel.addPropertyChangeListener(LayersPanel.ThematicSelectedCmd, this);*/
                    layersPanel.addPropertyChangeListener(this);
                } else {
                    if (obj instanceof NavigationBox) {
                        navigationBox = (NavigationBox) obj;
                    } else {
                        if (obj instanceof ConfigurationBox) {
                            configBox = (ConfigurationBox) obj;
                            configPanel = configBox.getConfigPanel();
                        }
                    }
                }
            }
        }
        return objs;
    }

    private void addElementsToSideBox(MovableBoxesContainer sideBox, Object[] objs) {
        for (int i = 0; i < objs.length; i++) {
            try {
                sideBox.addElement((MovableBox) objs[i]);
            } catch (ClassCastException exc) {
            }
        }
    }

    private void externalizeBoxes(Object[] boxes, int layout) {
        for (int i = 0; i < boxes.length; i++) {
            try {
                MovableBox box = (MovableBox) boxes[i];
                box.externalize(new Point(0, 0), layout);
            } catch (ClassCastException exc) {
            }
        }
    }

    private Object buildObjFromProperties(Properties props, String item) {
        String classProperty = item + ".class";
        String className = props.getProperty(classProperty, defaultClassName(item));
        Object obj = ComponentFactory.create(className,
                item,
                props);
        return obj;

    }

    private String defaultClassName(String s) {
        if (s.equals("navigationbox")) {
            return NavigationBox.className;
        }
        if (s.equals("layersbox")) {
            return LayersBox.className;
        }
        if (s.equals("configbox")) {
            return ConfigurationBox.className;
        }
        return "";
    }

    private String boxesList(MovableBoxesContainer mbc) {
        ArrayList<String> list = mbc.getBoxesList();
        String itemList = "";
        for (String item : list) {
            itemList += " " + item;
        }
        return itemList;
    }

    private String externalBoxesList(int type) {
        String itemList = "";
        if (configBox.isExternalized() && configBox.getParentBoxContainer().getType() == type) {
            itemList += " configbox";
        }
        if (layersBox.isExternalized() && layersBox.getParentBoxContainer().getType() == type) {
            itemList += " layersbox";
        }
        if (navigationBox.isExternalized() && navigationBox.getParentBoxContainer().getType() == type) {
            itemList += " navigationbox";
        }
        return itemList;
    }

    private void setHiddens(Object[] boxes) {
        showConfigurationPanel = true;
        showLayersPanel = true;
        showNavigationPanel = true;
        for (int i = 0; i < boxes.length; i++) {
            if (boxes[i] instanceof LayersBox) {
                showLayersPanel = false;
                layersBox.setHidden(true);
            } else {
                if (boxes[i] instanceof NavigationBox) {
                    showNavigationPanel = false;
                    navigationBox.setHidden(true);
                } else {
                    if (boxes[i] instanceof ConfigurationBox) {
                        showConfigurationPanel = false;
                        configBox.setHidden(true);
                    }
                }
            }
        }
    }

    private String hiddenList() {
        String list = "";
        if (configBox.isHidden()) {
            list += "configurationbox";
        }
        if (navigationBox.isHidden()) {
            list += " navigationbox";
        }
        if (layersBox.isHidden()) {
            list += " layersbox";
        }
        return list;
    }
    /*PROPERTIES*/
    public static String showLayersPanelPropertyString = "showLayersPanel";
    public static String showNavigationPanelPropertyString = "showNavigationPanel";
    public static String showConfigurationPanelPropertyString = "showConfigurationPanel";
    public static String westPropertyString = "west";
    public static String southPropertyString = "south";
    public static String eastPropertyString = "east";
    public static String verticalExternPropertyString = "verticalexternpanels";
    public static String horizontalExternPropertyString = "horizontalexternpanels";
    public static String externMapPropertyString = "externmap";
    public static String scrollableMapPropertyString = "scrollablemap";
    public static String hiddenPropertyString = "hiddenpanels";
    public static String atlasPropertyString = "atlasmode";
    public static String BoxesOnTopPropertyString = "boxesOnTop";

    @Override
    public void setProperties(String prefix, Properties props) {

        super.setProperties(prefix, props);

        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        String pl = props.getProperty(prefix + PreferredLocationProperty);
        if (pl != null) {
            setPreferredLocation(pl);
        }

        externMap = Boolean.parseBoolean(props.getProperty(prefix + externMapPropertyString, "false"));
        scrollableMap = Boolean.parseBoolean(props.getProperty(prefix + scrollableMapPropertyString, "false"));

        //West panels
        Vector westItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + westPropertyString, "navigationbox layersbox"));
        //South panels
        Vector southItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + southPropertyString, "configbox"));
        //East panels
        Vector eastItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + eastPropertyString));
        //Vertical Extern panels
        Vector verticalItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + verticalExternPropertyString));
        //Vertical Extern panels
        Vector horizontalItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + horizontalExternPropertyString));
        //Hidden panels
        Vector hiddenItems = PropUtils.parseSpacedMarkers(props.getProperty(prefix + hiddenPropertyString));


        Object[] vexts = buidObjectsFromProperties(verticalItems, props);
        Object[] hexts = buidObjectsFromProperties(horizontalItems, props);
        Object[] hiddens = buidObjectsFromProperties(hiddenItems, props);
        setHiddens(hiddens);
        externalizeBoxes(vexts, BoxLayout.Y_AXIS);
        externalizeBoxes(hexts, BoxLayout.X_AXIS);
        //externalizeBoxes(hiddens,BoxLayout.Y_AXIS, false);
        //place panels
        westBox.addElement(new DropArea(transferHandler, westBox));
        addElementsToSideBox(westBox, buidObjectsFromProperties(westItems, props));
        westBox.addElement(new DropArea(transferHandler, westBox));
        southBox.addElement(new VerticalDropArea(transferHandler, southBox));
        addElementsToSideBox(southBox, buidObjectsFromProperties(southItems, props));
        southBox.addElement(new VerticalDropArea(transferHandler, southBox));
        eastBox.addElement(new DropArea(transferHandler, eastBox));
        addElementsToSideBox(eastBox, buidObjectsFromProperties(eastItems, props));
        eastBox.addElement(new DropArea(transferHandler, eastBox));

        if (Boolean.parseBoolean(props.getProperty(prefix + atlasPropertyString, "false"))) {
            MODE = themamap_mode.ATLAS;
        } else {
            MODE = themamap_mode.CLASSIC;
        }

        BOXES_ON_TOP = Boolean.parseBoolean(props.getProperty(prefix + BoxesOnTopPropertyString, "false"));

        //atlasPath = props.getProperty(atlasPathPropertyString, "atlas");

        if (navigationBox != null) {
            children.addAll(navigationBox.getChildren());
        }
        if (layersBox != null) {
            children.addAll(layersBox.getChildren());
        }


        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof PropertyConsumer) {
                String newPrefix = prefix + ((PropertyConsumer) obj).getPropertyPrefix();
                ((PropertyConsumer) obj).setProperties(newPrefix, props);
            }
        }



    }
    public static String classString = "class";

    @Override
    public Properties getProperties(Properties props) {
        props = super.getProperties(props);

        props.put(PropUtils.getScopedPropertyPrefix(this) +
                PreferredLocationProperty, getPreferredLocation());

        /*props.put(PropUtils.getScopedPropertyPrefix(this) + 
        showNavigationPanelPropertyString, String.valueOf(showNavigationPanel));
        props.put(PropUtils.getScopedPropertyPrefix(this) +
        showLayersPanelPropertyString, String.valueOf(showLayersPanel));
        props.put(PropUtils.getScopedPropertyPrefix(this) +
        showConfigurationPanelPropertyString, String.valueOf(showConfigurationPanel));*/
        props.put(PropUtils.getScopedPropertyPrefix(this) +
                externMapPropertyString, String.valueOf(externMap));

        props.put(PropUtils.getScopedPropertyPrefix(this) + hiddenPropertyString,
                hiddenList());

        props.put(PropUtils.getScopedPropertyPrefix(this) + westPropertyString,
                boxesList(westBox));
        props.put(PropUtils.getScopedPropertyPrefix(this) + southPropertyString,
                boxesList(southBox));
        props.put(PropUtils.getScopedPropertyPrefix(this) + eastPropertyString,
                boxesList(eastBox));

        props.put(PropUtils.getScopedPropertyPrefix(layersBox) + classString, layersBox.getClass().getName());
        props.put(PropUtils.getScopedPropertyPrefix(navigationBox) + classString, navigationBox.getClass().getName());
        props.put(PropUtils.getScopedPropertyPrefix(configBox) + classString, configBox.getClass().getName());
        props.put(PropUtils.getScopedPropertyPrefix(this) + verticalExternPropertyString,
                externalBoxesList(BoxLayout.Y_AXIS));
        props.put(PropUtils.getScopedPropertyPrefix(this) + horizontalExternPropertyString,
                externalBoxesList(BoxLayout.X_AXIS));
        props.put(PropUtils.getScopedPropertyPrefix(this) + BoxesOnTopPropertyString,
                String.valueOf(BOXES_ON_TOP));
        // props.put(atlasPathPropertyString, atlasPath);
        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof PropertyConsumer) {
                ((PropertyConsumer) obj).getProperties(props);
            }
        }
        return props;
    }

    public Properties getPropertyInfo(Properties props) {
        props = super.getPropertyInfo(props);

        props.put(PreferredLocationProperty, "The preferred BorderLayout direction to place this component.");

        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof PropertyConsumer) {
                ((PropertyConsumer) obj).getPropertyInfo(props);
            }
        }
        return props;
    }

    public void findAndInit(Object someObj) {
        if (someObj instanceof DisplayMenu) {
            DisplayMenu menu = (DisplayMenu) someObj;
            if (navigationBox != null) {
                navigationBox.setMenuItem(menu.getNavigationMenuItem());
                menu.getNavigationMenuItem().setBox(navigationBox);
            }
            if (layersBox != null) {
                layersBox.setMenuItem(menu.getLayersMenuItem());
                menu.getLayersMenuItem().setBox(layersBox);
            }
            if (configBox != null) {
                configBox.setMenuItem(menu.getConfigurationMenuItem());
                menu.getConfigurationMenuItem().setBox(configBox);
            }
            menu.getNavigationMenuItem().setCheckSelected(showNavigationPanel);
            menu.getLayersMenuItem().setCheckSelected(showLayersPanel);
            menu.getConfigurationMenuItem().setCheckSelected(showConfigurationPanel);
            return;

        }
        /*if(someObj instanceof LayerControlButtonPanel){
        ((LayerControlButtonPanel)someObj).
        addPropertyChangeListener(LayersPanel.LayerRemoveCmd,this);
        System.out.println("**************************************************propertychange");
        return;
        }*/
        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof LightMapHandlerChild) {
                ((LightMapHandlerChild) obj).findAndInit(someObj);
            }
        }
    }

    public void findAndUndo(Object someObj) {

        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof LightMapHandlerChild) {
                ((LightMapHandlerChild) obj).findAndUndo(someObj);
            }
        }
    }
    /*public void freeze(){
    westBox.freeze();
    southBox.freeze();
    eastBox.freeze();
    }
    
    public void unFreeze(){
    westBox.unFreeze();
    southBox.unFreeze();
    eastBox.unFreeze();
    }*/
    /**
     * BorderLayout.WEST by default for this class.
     */
    protected String preferredLocation = java.awt.BorderLayout.EAST;

    /**
     * MapPanelChild method.
     */
    public void setPreferredLocation(String value) {
        preferredLocation = value;
    }

    /** 
     * MapPanelChild method. 
     */
    public String getPreferredLocation() {
        return preferredLocation;
    }

    public LayersPanelTree getLayersPanel() {
        return layersPanel;
    }

    public boolean isShowLayersPanel() {
        return showLayersPanel;
    }

    public void setShowLayersPanel(boolean showLayersPanel) {
        this.showLayersPanel = showLayersPanel;
    }

    public boolean isShowNavigationPanel() {
        return showNavigationPanel;
    }

    public void setShowNavigationPanel(boolean showNavigationPanel) {
        this.showNavigationPanel = showNavigationPanel;
    }

    public MovableBox getLayersBox() {
        return layersBox;
    }

    /*public void setLayersBox(MovableBox layersBox) {
    this.layersBox = layersBox;
    }*/
    public MovableBox getNavigationBox() {
        return navigationBox;
    }

    public boolean isShowConfigurationPanel() {
        return showConfigurationPanel;
    }

    public void setShowConfigurationPanel(boolean showConfigurationPanel) {
        this.showConfigurationPanel = showConfigurationPanel;
    }

    public MovableBoxesContainer getWestBox() {
        return westBox;
    }

    public void setWestBox(MovableBoxesContainer layersNavigationBox) {
        this.westBox = layersNavigationBox;
    }

    public MovableBoxesContainer getEastBox() {
        return eastBox;
    }

    public void setEastBox(MovableBoxesContainer eastBox) {
        this.eastBox = eastBox;
    }

    public MovableBoxesContainer getSouthBox() {
        return southBox;
    }

    public void setSouthBox(MovableBoxesContainer southBox) {
        this.southBox = southBox;
    }

    public JPanel getConfigPanel() {
        return configPanel;
    }

    public MovableBox getConfigBox() {
        return configBox;
    }

    public boolean isExternMap() {
        return externMap;
    }

    public void setExternMap(boolean externMap) {
        this.externMap = externMap;
    }

    public boolean isScrollableMap() {
        return scrollableMap;
    }

    public void setScrollableMap(boolean scrollableMap) {
        this.scrollableMap = scrollableMap;
    }

    public void removeConfigurationGUI() {
        configBox.removeConfigurationGUI();
        configBox.updateUI();
    }

    public void propertyChange(PropertyChangeEvent pce) {
        String command = pce.getPropertyName();
        Object obj = pce.getNewValue();
        if (command == LayersPanelTree.ThematicSelectedCmd &&
                (obj instanceof Thematic)) {
            Thematic tv = (Thematic) obj;
            configBox.updateNames(tv.getLayer().getName() + " > " + tv.getName());
            return;
        }
        if (command == LayersPanelTree.LayerSelectedCmd &&
                (obj instanceof Layer)) {
            configBox.updateNames(((Layer) obj).getName());
            return;
        }
        if (command == com.bbn.openmap.gui.LayersPanel.LayerRemoveCmd) {
            removeConfigurationGUI();
            configBox.updateNames("");

        }
    }

}
