// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * InteractiveDataDisplayLayer.java
 *
 * Created on 7 octobre 2005, 09:54
 *
 * @author Christophe.Turbout@info.unicaen.fr
 */

package fr.unicaen.openmap.layer.dataDisplayLayer;
import com.bbn.openmap.*;
import com.bbn.openmap.event.*;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.*;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.dataStorage.gui.*;
import fr.unicaen.openmap.layer.dataDisplayLayer.gui.*;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.awt.event.*;
import java.beans.beancontext.BeanContext;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;


/**
 *
 * @author Christophe.Turbout@info.unicaen.fr
 */
public class InteractiveDataDisplayLayer extends GeometryDataDisplayLayer implements MapMouseListener, ActionListener, DocumentListener, KeyListener {
    /* --------------------------  Begin   Properties  ---------------------*/
    public static final String quickLocationModeString = "quickLocationMode";
    public static final String quickLocationColumnString ="quickLocationColumn";
    public static final String drawToolTipsPropertyString = "drawToolTips";
    public static final String toolTipsColumnNamePropertyString = "toolTipsColumnName";
/*    public static final String scalePropertyString = "scale";
    public static final String latitudePropertyString = "latitude";
    public static final String longitudePropertyString = "longitude";*/
    
    // yes => draw all OMGraphic Names
    // no => nothing to draw
    // selected => only Selected OMGraphics
    protected boolean quickLocationMode = false;
    protected String quickLocationColumn = null;
    protected String toolTipsColumnName = null;
    protected boolean drawToolTipsMode=false;
    // default selection
/*    protected double scale=8000000f;
    protected double longitude = 2.6f;
    protected double latitude = 46.6f;*/
  

    
    protected boolean initializedScale = false;
    /* --------------------------  End Properties  ---------------------*/
    
    /* --------------------------  Begin   Variables  ---------------------*/
    private int debX=0,debY=0,finX=0,finY=0;
    private int tipsX=0,tipsY = 0;
    public int popupX=0,popupY=0;
    private boolean drawArea = false;
    public boolean changeZonePopup = false;
    private OMGraphic current_omgr = null;
    private JPopupMenu popup = new JPopupMenu();
    private JMenuItem detail = new JMenuItem();
    private OMCircle circle = null;
    private OMText selectionText = null;
    private OMText tipsText = null;
    private boolean initialized=false;
    // used when we use ResetMap action.
    private boolean nothingToDraw=false;
    
    // specific gui to configure layer
    private JButton centerAndZoomButton = null;
    private JButton resetMapButton = null;
    private JButton selectAllButton = null;
    private JButton deselectAllButton = null;
    private JButton redrawSelectionOnMapButton = null;
    private JButton addSelectionButton = null;
    private JButton addSubSelectionButton = null;
    private JButton removeSelectionButton = null;
    private JButton removeSubSelectionButton = null;
    private JTextField textField = null;
    // protected JRadioButton drawingSelectedButton = null;
    
    /* --------------------------  End Variables      ---------------------*/
    
    
    /** Creates a new instance of InteractiveDataDisplayLayer */
    public InteractiveDataDisplayLayer() {
        Debug.message(Util.className(this), this + " new InteractiveDataDisplayLayer()");
        selectionText = new OMText(0,0,"",OMText.JUSTIFY_CENTER);
        tipsText = new OMText(0,0,"",OMText.JUSTIFY_CENTER);
        circle = new OMCircle(0,0,1,1);
    }
    
    /**
     * Let's find our properties.
     * @param prefix string prefix used in the properties file for this layer
     * @param properties the properties set in the properties file
     */
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        Debug.message(Util.className(this), this + " setProperties() : InteractiveDataDisplayLayer");
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        quickLocationMode = Util.isTrue(properties.getProperty(prefix + quickLocationModeString, "false"));
        quickLocationColumn = properties.getProperty(prefix + quickLocationColumnString, "defaultColumn");
        drawToolTipsMode=Util.isTrue(properties.getProperty(prefix + drawToolTipsPropertyString, "false"));
        toolTipsColumnName = properties.getProperty(prefix + toolTipsColumnNamePropertyString, "defaultColumnName");
/*        scale = new Double(properties.getProperty(prefix + scalePropertyString, "8000000f"));
        longitude=new Double(properties.getProperty(prefix + longitudePropertyString, "2.6f"));
        latitude=new Double(properties.getProperty(prefix + latitudePropertyString, "46.6f"));*/
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        selectionDataStorage.addUnicaenListener(this);
        // showPalette();
        
    }
//--------------------------------------------------------------------------------------------------------------------NOT FINISHED
    /**
     * The properties and prefix are managed and decoded here, for the standard uses of GeometryDataDisplayLayer.
     * Notes this method allows to define what is present in file "save Map as properties file".
     * @param properties the properties set in the properties file
     * @return Properties.
     */
    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);
        Debug.message(Util.className(this), this + " getProperties InteractiveDataDisplayLayer()");
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        properties.put(prefix +  quickLocationModeString, new Boolean(quickLocationMode).toString());
        properties.put(prefix +  quickLocationColumnString, quickLocationColumn);
        properties.put(prefix +  drawToolTipsPropertyString, new Boolean(drawToolTipsMode).toString());
        properties.put(prefix +  toolTipsColumnNamePropertyString, toolTipsColumnName);
        
/* done in SimpleDataDisplayLayer, now        // get projection and center
        BeanContext mapHandler = getBeanContext();
        if (mapHandler==null)  Debug.error(Util.className(this)+" Error : mapHandler is null");
        Iterator it = mapHandler.iterator();
        Object someObj = null;
        MapBean map = null;
        MouseDelegator msd = null;
        while(it.hasNext()) {
            someObj = it.next();
            if (someObj instanceof MapBean) {
                map = (MapBean) someObj;
            } else if (someObj instanceof MouseDelegator){
                msd = (MouseDelegator) someObj;
            }
        }
        Projection projection = map.getProjection();
        Proj p = (Proj)projection;
        
        properties.put(prefix +  scalePropertyString, Float.toString(p.getScale()));
        properties.put(prefix +  longitudePropertyString, Float.toString(p.getCenter().getLongitude()));
        properties.put(prefix +  latitudePropertyString, Float.toString(p.getCenter().getLatitude()));*/
        return properties;
    }
//--------------------------------------------------------------------------------------------------------------------
    public int getQuickLocationColumn(){
        // we can have aliases in the column model so we have to check the name of the
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        DataTableModel model = (DataTableModel) selectionDataStorage.getDefaultDataTableModel();
        for (int i=0;i<model.getColumnCount();i++) if (quickLocationColumn.compareTo(model.getRealColumnName(i))==0) return i;
        return -1;
    }
    /**
     * Return the current LocationMode
     **/
    public boolean getQuickLocationMode(){ return quickLocationMode;}
    
// ------------------------------- Mouse Events ------------------------------------------
    
//----------------------------------------------------------------------
// MapMouseListener interface implementation
//----------------------------------------------------------------------
    
    /**
     * Note: A layer interested in receiving amouse events should
     * implement this function .  Otherwise, return the default, which
     * is null.
     */
    public synchronized MapMouseListener getMapMouseListener() {
        return this;
    }
    
    /**
     * Return a list of the modes that are interesting to the
     * MapMouseListener.
     * We can use mouse in Gestures mode to select OMGraphics.
     */
    public String[] getMouseModeServiceList() {
        String[] services = {"Gestures"};
        return services;
    }
    
    /**
     * Need to verify in which mode we are ...
     * Gesture Mode : set the position of the center of the selection circle.
     * Navigation Mode : nothing to do!
     */
    public boolean mousePressed(MouseEvent e) {
        // String mode = getCurrentMode();
        // Very strange if mode is null  ...
        // if (mode==null) return false;
        // normal navigation mode => nothing to do, return false;
        if (/* mode.equals("Gestures") && */ ((e.getModifiers() & InputEvent.BUTTON1_MASK)==InputEvent.BUTTON1_MASK || (e.getModifiers() & InputEvent.BUTTON3_MASK)==InputEvent.BUTTON3_MASK)){
            // Gesture mode
            debX=e.getX();
            debY=e.getY();
            return true;
        }
        return false;
    }
    
    /**
     * Find which OMGraphic is concerned by mouse clicked, and select
     * or deselect it in tableView. Works only in Gesture Mode.
     * @param e MouseEvent
     * @return true to disable mouseClicked on other layer ...
     *
     */
    public boolean mouseClicked(MouseEvent e) {
    /*    String mode = getCurrentMode();
        if (mode.equals("Gestures") && (e.getModifiers() & InputEvent.BUTTON1_MASK)==InputEvent.BUTTON1_MASK) {
            Projection projection = getProjection();
            if (projection.getScale()<1000000f) {
                popupX=e.getX();
                popupY=e.getY();
                changeZonePopup = true;                
                repaint();
            }            
        }                    */
        return true;
    }
    /**
     * Set the new circle in Gesture Mode.
     * @param e MouseEvent
     * @return true;
     */
    public boolean mouseDragged(MouseEvent e) {
	// Debug.message(Util.className(this), this + " : mouseDragged");
        // String mode = getCurrentMode();
        // Strange if this return is called ...
        // if (mode==null) return true;
        // nothing to do if not in gesture mode ...
        if (/* mode.equals("Gestures") && */ ((e.getModifiers() & InputEvent.BUTTON1_MASK)==InputEvent.BUTTON1_MASK || (e.getModifiers() & InputEvent.BUTTON3_MASK)==InputEvent.BUTTON3_MASK)) {
            finX=e.getX();
            finY=e.getY();
            drawArea=true;
            repaint();
        }
        return false;
    }
    
    /**
     * This method select all the OMGraphics mapped by a clip area
     *
     * ExpÃ©rimentations en cours sur cette mÃ©thode : les bugs de sÃ©lection y sont certainement liÃ©s.
     *
     * @param e MouseEvent
     * @return true in Gesture Mode false otherwise.
     */
    public boolean mouseReleased(MouseEvent e) {
        // String mode = getCurrentMode();
	// if (mode == null || mode != "Gesture") return false;

        Debug.message(Util.className(this), this + " : mouseReleased");
        // Strange if this return is called ...
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        if (selectionDataStorage==null) return true;
        // select OMGraphics in circle
        if ((e.getModifiers() & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK) {
	    Debug.message(Util.className(this), this + " : left button dragged");
            // we try to select omgraphics only in case of a dragged event
            // has been produced.
            OMGraphicList omgl = getList();
            if (omgl==null) return true;
            OMGraphic omgr = omgl.findClosest(e.getX(),e.getY());
            if (omgr==null) return true;
	    Debug.message(Util.className(this),this +" : select object");
            selectOMGraphic(omgr);
        }
        // deselect
        if ((e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
            Debug.message(Util.className(this),this +" : clicked with right button => deselect ");
            OMGraphicList omgl = getList();
            if (omgl==null) return true;
            OMGraphic omgr = omgl.findClosest(e.getX(),e.getY());
            if (omgr==null) return true;
	    Debug.message(Util.className(this),this +" : deselect object");
            deselectOMGraphic(omgr);
        }
        if (drawArea){
            drawArea=false;
            // we need to select omgraphics in the area defined
            // by debX,debY,finX,finY
            // get omGraphics under the selected area
            OMGraphicList omgl = getList();
            if (omgl==null) return true;
            // this is a circle with center(debX,debY)
            int x = finX-debX;
            int y = finY-debY;
            double rayon = Math.ceil(Math.sqrt(((x*x)+(y*y))));
            for (int i = 0 ; i < omgl.size() ; i++){
                OMGraphic omg = omgl.getOMGraphicAt(i);
                // verify if OMGraphic clip area
                if (omg.distance(debX,debY) < rayon){
                    // take into account the mode selection (multi or unique) and select it with left button, desemect with right button.
                    if ((e.getModifiers() & InputEvent.BUTTON1_MASK)==InputEvent.BUTTON1_MASK) {
			Debug.message(Util.className(this),this +" : _select object");
			selectOMGraphic(omg);
		    } else if ((e.getModifiers() & InputEvent.BUTTON3_MASK)==InputEvent.BUTTON3_MASK) {
			Debug.message(Util.className(this),this +" : _deselect object");
			deselectOMGraphic(omg);
		    }
                }
            }
        }
	// refresh(); // working ?
        repaint();
        return true;
    }
    
    /**
     * Nothing to do ...
     */
    public boolean mouseMoved(MouseEvent e) {
        tipsX=e.getX();
        tipsY=e.getY();
        Projection projection = getProjection();
        if (projection.getScale()<1000000f) repaint();
        return false;
    }
    
    public void mouseMoved() {}
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    
// -------------------------------------------------------------------------------------
    /**
     * Return the current MouseMode ID ...
     */
    
    private String getCurrentMode(){
        BeanContext mapHandler=getBeanContext();
        if (mapHandler==null) return null;
        
        Iterator it = mapHandler.iterator();
        Object someObj;
        while(it.hasNext()) {
            someObj = it.next();
            if (someObj instanceof MouseDelegator) {
                MouseDelegator md = (MouseDelegator) someObj;
                if (md!=null) return  md.getActiveMouseModeID();
            }
        }
        return null;
    }
    

    /**
     * Set the center of the projection and scale.
     **/
/*done in SimpleDataDisplayLayer, now    private void setCenterAndProjection(){
        BeanContext mapHandler = getBeanContext();
        if (mapHandler==null)  Debug.error(Util.className(this)+" Error : mapHandler is null");
        Iterator it = mapHandler.iterator();
        Object someObj = null;
        MapBean map = null;
        MouseDelegator msd = null;
        while(it.hasNext()) {
            someObj = it.next();
            if (someObj instanceof MapBean) {
                map = (MapBean) someObj;
            } else if (someObj instanceof MouseDelegator){
                msd = (MouseDelegator) someObj;
            }
        }
        Projection projection = map.getProjection();
        Proj p = (Proj)projection;
        p.setScale((float)scale);
        p.setCenter((float)latitude,(float)longitude);
        map.setProjection(p);
        repaint();
    };*/
    
    
    /**
     * Draw a circle the x,y center and rayon passed by rx,ry
     */
    private void drawCircle(int x, int y, int rx,int ry, Graphics g, Color c,Projection projection){
        int a = rx-x;
        int b = ry-y;
        int rayon = (int)Math.ceil(Math.sqrt(((a*a)+(b*b))));
        
        circle.setX(x);
        circle.setY(y);
        circle.setWidth(rayon*2);
        circle.setHeight(rayon*2);
        circle.setLinePaint(c);
        circle.setFillPaint(c);
        circle.setVisible(true);
        circle.select();
        circle.generate(projection);
        circle.render(g);
        
        // tell to the user how kilometer this circle represent.
        Color color = new Color(1f,1f,1f,1f);
        // need to compute ...
        // put it from XY to radian
        LatLonPoint center = projection.inverse(x,y);
        LatLonPoint border = projection.inverse(rx,ry);
        
        double distance = (double)GreatCircle.sphericalDistance(ProjMath.degToRad(center.getLatitude()),
                ProjMath.degToRad(center.getLongitude()),
                ProjMath.degToRad(border.getLatitude()),
                ProjMath.degToRad(border.getLongitude()));
        // Get it in kilometer
        Length lengthSelected = Length.KM;
        DecimalFormat df = new DecimalFormat("0.### Km");
        drawToolTips(selectionText,g,projection,rx,ry-5,df.format(lengthSelected.fromRadians((float)distance)),Color.BLACK);
    }
    
    /**
     * Draw a tips representing the data in the current OMGraphic
     */
    protected void drawToolTips(OMText text, Graphics g, Projection projection, int x, int y, String data,Color color){
        text.setData(data);
        text.setX(x);
        text.setY(y);
        text.setLinePaint(color);
        text.generate(projection);
        text.render(g);
    }
    
    /**
     * paint objects on the layer.
     * @param g the Graphics context for painting
     */
    public void paint(Graphics g) {
        super.paint(g);
	Debug.message(Util.className(this),this +" : paint(Graphics g)");

/* done in SimpleDataDisplayLayer, now        if (!initializedScale){
            initializedScale=true;
            setCenterAndProjection();
        }*/
        nothingToDraw=false;
        OMGraphicList omgl = getList();
        if (omgl==null) return;
        Projection projection = getProjection();
        if (projection==null) return;
        if (drawArea) drawCircle(debX,debY,finX,finY,g,((SelectionDataStorage)dataStorageIterator.getDataStorage()).getCurrentColorSelection(),projection);
        // we draw tips only if the are under a specific projection.
        if (drawToolTipsMode && projection.getScale()<1000000f ) {
            OMGraphic omgr = omgl.findClosest(tipsX,tipsY);
            if (omgr!=null) {
                int omgrIndex = (Integer)omgr.getAttribute(Util.OMGraphicIndex);
                SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
                if (selectionDataStorage==null) return;
                String tipsvalue = selectionDataStorage.getRowValue(omgrIndex,toolTipsColumnName);
                drawToolTips(tipsText,g,projection,tipsX,tipsY-5,tipsvalue,Color.BLACK);
                if (changeZonePopup) {
                    changeZonePopup(tipsvalue,omgr,popupX,popupY);
                    changeZonePopup = false;
                }
            }
        }
    }
    
    public void changeZonePopup(String cityName, OMGraphic omgr, int x, int y) {       
            popup = new JPopupMenu();
            current_omgr = omgr;
            SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
            int current_selection = selectionDataStorage.getCurrentSelection();            
            int nb_sous_zones = selectionDataStorage.getCurrentSelectionSize();
            if (nb_sous_zones>0) {
                detail = new JMenuItem("Placer "+cityName+" dans une sous-zone");
                detail.setHorizontalTextPosition(AbstractButton.LEADING);                
                detail.setEnabled(false);
                popup.add(detail);
                for (int i = 0; i< nb_sous_zones; i++) {
                    JMenuItem sz = new JMenuItem("Sous-zone "+(i+1));
                    sz.setHorizontalTextPosition(AbstractButton.LEADING);
                    sz.setActionCommand(""+i);
                    sz.addActionListener(this);
                    popup.add(sz);
                }
                popup.show(this,x,y);
            }
            changeZonePopup = false;
    }
    
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        if (e.getSource() == centerAndZoomButton) centerAndZoom();
        if (e.getSource() == resetMapButton) resetMap();
        if (e.getSource() == redrawSelectionOnMapButton) redrawSelectionOnMap();
        if (e.getSource() == selectAllButton) selectAll();
        if (e.getSource() == deselectAllButton) deselectAll();
        if (e.getSource() == addSelectionButton) addSelection();
        if (e.getSource() == addSubSelectionButton) addSubSelection();
        if (e.getSource() == removeSelectionButton) removeCurrentSelection();
        if (e.getSource() == removeSubSelectionButton) removeSubSelection();
        if (e.getSource() == drawingSelectedButton) {
            drawGraphicNameMode = LayerConstants.DRAWING_NAME_MODE_SELECTED;
            repaint();
        }
        if(e.getSource().getClass()==JMenuItem.class) {
            SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
            DataAndSelectionModel dsm = selectionDataStorage.getDataAndSelectionModel();
            int row = (Integer)current_omgr.getAttribute(Util.OMGraphicIndex);            
            int col = dsm.getDataColumnCount()+Integer.parseInt(e.getActionCommand());
            if (!(Boolean)dsm.getValueAt(row,col)) dsm.setValueAt(true,row,col);                        
        }
    }
    
    public Component getGUI() {
        Debug.message(Util.className(this), this + " Interactive.getGUI()");
        if (!createPalette) return null;
        if (box == null) {
            ColorsPanel colorPanel = new ColorsPanel(colors);
            colorPanel.addUnicaenListener(this);
            
            Component fatherGui = dataStorageIterator.getGUI();
            // if no gui, we create one, otherwise we add information to itit.
            
            if (fatherGui==null) {
                fatherGui = new JPanel(new BorderLayout());
            }
            JPanel radioPanel = new JPanel(new GridLayout(1, 0));
            radioPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Affichage des informations"), BorderFactory.createEmptyBorder(5,5,5,5)));
            String noDrawing = new String("Afficher aucune information");
            noDrawingButton = new JRadioButton(noDrawing);
            noDrawingButton.setActionCommand(noDrawing);
            
            String drawingAll = new String("Afficher toutes les informations");
            drawingAllButton = new JRadioButton(drawingAll);
            drawingAllButton.setActionCommand(drawingAll);
            
            // add button to congfiguration mode
            String drawingSelected = new String("Afficher les informations pour les objets sÃÂ©lectionnÃÂ©s");
            drawingSelectedButton = new JRadioButton(drawingSelected);
            drawingSelectedButton.setActionCommand(drawingSelected);
            
            // add group
            drawingGroup = new ButtonGroup();
            drawingGroup.add(noDrawingButton);
            drawingGroup.add(drawingAllButton);
            drawingGroup.add(drawingSelectedButton);
            // add listener
            noDrawingButton.addActionListener(this);
            drawingAllButton.addActionListener(this);
            drawingSelectedButton.addActionListener(this);
            // add to panel
            radioPanel.add(noDrawingButton);
            radioPanel.add(drawingAllButton);
            radioPanel.add(drawingSelectedButton);
            
            ((JComponent)fatherGui).add(radioPanel);
            
            switch (drawGraphicNameMode) {
                case LayerConstants.DRAWING_NAME_MODE_OFF :
                    noDrawingButton.setSelected(true);
                    break;
                case LayerConstants.DRAWING_NAME_MODE_ON :
                    drawingAllButton.setSelected(true);
                    break;
                case LayerConstants.DRAWING_NAME_MODE_SELECTED :
                    drawingSelectedButton.setSelected(true);
                    break;
                default :
                    noDrawingButton.setSelected(true);
                    break;
            }
            
            box = new SelectionPalette(fatherGui, colorPanel, (SelectionStoragePanel)dataStorageIterator.getDataGUI());
            if (box != null) {
                Debug.message(Util.className(this), this + " ajout des boutons interactifs ");
                // draw the map without any information.
                resetMapButton = new JButton(Util.loadIcon("ResetMap.png"));
                resetMapButton.setToolTipText("Afficher le fond de carte original");
                resetMapButton.setActionCommand("resetMap");
                resetMapButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(resetMapButton);
                
                // redraw seletions on map
                redrawSelectionOnMapButton = new JButton(Util.loadIcon("TableDisplayData.png"));
                redrawSelectionOnMapButton.setToolTipText("Afficher la sÃÂ©lection courante sur la carte");
                redrawSelectionOnMapButton.setActionCommand("redrawSelectionOnMap");
                redrawSelectionOnMapButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(redrawSelectionOnMapButton);
                
                // select all elements
                selectAllButton = new JButton(Util.loadIcon("SelectAll.png"));
                selectAllButton.setToolTipText("SÃÂ©lectionner tous les composants de la carte");
                selectAllButton.setActionCommand("selectAll");
                selectAllButton.addActionListener(this);
                // ((SelectionPalette)box).addToToolBar(selectAllButton);
                
                // unselect all elements
                deselectAllButton = new JButton(Util.loadIcon("DeselectAll.png"));
                deselectAllButton.setToolTipText("DÃÂ©sÃÂ©lectionner les communes de la sous-zone courante");
                deselectAllButton.setActionCommand("CenterAndZoom");
                deselectAllButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(deselectAllButton);
                
                // try to add QuickLocationMode ...
                SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
                if (getQuickLocationMode()){
                    // serching text field on column data
                    textField = new JTextField(20);
                    textField.setToolTipText("Entrez le dÃÂ©but de la commune recherchÃÂ©e");
                    //textField.setActionCommand("setCommuneName");
                    textField.getDocument().addDocumentListener(this);
                    textField.addKeyListener(this);
                    ((SelectionPalette)box).addToToolBar(textField);
                    
                    // zoom an center on selected space
                    centerAndZoomButton = new JButton(Util.loadIcon("CenterAndZoom.png"));
                    centerAndZoomButton.setToolTipText("Centrer et zoomer sur la commune sÃÂ©lectionnÃÂ©e");
                    centerAndZoomButton.setActionCommand("centerAndZoom");
                    centerAndZoomButton.addActionListener(this);
                    ((SelectionPalette)box).addToToolBar(centerAndZoomButton);
                }
                // create a new selection
                addSelectionButton = new JButton(Util.loadIcon("CreateSelection.png"));
                addSelectionButton.setToolTipText("CrÃÂ©er une nouvelle zone de chalandise");
                addSelectionButton.setActionCommand("addSelection");
                addSelectionButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(addSelectionButton);
                
                // delete current selection.
                removeSelectionButton = new JButton(Util.loadIcon("DeleteSelection.png"));
                removeSelectionButton.setToolTipText("Supprimer la zone de chalandise courante");
                removeSelectionButton.setActionCommand("removeSelection");
                removeSelectionButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(removeSelectionButton);
                
                
                // create a new sub-selection
                addSubSelectionButton = new JButton(Util.loadIcon("CreateSubSelection.png"));
                addSubSelectionButton.setToolTipText("CrÃÂ©er une nouvelle sous-zone");
                addSubSelectionButton.setActionCommand("addSubSelection");
                addSubSelectionButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(addSubSelectionButton);
                
                
                // delete current selection.
                removeSubSelectionButton = new JButton(Util.loadIcon("DeleteSubSelection.png"));
                removeSubSelectionButton.setToolTipText("Supprimer la sous-zone courante");
                removeSubSelectionButton.setActionCommand("removeSubSelection");
                removeSubSelectionButton.addActionListener(this);
                ((SelectionPalette)box).addToToolBar(removeSubSelectionButton);
                
            }
        }
        return box;
    }
    
    /**
     * Center on the current found index and zoom to view it well;
     */
    public void centerAndZoom(){
        BeanContext mapHandler = getBeanContext();
        if (mapHandler==null)  Debug.message(Util.className(this),this+"Error : mapHandler is null");
        Iterator it = mapHandler.iterator();
        Object someObj = null;
        MapBean map = null;
        MouseDelegator msd = null;
        while(it.hasNext()) {
            someObj = it.next();
            if (someObj instanceof MapBean) {
                map = (MapBean) someObj;
            } else if (someObj instanceof MouseDelegator){
                msd = (MouseDelegator) someObj;
            }
        }
        Projection projection = map.getProjection();
        Proj p = (Proj)projection;
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        // get the first selected line
        int line = getRowTableSelectionIndex();
        if (line!=-1) {
            // need to fix the center of the projection.
            DataObject object = selectionDataStorage.getElementAt(line);
            Centroid centroid = object.getOMDataLocation(geometryColumnName).getCentroid();
            p.setCenter((float)centroid.getLat(),(float)centroid.getLon());
            // get omgraphic and select it in the model.
            OMGraphic omgraphic = object.getOMDataLocation(geometryColumnName).getOMGraphic();
            selectOMGraphic(omgraphic);
        }
        // change Mouse mode => Gestures.
        msd.setActiveMouseModeWithID(SelectMouseMode.modeID);
        //  Fixed scale => TODO : add a property
        if (p.getScale()==400000f) {
            map.setProjection(p);
            repaint();
        } else {
            p.setScale(400000f);
            map.setProjection(p);
        }
    }
    /**
     * Method that we use to generate the list of drawable
     * objects.
     * This method is called by
     * @see #projectionChanged(ProjectionEvent e)
     * @see #renderDataForProjection(Projection proj, java.awt.Graphics g)
     */
    public synchronized OMGraphicList generateGraphics() {
        Debug.message(Util.className(this),this+"GeometryDataDisplayLayer : generateGraphics()");
        
        OMGraphicList omgraphics = new OMGraphicList();
        // No iterator == no data => return.
        if (dataStorageIterator == null) return omgraphics;
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        // get projection in order to compute the bounding box.
        Projection proj = getProjection();
        if (proj == null) return omgraphics;
        
        // get the bounding Box
        LatLonPoint upperLeft = proj.getUpperLeft();
        LatLonPoint lowerRight = proj.getLowerRight();
        BoundingBox area = new BoundingBox(upperLeft.getLongitude(), upperLeft.getLatitude(),
                lowerRight.getLongitude(), lowerRight.getLatitude());
        
        // clear the list.
        omgraphics.clear();
        
        // reset the iterator to the first record.
        dataStorageIterator.reset();
        
        // Find valid object to draw (i.e. their bounding box is inside
        // the area). We will try to draw an object if there is no bounding box for it.
        while (dataStorageIterator.hasNext()) {
            OMDataLocation omdl = dataStorageIterator.next().getOMDataLocation(geometryColumnName);
            if (omdl != null && omdl.getBoundingBox() != null && omdl.getBoundingBox().intersects(area)) {
                OMGraphic omg = omdl.getOMGraphic();
                omg.setVisible(true);
                omg.setLinePaint(colors.getLineColor());
                int index = (Integer)omg.getAttribute(Util.OMGraphicIndex);
                Color redrawcolor = selectionDataStorage.getOMGraphicColor(index);
                if (redrawcolor!=null)  omg.setFillPaint(redrawcolor);
                else  omg.setFillPaint(colors.getFillColor());
                if (omg instanceof OMGraphicList){
                    ((OMGraphicList)omg).setVague(true);
                }
                omgraphics.add(omg);
            }
        }
        Debug.message(classNameForDebug, this + " using " + omgraphics.size() + " objects.");
        omgraphics.generate(proj);
        return omgraphics;
    }
    
    /*
     * redraw current selections on map
     **/
    public void redrawSelectionOnMap(){
        OMGraphicList omgl = getList();
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        for (int i = 0 ; i < omgl.size() ; i++){
            OMGraphic omg = omgl.getOMGraphicAt(i);
            omg.setLinePaint(colors.getLineColor());
            int index = (Integer)omg.getAttribute(Util.OMGraphicIndex);
            Color redrawcolor = selectionDataStorage.getOMGraphicColor(index);
            if (redrawcolor!=null)  {
                omg.setFillPaint(redrawcolor);
            } else  omg.setFillPaint(colors.getFillColor());
        }
        repaint();
    }
    
    public OMGraphicList getTextFromOMGraphicList(){
        Projection proj = getProjection();
        if (proj == null) return null;
        if (nothingToDraw || proj.getScale()>1000000f) return null;
        OMGraphicList liste = new OMGraphicList();
        OMGraphicList omgl = getList();
        if (omgl==null) return null;
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        FontSizer fontsizer = new FontSizer(150000f,1,8, 16);
        for (int i = 0 ; i < omgl.size() ; i++){
            OMGraphic omg = omgl.getOMGraphicAt(i);
            int index = (Integer)omg.getAttribute(Util.OMGraphicIndex);
            Color redrawcolor = selectionDataStorage.getOMGraphicColor(index);
            if (drawGraphicNameMode==LayerConstants.DRAWING_NAME_MODE_ON || (drawGraphicNameMode==LayerConstants.DRAWING_NAME_MODE_SELECTED && redrawcolor!=null)) {
                DataObject dataObject = selectionDataStorage.getElementAt(index);
                Centroid centre = dataObject.getOMDataLocation(geometryColumnName).getCentroid();
                OMText omt = new OMText((float)centre.getLat(),(float)centre.getLon(),(String)dataObject.getData(drawColumnName),OMText.JUSTIFY_CENTER);
                omt.setLinePaint(drawColumnNameColor);
                omt.setFontSizer(fontsizer);
                liste.add(omt);
            }
        }
        liste.generate(proj);
        return liste;
    }
    
    /*
     * select all omgraphics on current selection
     **/
    public void selectAll(){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        selectionDataStorage.selectAllDefaultSelection();
        redrawSelectionOnMap();
    }
    /*
     * deselect all omgraphics on current selection
     **/
    public void deselectAll(){
        int choice = JOptionPane.showConfirmDialog(null,
                "Attention, toutes les donnÃÂ©es sÃÂ©lectionnÃÂ©es pour cette sous-zone seront perdues !\nÃÂtes-vous sÃÂ»rs de vouloir vider la sous-zone courante ?",
                "Vider la sous-zone courante",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE);
        if (choice != JOptionPane.YES_OPTION) { return; }
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        selectionDataStorage.deselectAllDefaultSelection();
        redrawSelectionOnMap();
    }
    
    /*
     * Draw the initial map
     **/
    public void resetMap(){
        OMGraphicList omgl = getList();
        for (int i = 0 ; i < omgl.size() ; i++){
            OMGraphic omg = omgl.getOMGraphicAt(i);
            omg.setLinePaint(colors.getLineColor());
            omg.setFillPaint(colors.getFillColor());
        }
        nothingToDraw=true;
        repaint();
    }
    
    /**
     * Create en new empty selection
     */
    public void addSelection(){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        String s = JOptionPane.showInputDialog("Entrer le nom de la nouvelle sÃÂ©lection puis valider", selectionDataStorage.getNextSelectionName());
        if (s==null) { return;}
        if (s.trim().equals("")) {
            JOptionPane.showMessageDialog(box,"Nom invalide pour une zone de chalandise.","Erreur",JOptionPane.ERROR_MESSAGE);
            addSelection();
            return;
        }
        if (!selectionDataStorage.isTitleExists(s)) {
            selectionDataStorage.createNewEmptySelection(s.trim());
            selectionDataStorage.setTabSelectedIndex(selectionDataStorage.getSelectionsSize()-1);
            // selectionDataStorage.setCurrentSelection(selectionDataStorage.getSelectionsSize()-1);
        } else JOptionPane.showMessageDialog(box,"La zone de chalandise \""+s.trim()+"\" existe dÃÂ©jÃÂ  !","Erreur",JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * Create en new empty sub-selection
     */
    public void addSubSelection(){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        selectionDataStorage.createSubSelection();
    }
    
    /**
     * Delete the current Selection.
     */
    public void removeCurrentSelection(){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        String msg = "ÃÂtes-vous sÃÂ»r de vouloir supprimer l'onglet \""+selectionDataStorage.getSelectionName(selectionDataStorage.getCurrentSelection())+"\" ?";
        if (JOptionPane.showConfirmDialog(new Frame(),msg,"Suppression...", JOptionPane.WARNING_MESSAGE,JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION){
            selectionDataStorage.deleteCurrentSelection();
        }
        
    }
    
    /**
     * Delete the current Sub-Selection.
     */
    public void removeSubSelection(){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        int currentSelection = selectionDataStorage.getCurrentSelection();
        int subselection = selectionDataStorage.getSelectionMatrix(currentSelection).getCurrentSubSelection();
        String msg = "ÃÂtes-vous sÃÂ»r de vouloir supprimer l'onglet \""+selectionDataStorage.getSubSelectionName(currentSelection,subselection )+"\" ?";
        if (JOptionPane.showConfirmDialog(new Frame(),msg,"Suppression...", JOptionPane.WARNING_MESSAGE,JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION){
            if (selectionDataStorage.getSelectionMatrix(currentSelection).getNbCols()>1) {
                selectionDataStorage.deleteSubSelection();
                selectionDataStorage.setTabSelectedIndex(currentSelection);
            } else JOptionPane.showMessageDialog(new Frame(),"Impossible de supprimer l'onglet","ERREUR", JOptionPane.ERROR_MESSAGE);
        }
        
    }
    //-------------------------------------- JTextField --------------------------------
     /*
      * We have to select the row which match the current text store in JTextField
      */
    
    public void insertUpdate(DocumentEvent e) {
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        int col = getQuickLocationColumn();
        String textefield=null;
        try {
            textefield = e.getDocument().getText(0,e.getDocument().getLength());
        } catch (BadLocationException excep){
            excep.printStackTrace();
            return;
        };
        if (textefield.length()==0) {
            Debug.message(Util.className(this), this + " : on vide la table ");
            clearTableSelection();
            return;
        }
        Debug.message(Util.className(this), this + " Textefield : "+textefield);
        int pos=-1;
        // we could use dichotomy to optimize ... TODO
        // need to permit Locale selection ... TODO
        // Collator myCollator = Collator.getInstance(Locale.FRENCH);
        // myCollator.setStrength(Collator.SECONDARY);
        
        RuleBasedCollator myCollator = (RuleBasedCollator)Collator.getInstance();
        String oldRules = myCollator.getRules();
        try {
            myCollator = new RuleBasedCollator("<"+oldRules.substring(1));
        } catch (ParseException pe) {}
        
        boolean goOn=true;
        for (int i=0; i<selectionDataStorage.getCount() && goOn;i++) {
            if (myCollator.compare((String)selectionDataStorage.getValueAt(i,col),textefield)<0)
                pos=i;
            else goOn=false;
        }
        // this is the first one ...
        if (!goOn && pos==-1) {
            setRowTableSelection(0);
            return;
        }
        // nothing found ...
        if (goOn && pos!=-1) {
            clearTableSelection();
            return;
        }
        // if there's no problem let's select the good position
        if (!goOn && pos!=-1) setRowTableSelection(pos+1);                    
    }
    /**
     * Refresh selection in table with the text after the remove action.
     **/
    public void removeUpdate(DocumentEvent e) {
        insertUpdate(e);
    }
    /**
     * Same action : Enter in keypressed or used CenterAndZoom button.
     **/
    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            centerAndZoom();
        }
    }
    
    // nothing to do ..
    public void keyReleased(KeyEvent ke) {}
    public void keyTyped(KeyEvent ke) {}
    public void changedUpdate(DocumentEvent e) { }
    
    
    /////// Update gui methods /////
    
    //
    private JTable getJTable(){
        JTable mytable = null;
        SelectionStoragePanel ssp = ((SelectionPalette)box).getDataPanel();
        DataTableSelectionStoragePanel dtssp = ssp.getDataTableSelectionStoragePanel();
        mytable = dtssp.getJTable();
        return mytable;
    }
    
    /**
     * clear all selection in Jtable used.
     **/
    private void clearTableSelection() {
        // need to update gui palette.
        getJTable().clearSelection();
    }
    
    /**
     * Selection a column int the JTable used to sotre dataAndSelectionModel
     **/
    
    private void setRowTableSelection(int row) {
        JTable mytable =  getJTable();
        // need to update Gui palette.
        mytable.setRowSelectionInterval(row,row);
        mytable.scrollRectToVisible(mytable.getCellRect(row,0,true));
    }
    
    /**
     * Return the index of the first selected row in DataTable, -1 if nothing is selected.
     **/
    private int getRowTableSelectionIndex(){
        // need to update gui palette
        return getJTable().getSelectedRow();
    }
    
    /**
     * select the omgraphic if selection mode
     **/
    private void selectOMGraphic(OMGraphic omgraphic){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        if (selectionDataStorage.selectIndexInModel((Integer)omgraphic.getAttribute(Util.OMGraphicIndex))){
            omgraphic.setFillPaint(selectionDataStorage.getCurrentColorSelection());
        }
    }
    
    
    /**
     * deselect the omgraphic if selection mode
     **/
    private void deselectOMGraphic(OMGraphic omgraphic){
        SelectionDataStorage selectionDataStorage = (SelectionDataStorage)dataStorageIterator.getDataStorage();
        // if we can deselect the omgraphic, we paint it with default colors.
        if (selectionDataStorage.deselectIndexInModel((Integer)omgraphic.getAttribute(Util.OMGraphicIndex))){
            omgraphic.setLinePaint(colors.getLineColor());
            omgraphic.setFillPaint(colors.getFillColor());
        }
    }
}


