// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.dataDisplayLayer;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.dataStorage.*;

import java.io.Writer;
import java.util.*;
import java.lang.reflect.*;

import java.awt.*;
import java.util.Properties;

import com.bbn.openmap.*;
import com.bbn.openmap.event.*;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.*;


/**
 * This class DOES NOT WORK for the moment. Please use SimpleDataDisplayLayer
 * instead.
 * <p>
 */
public abstract class ThreadedDataDisplayLayer extends Layer implements DataStorageListener {

    public String toString() { return "[" + Util.className(this) + ":" + getName() + "]"; }
    
    public Component getGUI() {
	return dataStorageIterator.getGUI();
    }

    public void dataStorageEvent (DataStorage source) {
	Debug.message("DataDisplayLayer" , this + " EVENT !");
    }
    
    /**
     * Given a class name, this method tries to instanciate a
     * DataStorage (for use within a DataStorageIterator).
     * 
     * For the moment, this method instanciates a DataStorage with
     * the default constructor. It should not do it that way but
     * should use a constructor with a DataStorageListener.
     *
     * @see DataStorage
     */
    private DataStorage createDataStorage(String className) {
	try {
	    Class cls = Class.forName(className);
	    Constructor[] constructors = cls.getConstructors();
	    for (int i = 0 ; i < constructors.length ; i++) {
		Class params [] = constructors[i].getParameterTypes();
		if (params.length == 1 && params[0].getName() == "fr.unicaen.openmap.dataStorage.DataStorageListener") {
		    Object obj = constructors[i].newInstance(new Object [] { this });
		    Debug.message("DataDisplayLayer" , this + " createDataStorage.");
		    return ((DataStorage) obj);
		}
	    }
	    return null;
	} catch (Exception e) {
	    Debug.message("DataDisplayLayer" , this + " createDataStorage failure.");
	    return null;
	}
    }
    
    String dataStorageClassPropertyString = "dataStorageClass";

    public void setProperties(String prefix, Properties properties) {
	super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);
	
	String className = properties.getProperty(prefix + dataStorageClassPropertyString);
		
	try {
	    DataStorage ds = createDataStorage(className);
	    ds.setProperties(prefix, properties);
	    Debug.message("DataDisplayLayer" , this + " using " + Util.className(ds) + " as DataStorage");
	    dataStorageIterator = new DataStorageIteratorAdapter(ds);
	} catch (Exception e) { }
	
    }
    /**
     *
     */
    protected DataStorageIterator dataStorageIterator;
    
    /**
     * Save all the data needed to reconstruct this layer
     */
    public abstract boolean save(Writer writer);
    
    /**
     * Method used by the generic paint method of DataDisplayLayer
     * @return the OMGraphicList that will be used to paint the layer
     */
    //    public abstract OMGraphicList getOMGraphicList(Projection projection); 

    /**
     * Swing worker containing the separate thread to do the work.
     */
    protected DataDisplayWorker currentWorker = null;
    
    /**
     * Projection that gets set on a projection event.
     */
    protected Projection projection;
    
    /** 
     * Set when the projection has changed while a swing worker is
     * gathering graphics, and we want him to stop early.
     */
    protected boolean cancelled = false;
    
    /**
     * The complete list of lines and markers, which will include the
     * other two lists (real polygons and associated data).
     */
    protected OMGraphicList omgraphics = new OMGraphicList();
    
    /**
     * We don't want to requery each time we draw a picture, so
     * the list will be cached.
     */
    protected OMGraphicList cachedOMGraphicList = new OMGraphicList();
        
    /**
     * Invoked when the projection has changed or this Layer has been added to
     * the MapBean.
     * <p>
     * Perform some extra checks to see if reprojection of the graphics is
     * really necessary.
     * @param e ProjectionEvent
     *
     */    
    public void projectionChanged(ProjectionEvent e) {
	if (projection != null){
	    if (projection.equals(e.getProjection())) {
		repaint();
		return;
	    }
	}
	projection = (Projection)e.getProjection().makeClone();
	//	setProjection(e);

	doPrepare();
    }
    
    /**
     * This method will create a thread if necessary.
     */
    public void doPrepare() {
	if (currentWorker == null) {
	    if (Debug.debugging("DataDisplayLayer")){
		Debug.output(this + " creating thread ...");
	    }
	    currentWorker = new DataDisplayWorker();
	    currentWorker.execute();
	}
	else setCancelled(true);
    }
    
    /**  
     * The DataDisplayWorker calls this method on the layer when it is
     * done working.  If the calling worker is not the same as the
     * "current" worker, then a new worker is created (<- is this
     * possible indeed ?).
     *
     * @param worker the worker that has the graphics.
     */
    protected synchronized void workerComplete(DataDisplayWorker worker) {
	if (!isCancelled()) {
	    Debug.message("DataDisplayLayer" , this + " Worker: not cancelled");
	    currentWorker = null;
	    setOMGraphics((OMGraphicList)worker.get());
	    repaint();
	}
	else{
	    Debug.message("DataDisplayLayer" , this + " Worker: cancelled");
	    setCancelled(false);
	    currentWorker = new DataDisplayWorker();
	    currentWorker.execute();
	}
    }

        
    public void paint(java.awt.Graphics g) {
	Debug.message("DataDisplayLayer" , this + " paint()");
	OMGraphicList list = getOMGraphics___();
	if (list != null) {
	    list.render(g);
	}
    }
    
    
    /** 
     * 
     */
    public synchronized void setOMGraphics(OMGraphicList graphics) {
	omgraphics = graphics;
    }

    /** 
     *
     */
    public synchronized OMGraphicList getOMGraphics___() {
	
	// get the bounding Box
	LatLonPoint upperLeft = projection.getUpperLeft();
	LatLonPoint lowerRight = projection.getLowerRight();

	BoundingBox area = new BoundingBox (upperLeft.getLongitude(),upperLeft.getLatitude(),lowerRight.getLongitude(),lowerRight.getLatitude());
	
	// OMGraphic omg = null;
	OMGraphicList omgl = new OMGraphicList();
	dataStorageIterator.reset();
	/*
	while (dataStorageIterator.hasNext()) {
	    // OMDataLocation omdl = dataStorageIterator.next().getOMDataLocation();
	    // if (omdl.getBoundingBox() != null) {
	    // if (omdl.getBoundingBox().intersects(area)) {
	    // System.out.println("bb ok");
	    // System.out.println(" + OMGraphic");
	    OMGraphic omg = omdl.getOMGraphic();
	    omg.setVisible(true);
	    omg.setLinePaint(Color.red);
	    omg.setFillPaint(Color.blue);
	    omgl.add(omg);
		    //} else
		    // System.out.println("ignoring object : bb out");		
		    // } else
		    // System.out.println("ignoring object : no bb");
	}
	*/
	double [] llpts = new double[8];
	llpts[0] = 10f; llpts[1] = -20f;
        llpts[2] = 45f; llpts[3] = -70f;
        llpts[4] = 0f;  llpts[5] = -90f;
        llpts[6] = -15f;llpts[7] = -40f;
	
	OMGraphic omg = new OMPoly(llpts, OMPoly.DECIMAL_DEGREES, OMGraphic.LINETYPE_GREATCIRCLE, 360);
	omg.setVisible(true);
	omg.setLinePaint(Color.red);
	omg.setFillPaint(Color.blue);
	omgl.add(omg);
	
	omgl.generate(getProjection());
	return omgl;
    }

    /** 
     * Used to set the cancelled flag in the layer.  The swing worker
     * checks this once in a while to see if the projection has
     * changed since it started working.  If this is set to true, the
     * swing worker quits when it is safe. 
     */
    public synchronized void setCancelled(boolean set){
	cancelled = set;
    }

    /**
     * Check to see if the cancelled flag has been set.
     */
    public synchronized boolean isCancelled(){
	return cancelled;
    }
    
    /**
     * Make a "hard" copy of a list.
     * Used to cache the OMGraphicList.
     */
    OMGraphicList copyOMGraphicList(OMGraphicList source) {
	OMGraphicList result = new OMGraphicList();
	java.util.List list = (java.util.List) source.getTargets();
	Iterator i = list.iterator();
	while (i.hasNext()) result.add((OMGraphic) i.next());
	return result;
    }
    
    /**
     * Sample implementation of the dataDisplayLayer construction.
     */
    // protected abstract OMGraphicList constructDataDisplayLayer();
    protected OMGraphicList constructDataDisplayLayer() {
	OMGraphicList omgl = new OMGraphicList();
	while (dataStorageIterator.hasNext()) {
	    // omgl.add(((DataObject) dataStorageIterator.next()).getOMDataLocation().getOMGraphic());
	}
	return omgl;
    }
    
    /**
     * Little Swing Worker does the job for us ...
     */
    class DataDisplayWorker extends SwingWorker {
	
	/**
	 *
	 */
	public DataDisplayWorker() { super(); }
	
	/**
	 *
	 */
	public Object construct() {
	    Debug.message("DataDisplayLayer" , this + " DataDisplayWorker.construct()");
	    fireStatusUpdate(LayerStatusEvent.START_WORKING);
	    try {
		return constructDataDisplayLayer();
		// return null ; //return setAssociatedData(constructDataDisplayLayer());
	    } catch (OutOfMemoryError e) {
		String msg = "[" + getName() + "] PostGisLayer.DataDisplayWorker.construct(): " + e;
		Debug.error(msg);
		e.printStackTrace();
		fireRequestMessage(new InfoDisplayEvent(this, msg));
		fireStatusUpdate(LayerStatusEvent.FINISH_WORKING);
		return null;
	    }
	}

	/**
	 *
	 */
	public void finished() {
	    Debug.message("DataDisplayLayer" , this + " DataDisplayWorker.finiched()");
	    workerComplete(this);
	    fireStatusUpdate(LayerStatusEvent.FINISH_WORKING);
	}
    }
}
