// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.DataStorageIterator;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Chart;
import fr.unicaen.openmap.util.Util;
import org.jdesktop.swingx.JXCollapsiblePane;

public class ChartPanel extends JPanel {

    protected I18n i18n = new BasicI18n();
    private JTextField fieldX, fieldY;
    private Chart chart;

    public ChartPanel(Chart aChart, DataStorageIterator dsi, ThematicGUI tgui, int axis) {
        this.chart = aChart;

        setLayout(new BoxLayout(this, axis));

        //Panel for chart type choice
        final JPanel panType = new JPanel(new GridLayout(1, 2));
        panType.setBorder(BorderFactory.createTitledBorder(""));
        Box boxLeft = Box.createVerticalBox();
        ButtonGroup bg = new ButtonGroup();
        final JRadioButton radioXY = new JRadioButton("X Y");
        radioXY.setSelected(true);
        final JRadioButton radioHisto = new JRadioButton("Histogramme");
        boxLeft.add(radioXY);
        boxLeft.add(radioHisto);
        bg.add(radioHisto);
        bg.add(radioXY);

        //boxLeft.add(panSize);

        //chart option panel
//		final JButton bfo = new JButton(i18n.get(this,"parameters","Chart parameters"),Util.loadIcon("visu/triangle.png"));
//    	bfo.setBackground(Color.LIGHT_GRAY);
//    	bfo.setBorder(null);
//    	bfo.setOpaque(false);
//		JPanel panbfo = new JPanel(new BorderLayout());
//		panbfo.add(bfo,BorderLayout.WEST);

        final JPanel panOptions = new JPanel(new GridLayout(1, 2));
        panOptions.setBorder(BorderFactory.createTitledBorder(""));
        JPanel panSize = new JPanel(new GridLayout(2, 2));
        //panSize.setBorder(BorderFactory.createTitledBorder(""));//i18n.get(this, "Dimensions","Dimensions")));
        JLabel labWidth = new JLabel(i18n.get(this, "width", "Width"));
        JSpinner spinWidth = new JSpinner();
        JLabel labHeight = new JLabel(i18n.get(this, "height", "Height"));
        JSpinner spinHeight = new JSpinner();
        spinWidth.setPreferredSize(new Dimension(100, 20));
        spinHeight.setPreferredSize(new Dimension(100, 20));
        JPanel p1 = new JPanel();
        p1.add(labWidth);
        JPanel p2 = new JPanel();
        p2.add(spinWidth);
        JPanel p3 = new JPanel();
        p3.add(labHeight);
        JPanel p4 = new JPanel();
        p4.add(spinHeight);
        panSize.add(p1);
        panSize.add(p2);
        panSize.add(p3);
        panSize.add(p4);

        JPanel panTitles = new JPanel(new GridLayout(2, 2));
        JPanel p5 = new JPanel();
        p5.add(new JLabel("titre x"));
        JPanel p6 = new JPanel();
        fieldX = new JTextField("", 10);
        p6.add(fieldX);
        JPanel p7 = new JPanel();
        p7.add(new JLabel("titre y"));
        JPanel p8 = new JPanel();
        fieldY = new JTextField("", 10);
        p8.add(fieldY);
        panTitles.add(p5);
        panTitles.add(p6);
        panTitles.add(p7);
        panTitles.add(p8);

        panOptions.add(panSize);
        panOptions.add(panTitles);

        //overview panel
        JPanel panRight = new JPanel();
        panRight.setBorder(
                BorderFactory.createTitledBorder(i18n.get(this, "chart_sample", "Sample")));
        panRight.setBackground(Color.WHITE);
        final JLabel labImage = new JLabel(Util.loadIcon("xy.png"));
        panRight.add(labImage);

        panType.add(boxLeft);
        panType.add(panRight);


        //data choice panel
//		final JButton bfd = new JButton(i18n.get(this,"data","Data"),Util.loadIcon("visu/triangle.png"));
//    	bfd.setBackground(Color.LIGHT_GRAY);
//    	bfd.setBorder(null);
//    	bfd.setOpaque(false);
//		JPanel panbfd = new JPanel(new BorderLayout());
//		panbfd.add(bfd,BorderLayout.WEST);

        final JPanel panData = new JPanel(new BorderLayout());
        panData.setBorder(BorderFactory.createTitledBorder(""));
        JLabel labHelp = new JLabel(i18n.get(this, "help_lists", ""));
        Font f = labHelp.getFont();
        labHelp.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));

        final JPanel panLists = new JPanel(new GridLayout(1, 3));
        final DefaultListModel columnListModel = new DefaultListModel();
        for (String columnName : dsi.getDataColumnNames()) {
            columnListModel.addElement(columnName);
        }
        final JList columnList = new JList(columnListModel);

        JPanel panButton = new JPanel();
        panButton.setLayout(new GridLayout(4, 1));
        panButton.add(new JPanel());
        JPanel panAdd = new JPanel();
        JButton bAdd = new JButton(">>");
        panAdd.add(bAdd);
        JPanel panRemove = new JPanel();
        JButton bRemove = new JButton("<<");
        panRemove.add(bRemove);
        panButton.add(panAdd);
        panButton.add(panRemove);
        panButton.add(new JPanel());
        final DefaultListModel selectedListModel = new DefaultListModel();
        final JList selectedColumnList = new JList(selectedListModel);
        panLists.add(columnList);
        panLists.add(panButton);
        panLists.add(selectedColumnList);

        panData.add(labHelp, BorderLayout.NORTH);
        panData.add(panLists, BorderLayout.CENTER);

        //add(panbft);
        //add(panType);
//        JXCollapsiblePane cp = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT, new BorderLayout());
//        cp.add(panType);
//        add(new ThematicConfigurationPanel(i18n.get(this, "chart_type", "Chart type"),
//                cp));

        ThematicConfPanel confType = ThematicConfPanel.createThematicConfPanel(i18n.get(this,  "chart_type", "Chart type"));
        confType.setContent(panType);
        tgui.addThematicConfigurationPanel(confType);
        add(confType);

        //add(panbfd);
        //add(panData);
//        JXCollapsiblePane cp2 = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT, new BorderLayout());
//        cp2.add(panData);
//        add(new ThematicConfigurationPanel(i18n.get(this, "data", "Data"),
//                cp2));

        ThematicConfPanel confData = ThematicConfPanel.createThematicConfPanel(i18n.get(this,  "data", "Data"));
        confData.setContent(panData);
        tgui.addThematicConfigurationPanel(confData);
        add(confData);
        //add(panbfo);
        //add(panOptions);
//        JXCollapsiblePane cp3 = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT, new BorderLayout());
//        cp3.add(panOptions);
//        add(new ThematicConfigurationPanel(i18n.get(this, "parameters", "Chart parameters"),
//                cp3));

        ThematicConfPanel confOptions = ThematicConfPanel.createThematicConfPanel(i18n.get(this,  "data", "Data"));
        confOptions.setContent(panOptions);
        tgui.addThematicConfigurationPanel(confOptions);
        add(confOptions);

        //panType.setVisible(false);
        //panData.setVisible(false);
        //panOptions.setVisible(false);

        //listeners
//		bft.addActionListener(new ActionListener(){
//			public void actionPerformed(ActionEvent e) {
//				if(!panType.isVisible()){
//					bft.setIcon(Util.loadIcon("visu/triangle_down.png"));
//					panType.setVisible(true);
//				}
//				else{
//					bft.setIcon(Util.loadIcon("visu/triangle.png"));
//					panType.setVisible(false);				
//				}
//			}
//    		
//    	});
//		
//		bfd.addActionListener(new ActionListener(){
//			public void actionPerformed(ActionEvent e) {
//				if(!panData.isVisible()){
//					bfd.setIcon(Util.loadIcon("visu/triangle_down.png"));
//					panData.setVisible(true);
//				}
//				else{
//					bfd.setIcon(Util.loadIcon("visu/triangle.png"));
//					panData.setVisible(false);				
//				}
//			}
//    		
//    	});
//		
//		bfo.addActionListener(new ActionListener(){
//			public void actionPerformed(ActionEvent e) {
//				if(!panOptions.isVisible()){
//					bfo.setIcon(Util.loadIcon("visu/triangle_down.png"));
//					panOptions.setVisible(true);
//				}
//				else{
//					bfo.setIcon(Util.loadIcon("visu/triangle.png"));
//					panOptions.setVisible(false);				
//				}
//			}
//    		
//    	});

        radioHisto.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioHisto.isSelected()) {
                    chart.setChartType(Chart.ChartType.BAR);
                    labImage.setIcon(Util.loadIcon("barchart.png"));
                    chart.getLayer().refreshGraphics();
                }
            }
        });
        radioXY.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioXY.isSelected()) {
                    chart.setChartType(Chart.ChartType.XY);
                    labImage.setIcon(Util.loadIcon("xy.png"));
                    chart.getLayer().refreshGraphics();
                }
            }
        });

        //buttons add & remove listeners
        bAdd.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String value = (String) columnList.getSelectedValue();
                selectedListModel.addElement(value);
                columnListModel.removeElement(value);
                chart.addXName(value);
                selectedColumnList.updateUI();
                columnList.updateUI();
            }
        });
        bRemove.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String value = (String) selectedColumnList.getSelectedValue();
                columnListModel.addElement(value);
                selectedListModel.removeElement(value);
                chart.removeXName(value);
                selectedColumnList.updateUI();
                columnList.updateUI();
            }
        });

        //spiners listeners
        spinWidth.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                chart.setChartWidth(Integer.parseInt(val));

            }
        });
        spinWidth.setValue(chart.getChartWidth());

        spinHeight.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                chart.setChartHeight(Integer.parseInt(val));

            }
        });
        spinHeight.setValue(chart.getChartHeight());

    }

    public String getXTitle() {
        return fieldX.getText();
    }

    public String getYTitle() {
        return fieldY.getText();
    }
}
