// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import javax.swing.event.*;
import java.util.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.classification.*;
import fr.unicaen.openmap.util.*;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.util.gui.ComponentSizer;

/**
 * 
 */
public class ColorizerPanel extends JPanel implements ClassificationListener
{
    protected Colorizer colorizer;
    //protected JScrollPane sp;

    protected HistogramPanel histo;
    protected ThresholdColorScalePanel colorScalePanel;

    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected static I18n i18n = new BasicI18n();

    /** Index of Custom in classifNameList */
    protected final int CUSTOM_INDEX = 0;
    /** Default classif index (here Quantile) */
    protected final int DEFAULT_CLASSIF_INDEX = 0;
    
    /** Classification name method to display in a combo box */
    public final static String classifNameList[] = {
	i18n.get(ColorizerPanel.class, "custom", "Custom"),
	i18n.get(ColorizerPanel.class, "quantile", "Quantile"), 
	i18n.get(ColorizerPanel.class, "nestedMeans", "Nested Means"), 
	i18n.get(ColorizerPanel.class, "fixedStep", "Fixed Step Classification"), 
	i18n.get(ColorizerPanel.class, "standardization", "Standardization"),
	i18n.get(ColorizerPanel.class, "boxplot", "Box Plot"),
	i18n.get(ColorizerPanel.class, "discrete", "Discrete"),
    };
    /** Classificaiton class names. Must be strictly parrallel to classifNameList. */
    public final static String classifClassNameList[] = {
	"fr.unicaen.openmap.util.classification.CustomizedClassification",
	"fr.unicaen.openmap.util.classification.Quantile", 
	"fr.unicaen.openmap.util.classification.NestedMeans",
	"fr.unicaen.openmap.util.classification.StepClassification",
	"fr.unicaen.openmap.util.classification.Standardization",
	"fr.unicaen.openmap.util.classification.BoxPlot",
	"fr.unicaen.openmap.util.classification.Discrete",
    };

    private JComboBox classifList = new JComboBox(classifNameList);    

    public ColorizerPanel(Colorizer _colorizer) {
	super();
        setLayout(new BorderLayout());
	this.colorizer = _colorizer;
	colorizer.addClassificationListener(this);
	JPanel classifPanel = new JPanel(new BorderLayout());
        ComponentSizer.fixComponentSize(classifList, new Dimension(120,20));
	classifList.setSelectedIndex(getIndexClassifClassName());
	classifList.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    if (inhibitClassifListItemStateChanged) { // see classificationChanged()
			inhibitClassifListItemStateChanged = false;
			return;
		    }
		    int index = ((JComboBox)e.getSource()).getSelectedIndex();

		    colorizer.resetClassification(classifClassNameList[index]);
		    colorScalePanel.setColorNumberModel(colorizer.getClassification().getClassificationNumberModel());
                    //colorScalePanel.set
		}
	    });

        colorScalePanel = new ThresholdColorScalePanel(colorizer.getColorScale(), colorizer);
        JPanel pClassif = new JPanel();

        pClassif.add(classifList);
	classifPanel.add(colorScalePanel);
	Debug.message("ThematicDataDisplayLayer", "[ColorizerPanel] ColorizerPanel: " + colorizer);
        JPanel p = new JPanel();
        p.setBorder(BorderFactory.createEmptyBorder(-10,0,0,0));
        p.add(classifPanel);
        add(pClassif, BorderLayout.NORTH);
        add(p,BorderLayout.CENTER);
    }

//    public void setScrollDimension(Dimension dim){
//        ComponentSizer.fixComponentSize(sp, dim);
//    }

    public JComboBox getClassifList(){
        return classifList;
    }

    public void setEnabled(boolean b){
        super.setEnabled(b);
        classifList.setEnabled(b);
        colorScalePanel.setEnabled(b);

    }

    private boolean inhibitClassifListItemStateChanged = false;
    public void classificationChanged(Object source) {

	inhibitClassifListItemStateChanged = true; // in order to avoid an infinite loop...
	classifList.setSelectedIndex(getIndexClassifClassName());	
	// reset ColorScale color number
	if (colorizer.getClassification() != null){
            colorScalePanel.updateColorNumberModel(colorizer.getClassification().getClassificationNumberModel());
            //colorScalePanel.setColorNumberModel(colorizer.getClassification().getClassificationNumberModel());
	    colorizer.getColorScale().setColorNumber(colorizer.getClassification().getNumberOfClasses());
        }
    }

    protected int getIndexClassifClassName() {
	if (colorizer == null || colorizer.getClassification() == null){
	    return DEFAULT_CLASSIF_INDEX; //default : Quantile
    }
	String classifClassName = colorizer.getClassification().getClass().getName();
	for (int i = 0 ; i < classifClassNameList.length; i++)
	    if (classifClassNameList[i].equals(classifClassName)){
            return i;
        }
	return -1;
    }

}
