// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.util.*;

import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMGraphic;

/**
 * A OMGraphicList that can be sorted by the size of their elements
 * <p> OMSizeable must extends OMGraphic and implements Comparable&lt;OMSizeable&gt;
 * <br>It should be possible to say it to the compiler, but ... !
 * <p>added elements are sorted in place. sort just returns the sorted list.
 */
public class OMSizeableGraphicList<OMSizeable extends OMGraphic> extends OMGraphicList {

    //    ArrayList<OMSizeable> sortableGraphics = new ArrayList<OMSizeable>();
    LinkedList<OMGraphic> sortedGraphics = new LinkedList<OMGraphic>();
    LinkedList<OMGraphic> noSizeGraphics = new LinkedList<OMGraphic>();
    /**
     * Add an OMSizeable to the list.
     */
    //@SuppressWarnings("unchecked")
    @Override
    public boolean add(OMGraphic g) {
	if (! (g instanceof HaveWidthAndData)) {
	    return noSizeGraphics.add(g);
	}
        else
            return sortedGraphics.add(g);
//	if (sortedGraphics.isEmpty()) {
//	    sortedGraphics.add(g);
//	    return true;
//	}
//
//	int gWidth = ((HaveWidthAndData)g).getWidth();
//	ListIterator<OMGraphic> iter = sortedGraphics.listIterator(0);
//	boolean inserted = false;
//	while(iter.hasNext()) {
//	    int w = ((HaveWidthAndData)iter.next()).getWidth();
//	    if (gWidth < w) {
//		iter.previous();
//		iter.add(g);
//		inserted = true;
//		break;
//	    }
//	}
//	if (! inserted) {
//	    iter.add(g);
//	}

	// for (iter = sortedGraphics.listIterator(0); iter.hasNext(); ) {
	//     HaveWidthAndData o = (HaveWidthAndData)(iter.next());
	//     System.out.print(" " + o.getWidth() + "("+o.getData()+")");
	// }
	// System.out.println();

    }

    @Override
    public void sort() {
        graphics.clear();	
	graphics.addAll(noSizeGraphics);
        Collections.sort(sortedGraphics, new Comparator<Object>() {
            public int compare(Object o1, Object o2) {
                if(((HaveWidthAndData)o1).getWidth() == ((HaveWidthAndData)o2).getWidth())
                    return 0;
                else
                   return  ((HaveWidthAndData)o1).getWidth() > ((HaveWidthAndData)o2).getWidth() ? 1 : -1;
            }
        });
	graphics.addAll(sortedGraphics);
    }

    @Override
    public String toString(){
        String s ="";
        for(OMGraphic omg : sortedGraphics){
            s+= ((HaveWidthAndData)omg).getWidth() + "\n";
        }
        return s;
    }
}
