// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.BasicStrokeEditorMenu;
import com.bbn.openmap.omGraphics.OMPoly;
import java.awt.Color;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import javax.swing.*;


import java.util.Properties;


import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.LegendGUIManager;

import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.legendable.LegendableLayer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.LayoutManager;

import java.awt.Paint;

import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A group of OMResizeableGraphic inside a frame that can be used to translate its content.
 */
public class OMResizeableFrame extends OMResizeableGroup implements PropertyChangeListener {

    protected OMPoly border;
    protected boolean visibleBorder;
    private JButton daButton;
    //protected JToggleButton borderCB;
    //protected JButton bgColButton, lineColButton;
    //protected BasicStroke borderStroke = new BasicStroke(1);
    //protected Color fillPaint = com.bbn.openmap.util.ColorFactory.parseColor("#33FFFFFF");
    //protected Color linePaint = Color.BLACK;
    protected LayoutManager layout;
    private OMResizeableFrameDrawingAttributes da;
    //final ThemaMapStrokeEditorMenu tse = new ThemaMapStrokeEditorMenu(borderStroke, linePaint);
    //protected Stroke stroke = new BasicStroke(1);
    // should replace title in sub classes. Shall we put a customizedTexts?    
    // private String title; // the title is placed at the top of the frame
    // public String getTitle() { return title; }
    // public void setTitle(String t) { title = t; }
    private boolean visuFrame = false;

    protected boolean isVisuFrame() {
        return visuFrame;
    }

    public void setVisuFrame(boolean v) {
        visuFrame = v;
    }

    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String nameForGUI) {
        this(r, ml, xAnchor, yAnchor, width, height, true, nameForGUI, true);
    }
    
    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String nameForGUI, boolean ra) {
        this(r, ml, xAnchor, yAnchor, width, height, true, nameForGUI, ra);
    }

    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String nameForGUI, LayoutManager l) {
        this(r, ml, xAnchor, yAnchor, width, height, true, nameForGUI, true);
        this.layout = l;
        da = new OMResizeableFrameDrawingAttributes(this);
        da.getPropertyChangeSupport().addPropertyChangeListener(this);
    }
    protected OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, boolean withControlAngles, String nameForGUI){
        this(r, ml, xAnchor, yAnchor, width, height, withControlAngles, nameForGUI, true);
    }


    protected OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, boolean withControlAngles, String nameForGUI, boolean ra) {
        super(r, ml, xAnchor, yAnchor, width, height, withControlAngles, nameForGUI, ra);
        da = new OMResizeableFrameDrawingAttributes(this);
        da.getPropertyChangeSupport().addPropertyChangeListener(this);
        border = new OMPoly(new int[]{});
        visibleBorder = true;
        setXYsBorder();
        add(border);
    }

    /** remove all OMResizeableGraphic it contains except omControlPoints */
    public void clear() {
        // as we must call refreshGUI after add(border), it's almost a cut & paste of OMResizeableGraphicList.clear
        resizeableGraphics.clear();
        graphics.clear();
        add(border);
        add(omControlPoints);
        refreshGUI();
    }

    /** The OMResizeableGraphic will be also availaible with getGraphics() */
    public void add(OMResizeableGraphic g) {
        resizeableGraphics.add(g);
        // beware border is first
        graphics.remove(border);
        graphics.add(g.getGraphics());
        graphics.add(border);
        // int s = graphics.size();
        // if (s == 0) {
        //     System.err.println("!!!! OMResizeableFrame.add() : "+this+".graphics are sze 0.");
        // }
        // graphics.add(s == 0 ? s : s-1, g.getGraphics());
    }

    /** extend the border but don't forward this action to the embedded graphics */
    public void extendBorder(int dw, int dh) {

        width += dw;
        height += dh;

        for (int i = 0; i < omControlPoints.size(); i++) {
            OMControlPoint p = ((OMControlPoint) omControlPoints.getOMGraphicAt(i));
            if (p instanceof OMControlAngle) // extends angles
            {
                p.extend(dw, dh);
            } else if (p instanceof OMControlNPoint) {
                p.setAnchor(xAnchor + width / 2, yAnchor);
            } else if (p instanceof OMControlEPoint) {
                p.setAnchor(xAnchor + width, yAnchor + height / 2);
            } else if (p instanceof OMControlSPoint) {
                p.setAnchor(xAnchor + width / 2, yAnchor + height);
            } else if (p instanceof OMControlWPoint) {
                p.setAnchor(xAnchor, yAnchor + height / 2);
            } else {
                System.err.println("### Error: funny control point " + p);
            }
        }

        setXYsBorder();
        // Debug.message("Legend2", "OMResizeableFrame.extendBorder("+dw+","+dh+") ->"+this);
    }

    public String toString() {
        return "Frame@" + hashCode() + " " + width + "x" + height + "@" + xAnchor + "," + yAnchor +
                (getVisible() ? " v" : " ") +
                " border " + (border == null ? "null" : border.isVisible());
    }

    protected void setXYsBorder() {
        border.setXs(new int[]{
                    xAnchor,
                    xAnchor + width,
                    xAnchor + width,
                    xAnchor,
                    xAnchor
                });
        border.setYs(new int[]{
                    yAnchor,
                    yAnchor,
                    yAnchor + height,
                    yAnchor + height,
                    yAnchor
                });

    }

    public void translate(int dx, int dy) {
        // Debug.message("Legend2", this+".translate("+dx+", "+dy+") \t<=");
        super.translate(dx, dy);
        setXYsBorder();
        // Debug.message("Legend2", this+".translate("+dx+", "+dy+") \t=>");
    }

    public void extend(int dw, int dh) {
        // Debug.message("Legend2", "<- "+this+".extend("+dw+", "+dh+") \t<=");
        extendOnlyItSelf(dw, dh);
        //// don't forward the extend by default
        // for (OMResizeableGraphic omrg : resizeableGraphics) omrg.extend(dw, dh);
        // Debug.message("Legend2", "-> "+this+".extend("+dw+", "+dh+") \t=>");
    }

    /** extend only the border */
    public void extendOnlyItSelf(int dw, int dh) {
        extendBorder(dw, dh);
    }

    public int getXAnchor() {
        return xAnchor;
    }

    public int getYAnchor() {
        return yAnchor;
    }

    public void setXAnchor(int x) {
        xAnchor = x;
    }

    public void setYAnchor(int y) {
        yAnchor = y;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    /** show border only if the graphic is visible */
    public void showBorder(boolean b) {
        visibleBorder = b;
        border.setVisible(b && getVisible());
        if(!b){

        }
        else{

        }
        //setBorderFillPaint(fillPaint);
        //setBorderLinePaint(linePaint);
    }

    public boolean isVisibleBorder(){return visibleBorder;}

    public void setBorderFillPaint(java.awt.Paint color) {
        border.setFillPaint(color);
        refreshable.refreshInteractiveGraphics();
    }

    public void setBorderLinePaint(Paint p) {
        border.setLinePaint(p);
        refreshable.refreshInteractiveGraphics();
    }

    public void setBorderLineStroke(Stroke s) {
        border.setStroke(s);
        refreshable.refreshInteractiveGraphics();
    }

    /** provides a visible/non visible toggle button, plus a border visible/non visible toggle button */
    public JComponent getToolGUI() {
        if (toolGUI != null) {
            return toolGUI;
        }
        JComponent p = super.getToolGUI();
        ComponentSizer.fixComponentSize(p, new Dimension(ThematicConfPanel.CONTENT_DIMENSION.width, 40));
        p.setBackground(Color.WHITE);
        p.add(new JLabel(getName(), 0));
        p.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, Color.BLACK));
//        //toolGUI.setBorder(BorderFactory.createLineBorder(Color.RED));

        final JToggleButton borderCB = new JToggleButton(Util.loadIcon(!border.isVisible() ? "noborder.png" : "border.png"), border.isVisible());
        borderCB.setToolTipText(i18n.get(this, "border", "Border: show/hide"));
        //borderCB.setBorder(null);
        borderCB.setBackground(Color.WHITE);
        ///borderCB.setContentAreaFilled(false);
        //borderCB.setOpaque(false);
        borderCB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                showBorder(borderCB.isSelected());
                borderCB.setIcon(Util.loadIcon(!border.isVisible() ? "noborder.png" : "border.png"));
                refreshGraphics();
                // System.out.println(OMResizeableFrame.this+" setBorder "+borderCB.isSelected()+ " = " + border.isVisible());
            }
        });
        ComponentSizer.fixComponentSize(borderCB, new Dimension(20, 20));
        //p.add(borderCB);
//        p.add(createStrokeEditor());
//        p.add(createBGColorButton());
        JComponent daGUI = (JComponent) da.getGUI();
        daGUI.setBackground(new Color(0, 0, 0, 0));
        daGUI.setBorder(null);
        daButton = (JButton) da.getGUI();
        if(!isVisibleBorder())
            daButton.setIcon(Util.loadIcon("empty.png"));
        p.add(daButton);
        //p.add(new JButton(Util.loadIcon("empty.png")));
        return p;
    }

    @Override
    public JComponent getGUI() {
        JComponent c = super.getGUI();

//        if (layout == null) {
//            layout = new BoxLayout(c, BoxLayout.Y_AXIS);
//        }
//        c.setLayout(layout);
//        //c.setBorder(BorderFactory.createLineBorder(Color.BLUE));
//        //ThematicConfPanel tcp = ThematicConfPanel.createThematicConfPanel(getName());
//        //tcp.setContent(c);
        if (isVisuFrame()) {
            LegendGUIManager guiManager = /*((LegendableLayer) refreshable)*/ThematicDataDisplayLayer.legendManager;
            ThematicConfPanel tcp = ThematicConfPanel.createThematicConfPanel(getName(), false);
            tcp.addCollapseListener(guiManager);
            JPanel wrap = new JPanel(new BorderLayout());
            wrap.add(c);
            tcp.setContent(wrap);
            if (guiManager.isOpened(getName())) {
                tcp.expand();
            } else {
                tcp.collapse();
            }
            return tcp;
        } else {
            return c;
        }
    }

    @Override
    protected void refreshGUI() {
        super.refreshGUI();
        if (guiPanel != null) {
            guiPanel.setBorder(null);
            visibleCB.setBackground(Color.WHITE);
            visibleCB.setText("");
        }
    }
    protected Box guiContent;

    public Component getGuiContent() {
        return guiContent;
    }
//    @Override
//    public void refreshGUI(){
//        if (guiPanel == null)
//	    return;
//        for (OMResizeableGraphic omrg : getResizeableGraphics()) {
//	    if (omrg != null) { // real self defense programming !-)
//                JComponent gui = omrg.getGUI();
//		if (gui != null) {
//                ThematicConfPanel tcp = ThematicConfPanel.createThematicConfPanel(getName());
//                tcp.setContent(gui);
//		guiPanel.add(tcp);
//		}
//	    }
//	}
//	guiPanel.updateUI();
//
//    }
    public static final String BORDER_PROPERTY_NAME = "border";
    public static final String BORDER_LINE_COLOR_NAME = "border_line_color";
    public static final String BORDER_FILL_COLOR_NAME = "border_fill_color";
    // public static final String CONTROL_PROPERTY_NAME = "control_angles";

    public void setProperties(String prefix, Properties properties) {
        // System.out.println(this+" props "+properties);
        super.setProperties(prefix, properties);
        if (properties == null) {
            return;
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //fillPaint = ColorFactory.parseColor(properties.getProperty(prefix + BORDER_FILL_COLOR_NAME, "#33FFFFFF"));
        //linePaint = ColorFactory.parseColor(properties.getProperty(prefix + BORDER_LINE_COLOR_NAME, "#FF000000"));
        da.setProperties(prefix, properties);
        showBorder(PropUtils.booleanFromProperties(properties, prefix + BORDER_PROPERTY_NAME, visibleBorder));
        setBorderFillPaint(da.getFillPaint());
        setBorderLinePaint(da.getLinePaint());
        setBorderLineStroke(da.getStroke());
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        properties = da.getProperties(properties);        
        properties.put(prefix + BORDER_PROPERTY_NAME, Boolean.toString(visibleBorder));

//        if(linePaint != null)
//            properties.put(prefix + BORDER_LINE_COLOR_NAME, ColorFactory.getHexColorString(linePaint));
//        if(fillPaint != null)
//            properties.put(prefix + BORDER_FILL_COLOR_NAME, ColorFactory.getHexColorString(fillPaint));


        return properties;
    }

    /** Set the visibility of the item of the graphic list according to their own visibility */
    protected void setGraphicsVisible(boolean v) {
        super.setGraphicsVisible(v);
        showBorder(visibleBorder);
    }

    public Component createStrokeEditor() {
        //tse = new ThemaMapStrokeEditorMenu(borderStroke, linePaint);
        //final ThemaMapStrokeEditorMenu tse = new ThemaMapStrokeEditorMenu(borderStroke, linePaint);
//        tse.getPropertyChangeSupport().addPropertyChangeListener(this);
//        ImageIcon icon = ThemaMapStrokeEditorMenu.createIcon(borderStroke, linePaint, 50, 25, true);
//        lineColButton = new JButton(icon);
//        lineColButton.setPreferredSize(new Dimension(20, 20));
//        lineColButton.setMaximumSize(new Dimension(20, 20));
//        lineColButton.setOpaque(false);
        //tse.setLaunchButton(lineColorB);
        /*lineColButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
        SwingUtilities.invokeLater(new Runnable() {
        public void run() {
        updateBorderLineColor();
        }
        });
        }
        });*/
//        lineColButton.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent ae) {
//                tse.show(lineColButton, 0, 0);
//            }
//        });
//        tse.setLaunchButton(lineColButton);
        return da.getGUI();
    }

//    public JButton createBGColorButton() {
//        bgColButton = new JButton();
//        ComponentSizer.fixComponentSize(bgColButton, new Dimension(20, 20));
//        bgColButton.setBackground(fillPaint);
//        bgColButton.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent arg0) {
//                SwingUtilities.invokeLater(new Runnable() {
//                    public void run() {
//                        updateFillColor();
//            }
//                });
//            }
//        });
//        return bgColButton;
//    }
//
    public void propertyChange(PropertyChangeEvent pce) {
        String propertyName = pce.getPropertyName();
        //System.out.println(propertyName + "; " + pce.getSource());
        if (propertyName.equals("stroke")) {
            setBorderLineStroke((Stroke) pce.getNewValue());
            return;
        }
        if (propertyName.equals("fillPaint")) {
            setBorderFillPaint((Paint) pce.getNewValue());
            return;
        }
        if (propertyName.equals("linePaint")) {
            setBorderLinePaint((Paint) pce.getNewValue());
            return;
        }
    }

    class OMResizeableFrameDrawingAttributes extends DrawingAttributes {

        private JCheckBoxMenuItem checkVisibility;
        private OMResizeableFrame frame;


        public OMResizeableFrameDrawingAttributes(OMResizeableFrame f) {
            super(true, false, false, true, true);
            this.frame = f;
        }

        @Override
        protected void setPostStrokeMenuOptions(JPopupMenu popup) {
            super.setPostStrokeMenuOptions(popup);
            checkVisibility = new JCheckBoxMenuItem(i18n.get(OMResizeableFrame.class, "border", "show border"));
            popup.add(checkVisibility);
            popup.updateUI();
            checkVisibility.setSelected(frame.isVisibleBorder());
            checkVisibility.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent arg0) {
                    frame.showBorder(checkVisibility.isSelected());
                    if(!frame.isVisibleBorder())
                        daButton.setIcon(Util.loadIcon("empty.png"));
                    else{
                        JComponent parent = (JComponent)daButton.getParent();
                        parent.remove(daButton);
                        daButton = (JButton)da.getGUI();
                        parent.add(daButton);
                        parent.updateUI();
                    }
                }
            });

        }

    }
//
//    public void updateBorderLineColor() {
//        linePaint = OMColorChooser.showDialog(lineColButton, "", linePaint);
//        ImageIcon icon = ThemaMapStrokeEditorMenu.createIcon(borderStroke, linePaint, 50, 25, true);
//        lineColButton.setIcon(icon);
//        setBorderLinePaint(linePaint);
//    }
//
//    public void updateFillColor() {
//        fillPaint = OMColorChooser.showDialog(bgColButton, "", fillPaint);
//        setBorderFillPaint(fillPaint);
//        bgColButton.setBackground(fillPaint);
//    }
}
