// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.swing.event.*;
import java.util.ArrayList;
import java.util.Properties;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.gui.WholeNumberField;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.awt.event.ActionListener;

import java.util.HashMap;
import java.util.concurrent.ExecutionException;

/**
 * Utility class to set an (x, y) offset
 * 2 cases:
 *  - an offset is defined for all graphics
 *  - each graphics has it own offset (stored in an hashmap)
 */
public class Offseter /*implements OffsetableListener*/ {

    private HashMap<String, OffsetLocation> offsets = new HashMap<String, OffsetLocation>();
    private String[] identificationCols = new String[0];
    private String[] columnNames;
    protected I18n i18n = new BasicI18n();
    private int x = 0, y = 0;
    private JComponent gui;
    private WholeNumberField XoffsetField;
    private WholeNumberField YoffsetField;
    private boolean cpVisible = false; // controlPoints visibility
    private boolean lineVisible = false;
    private boolean individual = false;
    private JCheckBox boxManual;
    private JCheckBox boxLine;
    private JButton bPal;

    public Offseter() {
        x = y = 0;
    }

    public Offseter(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public void setX(int x, String id) {
        offsets.get(id).x = x;
    }

    public void setY(int y, String id) {
        offsets.get(id).y = y;
    }

    public int getX(String id) {
        if (offsets.get(id) == null) {
            return getX();
        }
        return offsets.get(id).getX();

    }

    public int getY(String id) {
        if (offsets.get(id) == null) {
            return getY();
        }
        return offsets.get(id).getY();
    }

    public void setXY(String id, int x, int y) {
        setX(x, id);
        setY(y, id);
    }

    public void setX(int x) {
        this.x = x;
        fireOffseterChanged();
    }

    public void setY(int y) {
        this.y = y;
        fireOffseterChanged();
    }

    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }

    public void insert(String key) {
        offsets.put(key, new OffsetLocation(0, 0));

    }
    public static final String xOffsetPropertyString = "Xoffset";
    public static final String yOffsetPropertyString = "Yoffset";
    public static final String latOffsetPropertyString = "latOffset";
    public static final String lonOffsetPropertyString = "lonOffset";
    public static final String autoPropertyString = "auto";
    public static final String individualPropertyString = "individual";
    public static final String visiblePropertyString = "cpvisibility";
    public static final String lineVisiblePropertyString = "linevisibility";
    public static final String globalPropertyString = "global";
    public static final String nbOffsetPropertyString = "nb";
    public static final String idPropertyString = "id";
    public static final String nbIdentificationCols = "nbCols";
    public static final String identificationColNanme = "colName";

    /**
     * used to set the object with properties ressources
     */
    public void setProperties(String prefix, Properties properties) {
        x = PropUtils.intFromProperties(properties, prefix + xOffsetPropertyString, x);
        y = PropUtils.intFromProperties(properties, prefix + yOffsetPropertyString, y);
        cpVisible = Boolean.parseBoolean(properties.getProperty(prefix + visiblePropertyString, "false"));
        lineVisible = Boolean.parseBoolean(properties.getProperty(prefix + lineVisiblePropertyString, "false"));
        individual = Boolean.parseBoolean(properties.getProperty(prefix + individualPropertyString, "false"));
        //identification columns
        int nbCols = Integer.parseInt(properties.getProperty(prefix + nbIdentificationCols, "0"));
        identificationCols = new String[nbCols];
        for (int i = 0; i < nbCols; i++) {
            identificationCols[i] =
                    properties.getProperty(prefix + i + "." + identificationColNanme, "");
        }
        //offsets
        int nb = Integer.parseInt(properties.getProperty(prefix + nbOffsetPropertyString, "0"));
        for (int i = 0; i < nb; i++) {
            String id = properties.getProperty(prefix + i + "." + idPropertyString, "");
            int tmpX = Integer.parseInt(
                    properties.getProperty(prefix + id + "." + xOffsetPropertyString, "0"));
            int tmpY = Integer.parseInt(
                    properties.getProperty(prefix + id + "." + yOffsetPropertyString, "0"));
            offsets.put(id, new OffsetLocation(tmpX, tmpY));
        }
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties.put(prefix + xOffsetPropertyString, String.valueOf(x));
        properties.put(prefix + yOffsetPropertyString, String.valueOf(y));
        properties.put(prefix + visiblePropertyString, String.valueOf(cpVisible));
        properties.put(prefix + lineVisiblePropertyString, String.valueOf(lineVisible));
        properties.put(prefix + nbOffsetPropertyString, String.valueOf(offsets.size()));
        properties.put(prefix + nbOffsetPropertyString, String.valueOf(offsets.size()));
        properties.put(prefix + individualPropertyString, String.valueOf(individualPropertyString));
        properties.put(prefix + nbIdentificationCols, String.valueOf(identificationCols.length));
        int indexCol = 0;
        for (String colName : identificationCols) {
            properties.put(prefix + indexCol + "." + identificationColNanme, colName);
            indexCol++;
        }
        int index = 0;
        for (String key : offsets.keySet()) {
            properties.put(prefix + index + "." + idPropertyString, key);
            properties.put(prefix + key + "." + xOffsetPropertyString, String.valueOf(offsets.get(key).getX()));
            properties.put(prefix + key + "." + yOffsetPropertyString, String.valueOf(offsets.get(key).getY()));
            index++;
        }
        return properties;
    }
//    private final static String PAN_AUTO = "panAuto";
//    private final static String PAN_MANUAL = "panManual";
//    private final static String PAN_XY = "panXY";
//    private final static String PAN_INDIV = "panIndiv";

    public JComponent getGUI() {
        if (gui != null) {
            return gui;
        }
        gui = Box.createVerticalBox();
        gui.add(getGlobalGUI());
        JPanel panKey = new JPanel();
        panKey.add(new JLabel(i18n.get(Offseter.class, "individual", "Individual offset")));
        JButton bKey = new JButton(i18n.get(Offseter.class, "choosekey", "Choose a key"));
        panKey.add(bKey);
        bKey.setBackground(Color.WHITE);
        bKey.setOpaque(false);
        gui.add(panKey);

        JPanel panOffset = new JPanel();
        panOffset.add(new JLabel(i18n.get(Offseter.class, "pal", "Automatic placement")));
        bPal = new JButton(i18n.get(Offseter.class, "apply", "Apply"));
        bPal.setBackground(Color.WHITE);
        bPal.setOpaque(false);
        panOffset.add(bPal);
        boxManual = new JCheckBox(i18n.get(Offseter.class, "showControlPoints", "Show control points"));
        boxLine = new JCheckBox(i18n.get(Offseter.class, "showLines", "Display lines"));
        JPanel panManual = new JPanel(new GridLayout(2, 1));
        panManual.add(boxManual);
        panManual.add(boxLine);

        JPanel panManualWrap = new JPanel();
        panManualWrap.add(panManual);
        panManualWrap.setBorder(BorderFactory.createEmptyBorder(0, 15, -5, 15));
        gui.add(panManualWrap);
        gui.add(panOffset);

        ComponentSizer.fixComponentSize(bPal, new Dimension(120, 20));
        ComponentSizer.fixComponentSize(bKey, new Dimension(120, 20));

        //panManual.setBorder(BorderFactory.createEmptyBorder( 0, 5, 5, 0));
        bKey.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                KeyDialog keyDialog = new KeyDialog();
                keyDialog.setVisible(true);
            }
        });
        enableManualGui(identificationCols.length > 0);
        boxManual.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                setVisible(boxManual.isSelected());
                //individual = boxManual.isSelected();
            }
        });
        boxLine.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                setLineVisible(boxLine.isSelected());
            }
        });

        boxLine.setSelected(lineVisible);
        boxManual.setSelected(cpVisible);
   

        return gui;
    }

    public JComponent getGlobalGUI() {
        //JPanel pGlobalX = new JPanel();
        //JPanel pGlobalY = new JPanel();
        JPanel globalGui = new JPanel();
        JLabel labelG = new JLabel(i18n.get(Offseter.class, "globalOffset", "Global offset: "));
        JLabel labelX = new JLabel(i18n.get(Offseter.class, xOffsetPropertyString, "X offset: "));
        JLabel labelY = new JLabel(i18n.get(Offseter.class, yOffsetPropertyString, "Y offset: "));

        XoffsetField = new WholeNumberField(x, 5);
        XoffsetField.getDocument().putProperty("name", "X offset");
        XoffsetField.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                setX(XoffsetField.getValue());
            }

            public void removeUpdate(DocumentEvent e) {
                setX(XoffsetField.getValue());
            }

            public void changedUpdate(DocumentEvent e) {
                setX(XoffsetField.getValue());
            }
        });
        //JLabel labelX = new JLabel(i18n.get(Offseter.class, xOffsetPropertyString, "X offset: "));
        //pGlobalX.add(labelX);
        //pGlobalX.add(XoffsetField);
        YoffsetField = new WholeNumberField(y, 5);
        YoffsetField.getDocument().putProperty("name", "Y offset");
        YoffsetField.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                setY(YoffsetField.getValue());
            }

            public void removeUpdate(DocumentEvent e) {
                setY(YoffsetField.getValue());
            }

            public void changedUpdate(DocumentEvent e) {
                setY(YoffsetField.getValue());
            }
        });
        globalGui.add(labelG);
        globalGui.add(labelX);
        globalGui.add(XoffsetField);
        globalGui.add(labelY);
        globalGui.add(YoffsetField);
        /*JLabel labelY = new JLabel(i18n.get(Offseter.class, yOffsetPropertyString, "Y offset: "));
        pGlobalY.add(labelY);
        pGlobalY.add(YoffsetField);
        JPanel boxXY = new JPanel(new GridLayout(2, 1));
        boxXY.add(pGlobalX);
        boxXY.add(pGlobalY);*/

        return globalGui;

    }
    private ArrayList<OffseterListener> listenerList = new ArrayList<OffseterListener>();

    public void addOffseterListener(OffseterListener l) {
        if (!listenerList.contains(l)) {
            listenerList.add(l);
        }
    }

    public void removeOffseterListener(OffseterListener l) {
        listenerList.remove(l);
    }

    protected void fireOffseterChanged() {
        for (int i = 0; i < listenerList.size(); i++) {
            listenerList.get(i).offseterChanged(this);
        }
    }

    public void setColumnNames(String[] colNames) {
        columnNames = colNames;
    }

//    public boolean isIndividual() {
//        return individual;
//    }
    public String[] getIdentificationCols() {
        //System.out.println(" => " + hashCode());
        return identificationCols;
    }

    public void setOffset(String id, int xOffset, int yOffset) {
        //System.out.println("MOVE DE " + id);
        if (!contains(id)) {
            insert(id);
        }
        //Point2D p = projection.forward(offsets.get(id).getLat(), offsets.get(id).getLon());
        //int xOffset = (int) (x - p.getX());
        //int yOffset = (int) (y - p.getY());
        //System.out.println("move "+id+" ->" + xOffset +";" + yOffset);
        setXY(id, xOffset, yOffset);
    }

    public boolean contains(String key) {
        return offsets.containsKey(key);
    }

    public String buildKey(DataObject dataObject) {
        String key = "";
        if (identificationCols == null || identificationCols.length == 0) {
            return null;
        }
        for (int i = 0; i < identificationCols.length; i++) {
            key += dataObject.getData(identificationCols[i].toString());
        }
        return key;
    }

    /**
     * Stores original latitude/longitude, and offset positions
     */
    class OffsetLocation {

        private int x;
        private int y;

        public OffsetLocation(int x, int y) {
            this.x = x;
            this.y = y;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }

        public void setX(int x) {
            this.x = x;
        }

        public void setY(int y) {
            this.y = y;
        }
        public static final String latPropertyString = "lat";
        public static final String lonPropertyString = "lon";
        public static final String xPropertyString = "x";
        public static final String yPropertyString = "y";
        public static final String idPropertyString = "id";

        public void setProperties(String prefix, Properties properties) {
            prefix = PropUtils.getScopedPropertyPrefix(prefix);
            x = Integer.parseInt(properties.getProperty(prefix + xPropertyString, "0"));
            y = Integer.parseInt(properties.getProperty(prefix + yPropertyString, "0"));
        }

        public Properties getProperties(String prefix, Properties properties) {
            prefix = PropUtils.getScopedPropertyPrefix(prefix);
            properties.put(prefix + xPropertyString, String.valueOf(x));
            properties.put(prefix + yPropertyString, String.valueOf(y));

            return properties;
        }
    }


    public boolean isVisible() {
        return cpVisible;
    }

    public void setVisible(boolean b) {
        cpVisible = b;
    }

    public void setLineVisible(boolean b) {
        lineVisible = b;
    }

    public boolean isLineVisible() {
        return lineVisible;
    }

    public void enableManualGui(boolean b) {
        boxLine.setEnabled(b);
        boxManual.setEnabled(b);
        bPal.setEnabled(b);

    }

    class KeyDialog extends JDialog {

        JList keyList;

        public KeyDialog() {
            super();
            setModal(true);
            setSize(350, 280);
            JPanel main = new JPanel(new BorderLayout());
            add(main);
            setTitle(i18n.get(Offseter.class, "keyChoose","Choose a key"));
            keyList = new JList(columnNames);
            JButton okList = new JButton("ok");
            if(columnNames != null){
                int [] indices = indicesToSelect();
                if(indices.length > 0)
                    keyList.setSelectedIndices(indices);
            }
            okList.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    Object[] objs = keyList.getSelectedValues();
                    offsets.clear();
                    identificationCols = new String[objs.length];
                    for (int i = 0; i < objs.length; i++) {
                        identificationCols[i] = objs[i].toString();
                    }
                    fireOffseterChanged();
                    dispose();
                    enableManualGui(identificationCols.length > 0);
                    individual = identificationCols.length > 0;
                }
            });
            main.add(new JLabel(i18n.get(Offseter.class, "keyText", "Choose a or several column(s) as key")), BorderLayout.NORTH);
            main.add(new JScrollPane(keyList), BorderLayout.CENTER);
            JPanel panSouth = new JPanel();
            panSouth.add(okList);
            main.add(panSouth, BorderLayout.SOUTH);

        }

        private boolean isSelectedColumnName(String s){
            //System.out.println(s + " isSelected?");
            for(String sTemp : identificationCols){
                if(sTemp.equals(s))
                    return true;
            }
            return false;
        }

        private int[] indicesToSelect(){
            int[] indices = new int[identificationCols.length];
            int index = 0;
            //System.out.println("Column names = " + columnNames);
            for(int i = 0; i < columnNames.length; i++){
                if(isSelectedColumnName(columnNames[i])){
                    indices[index] = i;
                    index ++;
                }
            }
            return indices;

        }
    }
}

