// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import java.util.*;
import java.awt.event.*;

import javax.swing.*;


import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.classification.Quantile;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Dimension;

/**
 * Disc visualization mode.
 *
 */
public class Area extends Raw implements ClassificationListener {

    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";
    protected String colorDataColumnName = "";
    protected Colorizer colorizer = new Colorizer();
    //private JComponent panelGUI;

    public Area(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Area.class, "coloredArea", "Colored Area"));
    }

    public Area(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        colorizer.addClassificationListener(this);
    }

    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, false);
    }

    public void setProperties(String prefix, Properties properties) {
        //System.err.println("[ThematicDataDisplayLayer.Area] setProperties");
        super.setProperties(prefix, properties);
        colorizer.setProperties(prefix, properties);
        colorizer.addClassificationListener(this);
        colorDataColumnName = properties.getProperty(prefix + colorDataColumnNamePropertyString, "data");
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = colorizer.getProperties(prefix, properties);
        properties.put(prefix + colorDataColumnNamePropertyString, colorDataColumnName);

        return properties;
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        try {
            colorizer.setData(dsi, colorDataColumnName);
            OMGraphicList omgl = new OMGraphicList();

            while (dsi.hasNext()) {
                DataObject data = dsi.next();
                OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
                if (omdl == null) {
                    continue;
                }

                OMGraphic geom = omdl.getOMGraphic();

                if (geom == null && omdl.getCentroid() != null) {
                    geom = new OMPoint(Double.valueOf(omdl.getCentroid().getLat()).floatValue(), Double.valueOf(omdl.getCentroid().getLon()).floatValue());
                }


                if (geom == null) {
                    continue;
                }

                Object value = data.getData(colorDataColumnName);

                // clones may consume resource. TODO: a property ?
//	    OMGraphic omg = clone(geom);
//	    omg.setLinePaint(colors.getLineColor());
//	    omg.setFillPaint(colorizer.getColor(value));
//	    omg.putAttribute(ThematicDataDisplayLayer.VALUE, value);
//	    listClones.setLinePaint(colors.getLineColor());
//	    listClones.setFillPaint(colorizer.getColor(value));
//	    listClones.putAttribute(ThematicDataDisplayLayer.VALUE, value);
//
//	    omgl.add(listClones);
//                if (geom instanceof OMLine) {
//                    geom.setLinePaint(drawingAttributes.getLinePaint()/*colorizer.getColor(value)*/);
//                } else {
//                    geom.setLinePaint(/*colors.getLineColor()*/drawingAttributes.getLinePaint());
//                    geom.setFillPaint(colorizer.getColor(value));
//                    geom.setStroke(drawingAttributes.getStroke());
//                }

                paintGraphic(geom, value);
                geom.setStroke(drawingAttributes.getStroke());
                geom.putAttribute(ThematicDataDisplayLayer.VALUE, value);
                omgl.add(geom);
            }
            return omgl;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private void paintGraphic(OMGraphic omg, Object value) {
        if (omg instanceof OMGraphicList) {
            for (OMGraphic o : (OMGraphicList) omg)
                paintGraphic(o, value);
            return;
        }
	//        boolean fill = false;
	//        if (omg instanceof OMRect)
	//            fill = true;
	//        else if (omg instanceof OMPoly) {
	//            OMPoly poly = (OMPoly) omg;
	//            if (poly.isPolygon()) {
	//                fill = true;
	//            }
	//            double points[] = poly.getLatLonArray();
	//            if (points[0] != points[points.length - 2] ||
	//                    points[1] != points[points.length - 1])
	//                fill = false;
	//
	//        }
	boolean fill = true; // default is to fill
	if (omg instanceof OMLine)
	    fill = false;
	else {
	     if (omg instanceof OMPoly) {
		 OMPoly poly = (OMPoly) omg;
		 if (! poly.isPolygon()) {
		     double points[] = poly.getLatLonArray();
		     if (points[0] != points[points.length - 2] ||
			 points[1] != points[points.length - 1])
			 fill = false;
		 }
	     }
	}    

        if (fill) {
            omg.setLinePaint(drawingAttributes.getLinePaint());
            omg.setFillPaint(colorizer.getColor(value));
        } else {
            omg.setLinePaint(colorizer.getColor(value));
            omg.setFillPaint(drawingAttributes.getFillPaint());
        }


    }

    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        try {
            graphicsPanelToggleControlsDisplay(false, true, false, false, false);
            //Box panStyle = Box.createHorizontalBox();
            //JLabel lab = new JLabel(i18n.get(Raw.class, "line", "lines") + "     ");
            //panStyle.add(lab);
            // panStyle.add(lineButton);
            //graphicsPanel.add(panStyle);

            final JPanel coloredAreaPanel = new JPanel();
            //coloredAreaPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "areaTitle", "Colored Area")));
            coloredAreaPanel.setLayout(new BoxLayout(coloredAreaPanel, BoxLayout.Y_AXIS));

            final ColorizerPanel colorizerPanel = colorizer.getGUI();
            //JPanel colorDataPanel = new JPanel(); // used to set and display colorDataColumnName
            //colorDataPanel.setLayout(new BoxLayout(colorDataPanel, BoxLayout.X_AXIS));
            //colorDataPanel.add(new JLabel(i18n.get(this, "colorDataColumnName", "Column")));
            initDataList();
            //colorDataPanel.add(dataList);
            //colorDataPanel.add(Box.createHorizontalGlue());

            //coloredAreaPanel.add(colorDataPanel);
            JPanel p0 = (JPanel) colorizerPanel.getComponent(0);
            p0.add(dataList, 0);
            p0.add(new JLabel(i18n.get(this, "colorDataColumnName", "Column")), 0);
            coloredAreaPanel.add(colorizerPanel);

            ThematicConfPanel confp = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "classif", "Classification"), false);
            confp.setContent(coloredAreaPanel);
            JPanel wrapConf = new JPanel();
            wrapConf.add(confp);
            thematicGUI.add(wrapConf);
            thematicGUI.addThematicConfigurationPanel(confp);
            //coloredAreaPanel.setBorder(BorderFactory.createTitledBorder(""));

            if (colorizer.getClassification() == null) {
                Quantile quantile = new Quantile(null, 4);
                quantile.reInit(dsi, colorDataColumnName);
                colorizer.setClassification(quantile);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return thematicGUI;
    }
    protected JComboBox dataList;

    public String getDataColumnName() {
        return colorDataColumnName;
    }

    public void setDataColumnName(String name) {
        if (name != null && !name.equals(colorDataColumnName)) {
            colorDataColumnName = name;
            classificationChanged(this);
        }
        refresh();
    }

    //    protected OMResizeableColorScaleLegend omcsl;
    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        OMResizeableFrame frame = new OMResizeableColorScaleLegend(r, ml, 0, 0,
                colorizer.getColorScale(), colorizer.getColorsTexts(),
                colorDataColumnName, getFullName());
        frame.setVisuFrame(true);
        return frame;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        initDataList();
    }

    /** init or reinit the comboBox used to choose dataColumnName */
    private void initDataList() {

        DataStorageIterator dsi = getLayer().getDataStorageIterator();
        if (dataList == null) {
            dataList = new JComboBox(dsi.getDataColumnNames());
        } else {
            for (ActionListener a : dataList.getActionListeners()) {
                dataList.removeActionListener(a);
            }
            dataList.removeAllItems();
            for (String item : dsi.getDataColumnNames()) {
                dataList.addItem(item);
            }
        }
        //System.err.println("DATA COLUMN NAME" + getDataColumnName());
        if (!"".equals(getDataColumnName()) && getDataColumnName() != null) {
            dataList.setSelectedIndex(
                    dsi.getIndexForColumnName(getDataColumnName()));
        } else {
            if (dsi.getDataColumnNames().length > 0 && dsi.getDataColumnNames()[0].length() > 0) {
                colorDataColumnName = dsi.getDataColumnNames()[0];
            }
        }

        dataList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    setDataColumnName(getLayer().getDataStorageIterator().getRealColumnName(index));
                    //colorizer.setColorNumberModel(colorizer.getClassification().getClassificationNumberModel());
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });
        ComponentSizer.fixComponentSize(dataList, new Dimension(120, 20));
    }

    public void classificationChanged(Object source) {
        Debug.message("ThematicDataDisplayLayer", this + " => classificationChanged(@" + source.hashCode() + "  " + source + ")" +
                "colorDataColumn: " + colorDataColumnName);
        if (source != colorizer) {
            Debug.message("ThematicDataDisplayLayer", this + " <> classificationChanged(" + source + ")");
            colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName);
        }
        refresh();
        Debug.message("ThematicDataDisplayLayer", this + " <= classificationChanged(" + source + ")");
    }

    public boolean useSharedGraphics() {
        return this instanceof Area;
    }
    // public void refresh() {
    // 	super.refresh();
    // 	// if (colorizer != null && colorizer.getClassification() != null) {
    // 	//     Debug.message("Histogram", " " + this + " => Classification.unsetData");
    // 	//     colorizer.getClassification().unsetData();
    // 	// }
    // }
//    /**
//     * Clone an OMGraphicList in an other one - only OMPoly are cloned
//     */
//    private void cloneOMPolyInList(OMGraphicList lstFrom, OMGraphicList lstTo){
// 
//    	for(int i = 0; i < lstFrom.size(); i++){
//    		if(lstFrom.getOMGraphicAt(i) instanceof OMGraphicList){
//    			cloneOMPolyInList((OMGraphicList)(lstFrom.getOMGraphicAt(i)), lstTo);
//    		}else{
//    			if(lstFrom.getOMGraphicAt(i) instanceof OMPoly){
//    				lstTo.add(cloneOMPoly((OMPoly)(lstFrom.getOMGraphicAt(i))));
//    			}
//    		}   		
//    	}
//    }
//    
//    /**
//     * OMPoly shape clone
//     * @param source
//     * @return
//     */
//    private  OMPoly cloneOMPoly(OMPoly source){
//    	return  new OMPoly(
//				source.getLatLonArray(),
//				source.getUnits(), source.getLineType());
//    }
}
