// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphicList;
import java.util.*;
import java.awt.event.*;

import javax.swing.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

import com.bbn.openmap.util.*;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import java.awt.BasicStroke;
import java.awt.CardLayout;
import java.awt.Stroke;

/**
 * Histogram visualization mode.
 */
public class Histogram extends OffsetableRaw implements AttributesNames, ResizerListener, /*OffseterListener,*/ ColorScaleListener {

    public enum Orientation {

        LEFT(i18n.get(Histogram.class, "left", "left")),
        UP(i18n.get(Histogram.class, "up", "up")),
        RIGHT(i18n.get(Histogram.class, "right", "right")),
        DOWN(i18n.get(Histogram.class, "down", "down"));
        private final String text;

        Orientation(String text) {
            this.text = text;
        }

        @Override
        public String toString() {
            return this.text;
        }
    }
    public static final String colorDataColumnNamesPropertyString = "colorDataColumnNames";
    public static final String barsWidthPropertyString = "barsWidth";
    public static final String barsPropertyString = "bars";
    public static final String orientationPropertyString = "orientation";
    /** names of the column used to draw the histogram bars */
    protected String[] colorDataColumnNames = new String[0];
    protected double barsWidth = 10d;
    protected Orientation orientation = Orientation.UP;
    protected Resizer resizer = new Resizer();
    //protected Offseter offseter = new Offseter(0, 0);
    protected ColorScale colorScale = new ColorScale();
    protected Boolean bars = true;
    protected Stroke scaleStroke = new BasicStroke(1);
    protected ThemaMapStrokeEditorMenu tse;
    protected MinMaxValues minMaxValues;

    public Histogram(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Histogram.class, "histogram", "Histogram"));
    }

    public Histogram(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer.addResizerListener(this);
        //offseter.addOffseterListener(this);
        interactive = true;
    }

    @Override
    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, false);
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        barsWidth = PropUtils.doubleFromProperties(properties, prefix + barsWidthPropertyString, 0.1);
        try {
            bars = PropUtils.booleanFromProperties(properties, prefix + barsPropertyString, true);
            resizer.setProperties(prefix, properties);
            resizer.addResizerListener(this);
            colorScale.setProperties(prefix, properties);
            colorDataColumnNames = properties.getProperty(prefix + colorDataColumnNamesPropertyString, "").split("[ \t]+");
            orientation = Orientation.valueOf(properties.getProperty(prefix + orientationPropertyString, "UP"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties.put(prefix + barsWidthPropertyString, String.valueOf(barsWidth));
        properties.put(prefix + barsPropertyString, String.valueOf(bars));
        properties = resizer.getProperties(prefix, properties);
        properties = colorScale.getProperties(prefix, properties);
        StringBuffer colorDataColumnNamesString = new StringBuffer();
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            colorDataColumnNamesString.append(colorDataColumnNames[i]);
            colorDataColumnNamesString.append(" ");
        }
        properties.put(prefix + colorDataColumnNamesPropertyString, String.valueOf(colorDataColumnNamesString));
        properties.put(prefix + orientationPropertyString, orientation.name());
        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {

        mouseControlPointsListener.clearControlPoints();
        // may be we should use DataValueComputer? But like this, it works fine!
        resizer.setMinMaxValue();
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            resizer.adjustMinMaxValue(dsi, colorDataColumnNames[i]);
        }

        OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMHistogram>();

        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }
            double values[] = new double[colorDataColumnNames.length];
            boolean valuesAreAllNull = true;
            for (int i = 0; i < colorDataColumnNames.length; i++) {
                Object data_i = data.getData(colorDataColumnNames[i]);
                if (data_i != null && !"".equals(data_i.toString())) {
                    valuesAreAllNull = false;
                }
                values[i] = resizer.getSize(Util.parseDouble(data.getData(colorDataColumnNames[i])));
            }

            if (valuesAreAllNull) {
                Debug.message("ThematicDataDisplayLayer", "no Histo because all values are nul for: " + data);
                continue;
            }

            OMHistogram histo = null;
            if (bars) {
                float lat = (float) centroid.getLat();
                float lon = (float) centroid.getLon();
                int offX = 0;
                int offY = 0;
                String key = offseter.buildKey(data);
//                if (offseter.isIndividual()) {
//                    key = offseter.buildKey(data);
//                    if (!offseter.contains(key)) {
//                        offseter.insert(key, lat, lon);
//                    }
//                }
                offX = offseter.getX(key);
                offY = offseter.getY(key);
                omgl.add(histo = new OMHistogram(this, key, lat, lon,
                        //offseter.getX(), offseter.getY(),
                        offX, offY,
                        (float) barsWidth, drawingAttributes.getLinePaint(),
                        values, colorScale, (int) Math.ceil(resizer.getMaxSize()),
                        orientation, offseter.isVisible(), offseter.isLineVisible()));
                //histo.addOffsetableListener(offseter);
                mouseControlPointsListener.addOMControlPoints(histo.controlPoints);
                //omgl.add(histo.controlPoints);
            } else {
                omgl.add(new OMGraph((float) centroid.getLat(), (float) centroid.getLon(),
                        offseter.getX(), offseter.getY(),
                        (float) barsWidth, drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),
                        values, (int) Math.ceil(resizer.getMaxSize()),
                        orientation));
            }
            ///mouseControlPointsListener.addOMControlPoints(.getControlPoints());
        }
        omgl.sort();
        initMinMaxValues();
        return omgl;
    }

    protected void initMinMaxValues() {
        minMaxValues =
                new MinMaxValues(0, // resizer.getValuePerPixel()*(int) Math.floor(resizer.getMinSize()),
                resizer.getValuePerPixel() * (int) Math.ceil(resizer.getMaxSize()));

    }

    protected void setBarsWidth(double d) {
        barsWidth = d;
        refresh();
    }

    //private JCheckBox[] colorDataColumnNamesButtons = null;
    //private ItemListener colorDataColumnNamesButtonsListener = null;
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        //System.out.println(dsi.getDataColumnNames().length+ " ----------------");
        //offseter.setColumnNames(dsi.getDataColumnNames());
        graphicsPanelToggleControlsDisplay(false, true, false, false, false);
        //JPanel histogramPanel = new JPanel();
        //histogramPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"histogramTitle","Histogram")));
        //histogramPanel.setLayout(new BoxLayout(histogramPanel, BoxLayout.X_AXIS));

        final JSpinner barsWidthSpinner =
                new JSpinner(new SpinnerNumberModel((double) barsWidth, 0d, 10000d, 0.1d));
        barsWidthSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setBarsWidth(((Double) barsWidthSpinner.getValue()).doubleValue());
            }
        });
        final JPanel sizePanel = new JPanel();
        sizePanel.setBorder(BorderFactory.createTitledBorder(""));
        sizePanel.add(new JLabel(i18n.get(this, "barsWidthLabel", "Bars Width")), "label");
        sizePanel.add(barsWidthSpinner, "field");

        sizePanel.add(resizer.getGUI(dsi));

        //sizePanel.add(offseter.getGUI());

        ThematicConfPanel sizeConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "size", "Size"));
        sizeConfPan.setContent(sizePanel);
        thematicGUI.addThematicConfigurationPanel(sizeConfPan);
        JPanel wrapSize = new JPanel();
        wrapSize.add(sizeConfPan);
        thematicGUI.add(wrapSize);



        colorScalePanel =
                new AttributesColorScalePanel(colorScale, (AttributesNames) this) {

                    public void actionPerformed(ActionEvent event) {
                        super.actionPerformed(event);
                        refresh();
                    }
                };
        colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
        colorScalePanel.refreshPanel();
        colorScalePanel.addColorScaleListener(this);
        curveColorScalePanel =
                new CurveColorScalePanel(colorScale, (AttributesNames) this) {

                    public void actionPerformed(ActionEvent event) {
                        super.actionPerformed(event);
                        refresh();
                    }
                };
        curveColorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
        curveColorScalePanel.refreshPanel();

        final JPanel scalesPanel = new JPanel(new CardLayout());
        JPanel mainPanel = new JPanel(new BorderLayout());
        JPanel pTypeChoice = new JPanel();
        JRadioButton bBar = new JRadioButton(i18n.get(Histogram.class, "histogram", "Bars"));
        JRadioButton bCurve = new JRadioButton(i18n.get(Histogram.class, "curves", "Curves"));
        ButtonGroup bg = new ButtonGroup();
        bg.add(bBar);
        bg.add(bCurve);

        if (bars) {
            bBar.setSelected(true);
        } else {
            bCurve.setSelected(true);
        }

        pTypeChoice.add(bBar);
        pTypeChoice.add(bCurve);

        final JComboBox orientationBox = new JComboBox(Orientation.values());
        orientationBox.setSelectedItem(orientation);
        orientationBox.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent evt) {
                JComboBox cb = (JComboBox) evt.getSource();
                orientation = (Orientation) cb.getSelectedItem();
            }
        });

        bBar.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                bars = ((JRadioButton) e.getSource()).isSelected();
                CardLayout cl = (CardLayout) scalesPanel.getLayout();
                cl.show(scalesPanel, "histo");
                thematicGUI.setVisible(false);
                thematicGUI.setVisible(true);
                orientationBox.setEnabled(bars);
            }
        });

        bCurve.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                bars = !((JRadioButton) e.getSource()).isSelected();
                CardLayout cl = (CardLayout) scalesPanel.getLayout();
                cl.show(scalesPanel, "curve");
                thematicGUI.setVisible(false);
                thematicGUI.setVisible(true);
                orientationBox.setEnabled(bars);
            }
        });


        pTypeChoice.add(new JLabel("Orientation: "));
        pTypeChoice.add(orientationBox);

        pTypeChoice.setBorder(BorderFactory.createEmptyBorder(-10, 0, -10, 0));
        mainPanel.add(pTypeChoice, BorderLayout.NORTH);
        scalesPanel.add(colorScalePanel, "histo");
        scalesPanel.add(curveColorScalePanel, "curve");
//        mainPanel.add(new JScrollPane(scalesPanel,
//                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
//                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
        mainPanel.add(scalesPanel, BorderLayout.CENTER);



        ThematicConfPanel histoConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "attributes_choice", "Attributes choice"), false);
        /*histoConfPan.setContent(new JScrollPane(mainPanel,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER
        ));*/
        histoConfPan.setContent(mainPanel);
        thematicGUI.addThematicConfigurationPanel(histoConfPan);
        JPanel wrapHisto = new JPanel();
        wrapHisto.add(histoConfPan);
        thematicGUI.add(wrapHisto);

        //thematicGUI.add(histogramPanel);

        return thematicGUI;
    }

    private boolean isAColorDataColumnNames(String columnName) {
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            if (colorDataColumnNames[i].equals(columnName)) {
                return true;
            }
        }
        return false;
    }

    protected void setColorDataColumnNames(String[] colorDataColumnNames) {
        this.colorDataColumnNames = colorDataColumnNames;
    }

    /** gives colorDataColumnName[i]. @see Names interface */
    public String getName(int i) {
        return i >= 0 && i < colorDataColumnNames.length ? colorDataColumnNames[i] : "";
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        OMResizeableFrame frame = new OMResizeableHistogramLegend(r, ml, 0, 0,
                resizer, minMaxValues, bars ? barsWidth : 2, drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0),
                bars ? colorScale : null, colorDataColumnNames, getFullName());
        frame.setVisuFrame(true);
        return frame;
    }
    protected AttributesColorScalePanel colorScalePanel;
    protected CurveColorScalePanel curveColorScalePanel;

    //when we change dsi, data change so... refresh list *and* colorscale
    @Override
    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        if (colorScalePanel != null) {
            colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
            actualiseAttributesNames(dsi);
            colorScalePanel.refreshPanel(); // redraws all
        }
    }

    /** @see AttributesNames interface */
    public void setAttributesNames(String[] names) {
        setColorDataColumnNames(names);
    }

    /** remove names from AttributesNames that are no more attributes */
    private void actualiseAttributesNames(DataStorageIterator dsi) {
         setAttributesNames(colorDataColumnNames);
//        ArrayList<String> names = new ArrayList<String>();
//        for (String name : colorDataColumnNames) {
//            for (String attribute : dsi.getDataColumnNames()) {
//                if (name.equals(attribute)) {
//                    names.add(name);
//                    break;
//                }
//            }
//        }
//        setAttributesNames(names.toArray(new String[names.size()]));
    }

    protected ArrayList<String> getNoColorDataColumnNames(DataStorageIterator dsi) {
        ArrayList<String> names = new ArrayList<String>();
        for (String name : dsi.getDataColumnNames()) {
            if (!isAColorDataColumnNames(name)) {
                names.add(name);
            }
        }
        return names;
    }

    public void resizerChanged(Resizer resizer) {
        if (this.resizer == resizer) {
            refresh();
        }
    }

//   public void offseterChanged(Offseter offseter) {
//        if (this.offseter == offseter) {
//            refresh();
//        }
//    }

    /*public void colorScaleChanged(ColorScalePanel2 colorScalePanel) {
    if (this.colorScalePanel == colorScalePanel)
    refresh();
    }*/
    public void colorScaleChanged(ColorScalePanel colorScalePanel) {
        if (this.colorScalePanel == colorScalePanel) {
            refresh();
        }
    }
//    @Override
//    public void propertyChange(PropertyChangeEvent pce) {
//        if (pce.getSource() instanceof ThemaMapStrokeEditorMenu) {
//            ThemaMapStrokeEditorMenu tsem = (ThemaMapStrokeEditorMenu) pce.getNewValue();
//            if (tsem != bse) {
//                scaleStroke = tsem.getStroke();
//                colors.setAxesLineColor(tsem.getLineColor());
//            } else {
//                setStroke(tsem.getStroke());
//                colors.setLineColor(tsem.getLineColor());
//                graphicsPanel.updateSamplePanel();
//            }
//        }
//    }
//    public JButton createStrokeEditor() {
//        tse = new ThemaMapStrokeEditorMenu((BasicStroke) scaleStroke, colors.getAxesLineColor());
//        tse.getPropertyChangeSupport().addPropertyChangeListener(this);
//        ImageIcon icon = ThemaMapStrokeEditorMenu.createIcon((BasicStroke) scaleStroke, colors.getAxesLineColor(), 50, 25, true);
//        final JButton axeColorB = new JButton(icon);
//        axeColorB.setPreferredSize(new Dimension(40, 20));
//        axeColorB.setMaximumSize(new Dimension(40, 20));
//        axeColorB.setBackground(Color.WHITE);
//        axeColorB.setOpaque(false);
//        tse.setLaunchButton(axeColorB);
//        axeColorB.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent ae) {
//                tse.show(axeColorB, 0, 0);
//            }
//        });
//        return axeColorB;
//    }
}
