// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.Environment;
import java.util.*;
import java.awt.event.*;

import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.FlowLayout;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.*;
import java.awt.Dimension;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Pie visualization mode.
 * <p>Draw pies in place of discs. The property radiusIsSum fixes the disc dimension. 
 */
public class Pie extends Disc implements AttributesNames, ColorScaleListener {

    public static final String colorDataColumnNamesPropertyString = "colorDataColumnNames";
    public static final String radiusIsSumPropertyString = "radiusIsSum";
    /** names of the column used to draw the pie portions */
    protected String[] colorDataColumnNames = new String[0];
    protected double[] generalAngles;
    protected ColorScale colorScale = new ColorScale();
    protected boolean radiusIsSum = false;
    protected boolean badLegendCache = false;

    public Pie(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Pie.class, "pie", "Pie"));
    }

    public Pie(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer.setWithRadius(true);
        //offseter.addOffseterListener(this);
        interactive = true;
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        colorScale.setProperties(prefix, properties);
        colorDataColumnNames = properties.getProperty(prefix + colorDataColumnNamesPropertyString, "").split("[ \t]+");
        radiusIsSum = Util.isTrue(properties.getProperty(prefix + radiusIsSumPropertyString, new Boolean(radiusIsSum).toString()));
        drawingAttributes.setFillPaint(new Color(0, 0, 0, 0));
        //colors.setFillColor(new Color(0, 0, 0, 0));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = colorScale.getProperties(prefix, properties);
        StringBuffer colorDataColumnNamesString = new StringBuffer();
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            colorDataColumnNamesString.append(colorDataColumnNames[i]);
            colorDataColumnNamesString.append(" ");
        }
        properties.put(prefix + colorDataColumnNamesPropertyString, colorDataColumnNamesString.toString());
        properties.put(prefix + radiusIsSumPropertyString, new Boolean(radiusIsSum).toString());

        return properties;
    }

    public boolean getRadiusIsSum() {
        return radiusIsSum;
    }

    protected void setRadiusIsSum(boolean b) {
        radiusIsSum = b; //note that resizerPanel is enabled or disabled, via the radio buttons group
        refresh();
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        // Debug.message("ThematicDataDisplayLayer", "[Pie] resizer: " + resizer);
        mouseControlPointsListener.clearControlPoints();
        if (radiusIsSum) {
            resizer.setDataValueComputer(new DataSquareRootSumColumnsValueComputer(colorDataColumnNames));
            resizer.setIsFixedSize(false); // in order for it to take the DataValueComputer into account
        } else {
            resizer.setDataValueComputer(new DataSquareRootColumnValueComputer(getDataColumnName()));
        }

        resizer.setMinMaxValue(dsi);
        DataSumColumnsValueComputer colordc = new DataSumColumnsValueComputer(colorDataColumnNames);
        generalAngles = new double[colorDataColumnNames.length];
        int nbPie = 0;
        OMGraphicList omgl = new OMGraphicList();
        OMSizeableGraphicList omglDiscs = new OMSizeableGraphicList<OMDisc>();
        OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Discs of null siz            boolean offVis = offsetsOn ? offseter.isVisible() : false;
        boolean offLineVis = offsetsOn ? offseter.isLineVisible() : false;
        boolean offVis = offsetsOn ? offseter.isVisible() : false;
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            double[] values = colordc.getValues(data);
            if (values == null) {
                Debug.message("ThematicDataDisplayLayer", "no Pie because of a null value ");
                continue;
            }
            double sum = values[colorDataColumnNames.length];
            boolean zeroValues = true; //true if all values are equal to 0 or NaN
            for (int i = 0; i < colorDataColumnNames.length; i++) {
                values[i] *= 360 / sum;
                if(values[i] != 0.0 && !Double.valueOf(values[i]).isNaN()){                  
                    zeroValues = false;
                }
            }
            nbPie++;
            for (int i = 0; i < colorDataColumnNames.length; i++) {
                generalAngles[i] += values[i];
            }
            if(zeroValues){
                continue;//pie won't be drawn
            }
            /*for(double d : values)
                System.out.println(d);*/
            float radius = (float) (radiusIsSum
                    ? resizer.getSize(sum)//TODO: test if sqrt is needed
                    : resizer.getSize(data));
            OMGraphic disc;
            // 0.001 is to be adapted with LatLon Graphics
            if (radius < 0.001 && getShowNullAsPoint()) { // null sized pie => OMPoint
                disc = new OMPoint((float) centroid.getLat(), (float) centroid.getLon(), 1);
                ompl.add(disc);
            } else {
            int offX = 0;
            int offY = 0;
            String key = "";

            if (offsetsOn) {
                key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
            }
                disc = getZoomableSymbol()
                        ? new OMPie(this, key, (float) centroid.getLat(), (float) centroid.getLon(), offX, offY, radius,
                        colorDataColumnNames.length, values, colorScale,
                        (radiusIsSum ? String.valueOf(sum) : resizer.getString(data)), offVis, offLineVis)
                        : new OMPie(this, key, (float) centroid.getLat(), (float) centroid.getLon(),offX, offY, (int) (radius * 2),
                        colorDataColumnNames.length, values, colorScale,
                        (radiusIsSum ? String.valueOf(sum) : resizer.getString(data)), offVis, offLineVis);
                omglDiscs.add(disc);
                if(disc instanceof OMPie){
                    OMPie p = (OMPie)disc;
                    /*if(offsetsOn)
                        p.addOffsetableListener(offseter);*/
                    mouseControlPointsListener.addOMControlPoints(p.controlPoints);
                }
            }

            disc.setLinePaint(drawingAttributes.getLinePaint());
            disc.setStroke(drawingAttributes.getStroke());
        }

        for (int i = 0; i < generalAngles.length; i++) {
            generalAngles[i] = generalAngles[i] / nbPie;
        }

        if (nbPie > 0) {
            getLayer().thematicChanged(this);
        }
        omglDiscs.sort();
        initMinMaxValues(omglDiscs);
        omgl.add(ompl);
        omgl.add(omglDiscs);
        return omgl;
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {

        if (thematicGUI != null) {
            return thematicGUI;
        }
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        graphicsPanelToggleControlsDisplay(false, true, false, false, false);

//        final JRadioButton radiusIsSumRB = new JRadioButton(i18n.get(this, "radiusIsSumButton", "Proportional to the sum"), radiusIsSum);
//
//        radiusIsSumRB.addItemListener(new ItemListener() {
//
//            public void itemStateChanged(ItemEvent e) {
//
//                boolean b = ((AbstractButton) (e.getItemSelectable())).isSelected();
//                //radiusResizerPanel.setEnabled(b);
//                setRadiusIsSum(b);
//                // resizer updates via the Radio Button Group
//            }
//        });

        //resizer.getFixedSizeRBGroup().add(radiusIsSumRB);


       // ThematicConfPanel confSize = thematicGUI.getConfigPanels().get(thematicGUI.getConfigPanels().size() - 1);
        //JPanel p = new JPanel(new BorderLayout());
        //confSize.getContent().add(radiusIsSumRB);
        //p.add(confSize.getContent(), BorderLayout.CENTER);
        //confSize.getContent().setBorder(null);
        //JPanel radiusWrap = new JPanel(new BorderLayout());
        //radiusWrap.add(radiusIsSumRB, BorderLayout.WEST);
        //radiusIsSumRB.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));
       // radiusWrap.add(showNullAsPointCB, BorderLayout.SOUTH);
        //JPanel pShowNull = new JPanel(new BorderLayout());
        //pShowNull.add(showNullAsPointCB, BorderLayout.WEST);
        //pShowNull.setBorder(BorderFactory.createEmptyBorder(0, 50, 0, 0));
        //resizer.getProportionalGUI().add(pShowNull);
        resizer.setShowNullAsPoint(showNullAsPointCB);
        final JComboBox combo = resizer.getComboSize();
        combo.addItem("Proportionnelle à la somme");
        combo.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent arg0) {
                setRadiusIsSum(combo.getSelectedIndex() == combo.getItemCount() - 1);
            }
        });
        //showNullAsPointCB.setBorder(BorderFactory.createEmptyBorder(0, 50, 0, 0));
        //p.add(radiusWrap, BorderLayout.SOUTH);
        //radiusWrap.setBorder(BorderFactory.createEmptyBorder(8, 2, 2, 2));
        //confSize.setContent(p);
        //confSize.getContent().add(radiusIsSumRB);
        colorScalePanel = new AttributesColorScalePanel(colorScale, (AttributesNames) this);
        colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
        // colorScalePanel.refreshPanel();
        colorScalePanel.addColorScaleListener(
                this);

        ThematicConfPanel pieConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "classification", "Classification"), false);
        /*pieConfPan.setContent(new JScrollPane(colorScalePanel,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER));*/
        JPanel wrapClassif = new JPanel();
        wrapClassif.add(pieConfPan);
        pieConfPan.setContent(colorScalePanel);
        thematicGUI.addThematicConfigurationPanel(pieConfPan);
        thematicGUI.add(wrapClassif);

        return thematicGUI;
    }

    private boolean isAColorDataColumnNames(String columnName) {
        for (int i = 0; i <
                colorDataColumnNames.length; i++) {
            if (colorDataColumnNames[i].equals(columnName)) {
                return true;
            }

        }
        return false;
    }

    public void setColorDataColumnNames(String[] colorDataColumnNames) {
        
        this.colorDataColumnNames = colorDataColumnNames;
        badLegendCache = true;
    }

    /** gives colorDataColumnName[i]. @see AttributesNames interface */
    public String getName(
            int i) {
        return i >= 0 && i < colorDataColumnNames.length ? colorDataColumnNames[i] : "";
    }
    public final static int GAP = 10;
    private OMResizeableColorScaleLegend omrcsl;
//    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
//	OMResizeableGraphic discLegend = super.getLegend(r, ml);
//	OMResizeableGraphic pieLegend =
//	    new OMResizeableColorScaleLegend(r, ml, 0, 0, colorScale, colorDataColumnNames, "");
//	omrcsl = (OMResizeableColorScaleLegend) pieLegend;
//
//	if (discLegend == null)
//	    return pieLegend;
//	OMResizeableFrame legend =
//	    new OMResizeableFrame(r, ml, 0, 0,
//				  Math.max(discLegend.getWidth(), pieLegend.getWidth()),
//				  discLegend.getHeight() + pieLegend.getHeight() + GAP);
//	legend.showBorder(false);
//	legend.add(discLegend);
//	pieLegend.translate(0, discLegend.getHeight()+ GAP);
//	legend.add(pieLegend);
//	return legend;
//    }
    private OMResizeableFrame frame = null;
    //    private OMResizeablePie generalPie = null;
    private OMResizeableColorScaleLegend pieLegend = null;

    @Override
    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {

        OMResizeableDiscLegend discLegend = (OMResizeableDiscLegend) super.getLegend(r, ml);

        if (discLegend == null) {
            frame = new OMResizeablePieLegend(r, ml, 0, 0, resizer, generalAngles, drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0), colorScale, colorDataColumnNames, getFullName());
        } else {
            frame = new OMResizeablePieLegend(r, ml, 0, 0,resizer, generalAngles, drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0), colorScale, colorDataColumnNames, discLegend, getFullName());
        }
        frame.setVisuFrame(true);
        return frame;

    }
    protected AttributesColorScalePanel colorScalePanel;

    //when we change dsi, data change so... refresh list *and* colorscale
    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        if(colorScalePanel != null){
        colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
        //System.out.println("getNoColorDataColumnNames(dsi) " + getNoColorDataColumnNames(dsi));
        actualiseAttributesNames(dsi);
        colorScalePanel.refreshPanel(); // redraws all
        }
    }

    /** @see AttributesNames interface */
    public void setAttributesNames(String[] names) {
        setColorDataColumnNames(names);
    }

// remove names from AttributesNames that are no more attributes 
    private void actualiseAttributesNames(DataStorageIterator dsi) {
        ArrayList<String> names = new ArrayList<String>();
        // System.out.println(" colorDataColumnNames ");
        // for(int i = 0 ; i < colorDataColumnNames.length; i ++ ){
        // System.out.println(colorDataColumnNames[i]);
        // }
        for (String name : colorDataColumnNames) {
            for (String attribute : dsi.getDataColumnNames()) {
                if (name.equals(attribute)) {
                    names.add(name);
                    break;

                }


            }
        }
        setAttributesNames(names.toArray(new String[names.size()]));
    }

    protected ArrayList<String> getNoColorDataColumnNames(DataStorageIterator dsi) {
        ArrayList<String> names = new ArrayList<String>();
        for (String name : dsi.getDataColumnNames()) {
            if (!isAColorDataColumnNames(name)) {
                names.add(name);
            }

        }
        return names;
    }

    /*public void colorScaleChanged(ColorScalePanel2 colorScalePanel) {
    if (this.colorScalePanel == colorScalePanel) {
    refresh();
    }
    }*/
    public String[] getLegendColorDataColumnNames() {
        // ArrayList<OMResizeableGraphic> omrgl = pieLegend.getResizeableGraphics();
        try {
            //String[] colorDataNames = new String[colorDataColumnNames.length];
            if (badLegendCache) {
                badLegendCache = false;
                return colorDataColumnNames;
            }

            ArrayList<String> colorDataNames = new ArrayList<String>();
            //int i = 0;
            for (OMResizeableGraphic omrg : pieLegend.getResizeableGraphics()) {
                if (omrg instanceof OMResizeableText) {
                    OMResizeableText omrtext = (OMResizeableText) omrg;
                    if (omrtext.getText() != null && !omrtext.getText().equals("")) {
                        colorDataNames.add(omrtext.getText());
                    }
// colorDataNames[i] = omrtext.getText();
// System.out.println(i + " "+colorDataNames[i]);
		    /*if(colorDataNames[i] == null)
                    colorDataNames[i] = "";*/
//i++;
// throw new NullPointerException();

                }
            }
            return colorDataNames.toArray(new String[colorDataNames.size()]);
        } catch (NullPointerException exc) {
            return colorDataColumnNames;
        }

    }

    public void colorScaleChanged(ColorScalePanel colorScalePanel) {
        if (this.colorScalePanel == colorScalePanel) {
            refresh();
        }
    }
}
