// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import fr.unicaen.openmap.event.RefreshableGUI;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;

/**
 * Regular polygons visualization mode.
 * <p>Should inherit from Disc, but that make problems for the GUI.
 */
public class RegularPolygon extends /*Raw*/ OffsetableRaw implements ResizerListener {

    public static final String nbEdgesPropertyString = "nbEdges";
    public static final String starCoefPropertyString = "starCoef";
    public static final String startAnglePropertyString = "startAngle";
    protected SymbolResizer resizer = new SymbolResizer(new DataSquareRootColumnValueComputer("data"));

    public String getDataColumnName() {
        return resizer.getDataColumnName();
    }

    public void setDataColumnName(String name) {
        resizer.setDataColumnName(name);
        refresh();
    }
    //protected Offseter offset = new Offseter();
    protected int nbEdges = 4;
    protected int starCoef = 0;
    protected int startAngle = 45; // in degrees

    public RegularPolygon(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(RegularPolygon.class, "regularPolygon", " Regular Polygon"));
    }

    public RegularPolygon(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer.addResizerListener(this);
        //colors.setFillMode(true);
        //offseter.addOffseterListener(this);
        interactive = true;
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        resizer.setProperties(prefix, properties);
        setNbEdges(PropUtils.intFromProperties(properties, prefix + nbEdgesPropertyString, getNbEdges()));
        setStartAngle(PropUtils.intFromProperties(properties, prefix + startAnglePropertyString, getStartAngle()));
        setStarCoef(PropUtils.intFromProperties(properties, prefix + starCoefPropertyString, getStarCoef()));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        //Debug.message("ThematicDataDisplayLayer","RegularPolygon.getProperties = \nedges : "+getNbEdges()+"\nstartAngle : "+getStartAngle()+"\nstarcoeff"+getStarCoef());
        properties = resizer.getProperties(prefix, properties);
        properties.put(prefix + nbEdgesPropertyString, Integer.toString(getNbEdges()));
        properties.put(prefix + startAnglePropertyString, Integer.toString(getStartAngle()));
        properties.put(prefix + starCoefPropertyString, Integer.toString(getStarCoef()));
        return properties;
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {

        resizer.setMinMaxValue(dsi);

        OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMRegularPolygon>();
        OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Symbols of null sizes)
        float startAngleInRadians = getStartAngleInRadians();
        boolean offVis = offsetsOn ? offseter.isVisible() : false;
        boolean offLineVis = offsetsOn ? offseter.isLineVisible() : false;
        // Debug.message("ThematicDataDisplayLayer", "RegularPolygon.generateGraphics nbEdges="+nbEdges+" starCoef"+starCoef+" startAngle="+startAngle);
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }
            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            double size = resizer.getSize(data);
            float lat = (float) centroid.getLat();
            float lon = (float) centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";
            if (offsetsOn) {
                key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
            }
            OMGraphic p;
            // 0.001 is to be adapted with LatLon Graphics
            if (size < 0.001) { // null sized disc => OMPoint
                p = new OMPoint(lat, lon);
                ompl.add(p);
            } else {
                p = new OMRegularPolygon(this, key, lat, lon,
                        nbEdges, (float) size,
                        offX, offY,
                        startAngleInRadians, starCoef,
                        resizer.getString(data), offVis, offLineVis);
                omgl.add(p);
                if (p instanceof OMRegularPolygon) {
                    OMRegularPolygon r = (OMRegularPolygon) p;
                    /*if(offsetsOn)
                        r.addOffsetableListener(offseter);*/
                    mouseControlPointsListener.addOMControlPoints(r.controlPoints);
                }
            }
            p.setLinePaint(drawingAttributes.getLinePaint());
            p.setFillPaint(drawingAttributes.getFillPaint());
            p.setStroke(drawingAttributes.getStroke());
        }
        omgl.sort();
        initMinMedianMaxValuesAndSizes(omgl);  // used for the legend
        if (ompl.size() > 0) // add null sized polygons
        {
            omgl.add(ompl);
        }
        return omgl;
    }
    protected JPanel regpolysPanel; // protected attribute in order for derived class to remove this component.
    protected JPanel dataPanel; // protected attribute in order for derived class to keep this one!

    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        regpolysPanel = new JPanel();
        regpolysPanel.setLayout(new BoxLayout(regpolysPanel, BoxLayout.Y_AXIS));
        regpolysPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "regularPolygonTitle", "Regular Polygon")));
        regpolysPanel.add(resizer.getGUI(dsi));
        regpolysPanel.add(getRegularPolygonGUI());
        thematicGUI.add(regpolysPanel);
        return thematicGUI;
    }
    private RegularPolygonPanel regularPolygonPanel = null;

    public RegularPolygonPanel getRegularPolygonGUI() {
        if (regularPolygonPanel == null) {
            regularPolygonPanel = new RegularPolygonPanel(this, true);
        }
        return regularPolygonPanel;
    }

    public void setStartAngle(int a) {
        startAngle = a;
        refresh();
    }

    public void setStarCoef(int c) {
        starCoef = c;
        refresh();
    }

    public void setNbEdges(int n) {
        nbEdges = n;
        refresh();
    }

    public void setElemRadius(int r) { // in order for MultiRegularPolygons to overload this method
        refresh();
    }

    public int getStartAngle() {
        return startAngle;
    }

    public float getStartAngleInRadians() {
        return (float) (startAngle * Math.PI / 180.0);
    }

    public int getStarCoef() {
        return starCoef;
    }

    public int getNbEdges() {
        return nbEdges;
    }

    public int getElemRadius() { // in order for MultiRegularPolygons to overload this method
        return 20;
    }

    // overrided in MultiRegularPolygons 
    public int getGapPercentage() {
        return 0;
    }
    private MinMeanMaxValuesAndSizes minMedianMaxValuesAndSizes = null;

    protected void initMinMedianMaxValuesAndSizes(OMGraphicList omgl) {
        //minMedianMaxValuesAndSizes = new MinMedianMaxValuesAndSizes(omgl);
    }

    //    private OMResizeableRegularPolygonLegend omrrpl;
    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        if (minMedianMaxValuesAndSizes == null) {
            return null;
        }
        /*
        //One object for all => pbs with many legends : no multi positions
        if (omrrpl == null) {
        omrrpl =
        new OMResizeableRegularPolygonLegend(r, ml, 0, 0, minMedianMaxValuesAndSizes,
        nbEdges, getStartAngleInRadians(), starCoef,
        getDataColumnName(),
        colors.getLineColor(), colors.getFillColor());
        } else {
        omrrpl.init(minMedianMaxValuesAndSizes,
        nbEdges, getStartAngleInRadians(), starCoef,
        getDataColumnName(),
        colors.getLineColor(), colors.getFillColor());
        }

        return omrrpl;
         */

        //Each Legend can setProperties() to have others positions
        //PB -- if RegularPolygon has changed : impossible to refresh Legend without save/reload (because Legend is not refreshed)
	/*return 	new OMResizeableRegularPolygonLegend(r, ml, 0, 0, minMedianMaxValuesAndSizes,
        nbEdges, getStartAngleInRadians(), starCoef,
        getDataColumnName(),
        colors.getLineColor(), colors.getFillColor());*/
        return null;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        ((RefreshableGUI) resizer).refreshGUI(dsi);
    }

    public void resizerChanged(Resizer resizer) {
        if (this.resizer == resizer) {
            refresh();
        }
    }
}
