// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certicf
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.*;
import javax.swing.JComponent;


import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.util.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.event.*;
import fr.unicaen.openmap.event.RefreshableGUI;
import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPalette;

/** Base class for thematic visualisation. Just defines a name and visibility for this pseudo layer */
public abstract class Thematic implements DataStorageListener, RefreshableGUI {

    public static final String namePropertyString = "name";
    public static final String VisiblePropertyString = "visible";
    protected int guiOrientation = ThematicPalette.AXIS;
    // protected boolean newOrientation = false;
    //Thematic could implement PropertyConsumer ? Maybe if, one day, we want to use getPropertyInfo()...
    private String prefix = " ";

    public String getPropertyPrefix() {
        return prefix;
    }

    public void setPropertyPrefix(String prefix) {
        this.prefix = prefix;
    }
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    public static I18n i18n = new BasicI18n();
    protected ThematicGUI thematicGUI;
    protected boolean interactive = false;
    protected boolean visible = true;
    protected OMGraphicList interactiveGraphics = new OMGraphicList();
    private String name = "";
    private String typeName = "Thematic";
    private ThematicDataDisplayLayer layer;

    public Thematic(ThematicDataDisplayLayer layer, String name, String typeName) {
        this.layer = layer;
        this.name = name;
        this.typeName = typeName;
    }

    public String getTypeName() {
        return typeName;
    }

    public String getName() {
        return name;
    }

    public String getFullName() {
        return layer.getName() + "-" + getName();
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
        getLayer().thematicChanged(this);

    }

    public boolean isVisible() {
        return visible;
    }

    public ThematicDataDisplayLayer getLayer() {
        return layer;
    }

    public boolean isInteractiveAndBadCache() {
        return (getBadCache() && interactive);
    }

    public void setProperties(String prefix, Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        setPropertyPrefix(prefix);

        name = properties.getProperty(prefix + namePropertyString, getName());
        visible = Util.isTrue(properties.getProperty(prefix + VisiblePropertyString, "true"));
    }

    public Properties getProperties(String prefix, Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        setPropertyPrefix(prefix);

        properties.put(prefix + ThematicDataDisplayLayer.visuClassNamePropertyString, getClass().getName());
        properties.put(prefix + namePropertyString, getName());
        properties.put(prefix + VisiblePropertyString, new Boolean(visible).toString());
        return properties;
    }

    /** responsible for the construction of the thematic visu graphics */
    public abstract OMGraphicList generateGraphics(DataStorageIterator dsi);
    //*or ?*    public abstract OMGraphicList generateGraphics(DataStorageIterator dsi, Projection proj);
    /** By default we never need to regenerate Graphics, because we can use cached OMGraphics */
    //protected boolean needsToRegenerateGraphics() { return cachedOMGraphicList == null; }
    protected OMGraphicList cachedOMGraphicList = null;
    //badCache is true if cachedList is null or out-of-date
    protected boolean badCache = true;

    protected OMGraphicList getList() {
        return cachedOMGraphicList;
    }

    protected OMGraphicList cachedGenerateGraphics(DataStorageIterator dsi) {
        if (getBadCache()) {
            Debug.message("ThematicDataDisplayLayer", this + " omgraphiclist regenerated");
            cachedOMGraphicList = generateGraphics(dsi);
            //badCache = false; // done in each thematic visu
        } else {
            Debug.message("ThematicDataDisplayLayer", this + " omgraphiclist cached");
        }
        return cachedOMGraphicList;
    }

    /**
     * The result graphic must be placed at (0, 0).
     * It is the responsability of Legend visu to translate it in the right place.
     */
    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        return null;
    }

    /**
     * This method must update the GUI component(s) when underlying data has changed. Remember
     * to call super if needed. Be carefull whend dealing also with dataStorageEvent(DataStorage ds).
     */
    public abstract void refreshGUI(DataStorageIterator dsi);

    /**
     * Default implementation for a DataStorage event. If one overrides this function
     * remember you must not call super if you call it also in refreshGUI(DataStorageIterator dsi).
     */
    public void dataStorageEvent(DataStorage ds) {
        cachedOMGraphicList = null;
        refreshGUI(new DataStorageIteratorAdapter(ds));
    }

    public abstract JComponent getGUI(DataStorageIterator dsi);

    public String toString() {
        return name + ":" + typeName;
    }

    public MapMouseListener getMapMouseListener() {
        return null;
    }

    //in test: willb e useless if all JTextFields have a default value
    //not null (and != "")
    public abstract boolean isReady();
    //public abstract String getMissingInfos();
    //current proj for graphics
    private Projection proj;

    public Projection getProjection() {
        return proj;
    }
    //public void setProjection(Projection proj){ this.proj = proj;}

    //Thematics are able to provide a projected OMGraphicList, if any
    public OMGraphicList getGraphics(DataStorageIterator dsi, Projection proj) {
        Debug.message("ThematicDataDisplayLayer", this + " getGraphics()");
        if (this.proj != proj) {
            cachedOMGraphicList = null;
            setBadCache(true);
        }
        this.proj = proj;
        OMGraphicList omgl = cachedGenerateGraphics(dsi);
        if (getBadCache()) {
            Debug.message("ThematicDataDisplayLayer", this + " graphics projected");
            omgl.generate(proj);
            setBadCache(false);
        }
        return omgl;
    }

    public OMGraphicList getInteractiveGraphics() {
        return interactiveGraphics;
    }

    public void addInteractiveGraphic(OMGraphic omg) {
        interactiveGraphics.add(omg);
        omg.setVisible(false);
        getLayer().refreshGraphicsAndGUI();
    }

    public void removeInteractiveGraphic(OMGraphic omg) {
        interactiveGraphics.remove(omg);
    }

    public void removeGraphic(OMGraphic g){
        cachedOMGraphicList.remove(g);
    }

    public void clearInteractiveGraphics() {
        interactiveGraphics.clear();
    }

    public void refresh() {
        //setBadCache(true);
    }

    public void setBadCache(boolean b) {
//        try {
//            throw new Exception();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
        Debug.message("ThematicDataDisplayLayer", this + " setBadCache " + b);
        badCache = b;
    }

    public boolean getBadCache() {
        // in order to be sure to refresh effectively the graphics.
        if (getLayer().getDataStorageIterator().getDataStorage() instanceof MonoValuedFilterDataStorage) {
            return true;
        }
        return badCache;
    }

    public boolean useSharedGraphics() {
        return false;
    }

    public void guiOrientationChanged(int axis) {
    }
}
