// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;

/**
 * Computes quantiles i.e. classes with the same cardinal
 * <p>problem with this implementation exists in case of many equals values...
 */
public class Quantile extends ClassificationOfDouble {

    public Quantile(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }
    
    protected void init() {
	if (data == null){
            return;
        }
	Collections.sort(data);
	int N = data.size();
	if (N == 0 || nbClasses == 0) {
	    System.err.println("[Classification] *** Error *** empty data");
	    return;
	}

	breaks = new double[nbClasses+1];
	breaks[0] = data.get(0); // first break is the min value;
	breaks[nbClasses] = data.get(data.size()-1); // last break is the max value

	int totalCounts = 0; // effective count of values
	counts = new int[nbClasses];

	int currentClass = 1; // class index
	int currentClassSize = 0;
	int classSize =  (N - totalCounts) / (nbClasses - currentClass + 1);

	//	System.err.println("class size " + currentClass + " = " + classSize);
       		   
	for (int dd = 0; dd < data.size(); dd++) {
	    double d = data.get(dd);
	    currentClassSize++;
	    //	    System.err.println(dd + " " + currentClassSize + " " + d);

	    // in case of equal values, we extend the cardinal of the current class
	    if (currentClassSize >= classSize && (dd+1 < data.size() ? d != data.get(dd+1) : true)) {
		breaks[currentClass] = // the mean value of d and next d
		    dd+1 < data.size() ? (d+data.get(dd+1))/2.0 : d;
		counts[currentClass-1] = currentClassSize;
		totalCounts += currentClassSize;
	        // System.err.println("breaks["+currentClass+"]="+breaks[currentClass]+ " ("+currentClassSize+")");
		currentClass++;
		currentClassSize = 0;
		if (currentClass < nbClasses+1)
		    classSize =  (N - totalCounts) / (nbClasses - currentClass + 1);
		else
		    classSize = N - totalCounts;

		// System.err.println("class size " + currentClass + " = "  + classSize);
	    }
	}
	if (currentClassSize != 0) 
	    counts[nbClasses-1] = currentClassSize;

        //System.out.println("BREAKS++++++++++++ = " + breaks.toString());
    }

}
