// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.util.propertyEditor;

import com.bbn.openmap.PropertyConsumer;
import com.bbn.openmap.gui.WindowSupport;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.PropertyEditor;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;

/**
 *
 * @author jchauvea
 */
public class Inspector extends com.bbn.openmap.util.propertyEditor.Inspector {

    /**
     * Inspect and configure a PropertyConsumer object. Main method of this
     * class. The argument PropertyConsumer is inspected through the
     * ProperyConsumer interface, the properties are displayed to be edited.
     */
    @Override
    public void inspectPropertyConsumer(PropertyConsumer propertyConsumer) {
        String prefix = propertyConsumer.getPropertyPrefix();

        // construct GUI
        if (windowSupport != null) {
            windowSupport.killWindow();
            windowSupport = null;
        }

        JComponent comp = createPropertyGUI(propertyConsumer);
        windowSupport = new WindowSupport(comp, prefix);
        windowSupport.setMaxSize(-1, 500);
        windowSupport.displayInWindow();
    }

    /**
     * Creates a JComponent with the properties to be changed. This component is
     * suitable for inclusion into a GUI. Don't use this method directly! Use
     * the createPropertyGUI(PropertyConsumer) instead. You will get a
     * NullPointerException if you use this method without setting the
     * PropertyConsumer in the Inspector.
     *
     * @param prefix
     *            the property prefix for the property consumer. Received from
     *            the PropertyConsumer.getPropertyPrefix() method. Properties
     *            that start with this prefix will have the prefix removed from
     *            the display, so the GUI will only show the actual property
     *            name.
     * @param props
     *            the properties received from the
     *            PropertyConsumer.getProperties() method.
     * @param info
     *            the properties received from the
     *            PropertyConsumer.getPropertyInfo() method, containing
     *            descriptions and any specific PropertyEditors that should be
     *            used for a particular property named in the
     *            PropertyConsumer.getProperties() properties.
     * @return JComponent, a panel holding the interface to set the properties.
     */
    @Override
    @SuppressWarnings("unchecked")
    public JComponent createEmbeddedPropertyGUI(String prefix,
            Properties props, Properties info) {

        if (Debug.debugging("inspectordetail")) {
            Debug.output("Inspector creating GUI for " + prefix + "\nPROPERTIES " + props + "\nPROP INFO " + info);
        }

        // collect the info needed...
        Collection keySet = props.keySet();
        String propertyList = info.getProperty(PropertyConsumer.initPropertiesProperty);
        Vector sortedKeys;

        if (propertyList != null) {
            Vector propertiesToShow = PropUtils.parseSpacedMarkers(propertyList);
            for (int i = 0; i < propertiesToShow.size(); i++) {
                propertiesToShow.set(i, prefix + "." + propertiesToShow.get(i));
            }
            sortedKeys = propertiesToShow;
        } else {
            // otherwise, show them all, in alphabetical order
            sortedKeys = sortKeys(keySet);
        }

        int num = sortedKeys.size();
        editors = new Hashtable(num);

        JPanel component = new JPanel();
        component.setLayout(new BorderLayout());

        JPanel propertyPanel = new JPanel();

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 10, 2, 10);
        propertyPanel.setLayout(gridbag);

        int i = 0;
        for (Iterator it = sortedKeys.iterator(); it.hasNext();) { // iterate
            // properties
            String prop = (String) it.next();

            String marker = prop;
            if (prefix != null && prop.startsWith(prefix)) {
                marker = prop.substring(prefix.length() + 1);
            }

            if (marker.startsWith(".")) {
                marker = marker.substring(1);
            }

            String editorMarker = marker + PropertyConsumer.ScopedEditorProperty;
            String editorClass = info.getProperty(editorMarker);
            if (editorClass == null) {
                editorClass = defaultEditorClass;
            }

            // instantiate PropertyEditor
            Class propertyEditorClass = null;
            PropertyEditor editor = null;
            try {
                propertyEditorClass = Class.forName(editorClass);
                editor = (PropertyEditor) propertyEditorClass.newInstance();
                if (editor instanceof PropertyConsumer) {
                    ((PropertyConsumer) editor).setProperties(marker, info);
                }
                editors.put(prop, editor);
            } catch (Exception e) {
                e.printStackTrace();
                editorClass = null;
            }

            Component editorFace = null;
            if (editor != null && editor.supportsCustomEditor()) {
                editorFace = editor.getCustomEditor();
            } else {
                editorFace = new JLabel("Does not support custom editor");
            }

            if (editor != null) {
                Object propVal = props.get(prop);
                if (Debug.debugging("inspector")) {
                    Debug.output("Inspector loading " + prop + "(" + propVal + ")");
                }
                editor.setValue(propVal);
            }

            // Customized labels for each property, instead of the
            // abbreviated nature of the true property names.
            String labelMarker = marker + PropertyConsumer.LabelEditorProperty;
            String labelText = info.getProperty(labelMarker);
            if (labelText == null) {
                labelText = marker;
            }

            JLabel label = new JLabel("<html><b>"+labelText + ":</b></html>");
            label.setHorizontalAlignment(SwingConstants.RIGHT);

            c.gridx = 0;
            c.gridy = i++;
            c.weightx = 0;
            c.fill = GridBagConstraints.NONE;
            c.anchor = GridBagConstraints.EAST;

            gridbag.setConstraints(label, c);
            propertyPanel.add(label);

            c.gridx = 1;
            c.anchor = GridBagConstraints.WEST;
            c.fill = GridBagConstraints.HORIZONTAL;
            c.weightx = 1f;

            gridbag.setConstraints(editorFace, c);
            propertyPanel.add(editorFace);

            String toolTip = (String) info.get(marker);
            label.setToolTipText(toolTip == null ? "No further information available."
                    : toolTip);
        }

        // create the palette's scroll pane
        JScrollPane scrollPane = new JScrollPane(propertyPanel,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.setBorder(null);
        // scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
        scrollPane.setAlignmentY(Component.TOP_ALIGNMENT);
        component.add(scrollPane, BorderLayout.CENTER);
        return component;
    }
}
